@@ -1,3 +1,4 @@
+/* -*- mode: C; c-file-style: "linux" -*- */
 /* GdkPixbuf library - Main loading interface.
  *
  * Copyright (C) 1999 The Free Software Foundation
@@ -155,12 +156,32 @@ pixbuf_check_ico (guchar *buffer, int size)
 	    buffer [1] != 0x0 ||
 	    ((buffer [2] != 0x1)&&(buffer[2]!=0x2)) ||
 	    buffer [3] != 0x0 ||
+	    buffer [4] == 0x0 || 
 	    buffer [5] != 0x0 )
 		return FALSE;
 
 	return TRUE;
 }
 
+static gboolean
+pixbuf_check_ani (guchar *buffer, int size)
+{
+        if (size < 12)
+                return FALSE;
+
+	if (buffer [0] != 'R' ||
+	    buffer [1] != 'I' ||
+	    buffer [2] != 'F' ||
+	    buffer [3] != 'F' ||
+	    buffer [8] != 'A' ||
+	    buffer [9] != 'C' ||
+	    buffer[10] != 'O' ||
+	    buffer[11] != 'N')
+		return FALSE;
+
+        return TRUE;
+}
+
 
 static gboolean
 pixbuf_check_bmp (guchar *buffer, int size)
@@ -203,9 +224,10 @@ pixbuf_check_xbm (guchar *buffer, int size)
 	    && buffer [7] == ' ')
 		return TRUE;
 
+	/* Note that this requires xpm to be checked before xbm. */
 	if (buffer [0] == '/'
 	    && buffer [1] != '*')
-	return TRUE;
+		return TRUE;
 
 	return FALSE;
 }
@@ -239,10 +261,9 @@ static GdkPixbufModule file_formats [] = {
 	{ "ras",  pixbuf_check_sunras, NULL,  NULL, NULL, NULL, NULL, NULL, NULL },
 	{ "bmp",  pixbuf_check_bmp, NULL,  NULL, NULL, NULL, NULL, NULL, NULL },
 	{ "xbm",  pixbuf_check_xbm, NULL,  NULL, NULL, NULL, NULL, NULL, NULL },
-	{ "tga", pixbuf_check_tga, NULL, NULL, NULL, NULL, NULL, NULL, NULL },
-	/* Moved at the bottom, because it causes false positives against many
-	   of my TGA files. */
 	{ "ico",  pixbuf_check_ico, NULL,  NULL, NULL, NULL, NULL, NULL, NULL },	
+	{ "ani",  pixbuf_check_ani, NULL,  NULL, NULL, NULL, NULL, NULL, NULL },	
+	{ "tga", pixbuf_check_tga, NULL, NULL, NULL, NULL, NULL, NULL, NULL },
 	{ "wbmp", pixbuf_check_wbmp, NULL, NULL, NULL, NULL, NULL, NULL, NULL },
 	{ NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL }
 };
@@ -393,6 +414,7 @@ m_fill_vtable (bmp);
 m_fill_vtable (wbmp);
 m_fill_vtable (gif);
 m_fill_vtable (ico);
+m_fill_vtable (ani);
 m_fill_vtable (jpeg);
 m_fill_vtable (pnm);
 m_fill_vtable (ras);
@@ -442,6 +464,12 @@ _gdk_pixbuf_load_module (GdkPixbufModule *image_module,
 	}
 #endif
 
+#ifdef INCLUDE_ani
+	else if (strcmp (image_module->module_name, "ani") == 0){
+                fill_vtable = mname (ani, fill_vtable);
+	}
+#endif
+
 #ifdef INCLUDE_jpeg
 	else if (strcmp (image_module->module_name, "jpeg") == 0){
                 fill_vtable = mname (jpeg, fill_vtable);
@@ -551,6 +579,55 @@ _gdk_pixbuf_get_module (guchar *buffer, guint size,
 	return NULL;
 }
 
+
+static void
+prepared_notify (GdkPixbuf *pixbuf, 
+		 GdkPixbufAnimation *anim, 
+		 gpointer user_data)
+{
+	if (pixbuf != NULL)
+		g_object_ref (pixbuf);
+	*((GdkPixbuf **)user_data) = pixbuf;
+}
+
+GdkPixbuf *
+_gdk_pixbuf_generic_image_load (GdkPixbufModule *module,
+				FILE *f,
+				GError **error)
+{
+	guchar buffer[4096];
+	size_t length;
+	GdkPixbuf *pixbuf = NULL;
+	gpointer context;
+
+	if (module->load != NULL)
+		return (* module->load) (f, error);
+	
+	context = module->begin_load (NULL, prepared_notify, NULL, &pixbuf, error);
+	
+	if (!context)
+		return NULL;
+	
+	while (!feof (f)) {
+		length = fread (buffer, 1, sizeof (buffer), f);
+		if (length > 0)
+			if (!module->load_increment (context, buffer, length, error)) {
+				module->stop_load (context, NULL);
+				if (pixbuf != NULL)
+					g_object_unref (pixbuf);
+				return NULL;
+			}
+	}
+
+	if (!module->stop_load (context, error)) {
+		if (pixbuf != NULL)
+			g_object_unref (pixbuf);
+		return NULL;
+	}
+	
+	return pixbuf;
+}
+
 /**
  * gdk_pixbuf_new_from_file:
  * @filename: Name of file to load.
@@ -611,26 +688,15 @@ gdk_pixbuf_new_from_file (const char *filename,
                         return NULL;
                 }
 
-	if (image_module->load == NULL) {
-                g_set_error (error,
-                             GDK_PIXBUF_ERROR,
-                             GDK_PIXBUF_ERROR_UNSUPPORTED_OPERATION,
-                             _("Don't know how to load the image in file '%s'"),
-                             filename);
-                
-		fclose (f);
-		return NULL;
-	}
-
 	fseek (f, 0, SEEK_SET);
-	pixbuf = (* image_module->load) (f, error);
+	pixbuf = _gdk_pixbuf_generic_image_load (image_module, f, error);
 	fclose (f);
 
         if (pixbuf == NULL && error != NULL && *error == NULL) {
                 /* I don't trust these crufty longjmp()'ing image libs
                  * to maintain proper error invariants, and I don't
                  * want user code to segfault as a result. We need to maintain
-                 * the invariant that error gets set if NULL is returned.
+                 * the invariastable/gdk-pixbuf/nt that error gets set if NULL is returned.
                  */
                 
                 g_warning ("Bug! gdk-pixbuf loader '%s' didn't set an error on failure.", image_module->module_name);
@@ -788,9 +854,7 @@ gdk_pixbuf_real_save (GdkPixbuf     *pixbuf,
  * "quality" parameter; its value should be in the range [0,100]. 
  * Text chunks can be attached to PNG images by specifying parameters of
  * the form "tEXt::key", where key is an ASCII string of length 1-79.
- * The values are UTF-8 encoded strings. Note however that PNG text
- * chunks are stored in ISO-8859-1 encoding, so you can only set texts
- * that can be represented in this encoding.
+ * The values are UTF-8 encoded strings. 
  *
  * Return value: whether an error was set
  **/
