@@ -134,11 +134,10 @@ struct headerpair {
 /* Data needed for the "state" during decompression */
 struct bmp_compression_state {
 	gint phase;
-	gint RunCount;
-
-	guchar *linebuff;
-	gint linebuffsize;	/* these two counts in nibbles */
-	gint linebuffdone;
+	gint run;
+	gint count;
+	gint x, y;
+	guchar *p;
 };
 
 /* Progressive loading */
@@ -167,7 +166,7 @@ struct bmp_progressive_state {
 				   8  = 8 bpp colormapped
 				   1  = 1 bit bitonal
 				 */
-	gint Compressed;
+	guint Compressed;
 	struct bmp_compression_state compr;
 
 
@@ -182,7 +181,8 @@ struct bmp_progressive_state {
 };
 
 static gpointer
-gdk_pixbuf__bmp_image_begin_load(ModulePreparedNotifyFunc prepared_func,
+gdk_pixbuf__bmp_image_begin_load(ModuleSizeFunc size_func,
+                                 ModulePreparedNotifyFunc prepared_func,
 				 ModuleUpdatedNotifyFunc updated_func,
                                  gpointer user_data,
                                  GError **error);
@@ -194,45 +194,6 @@ static gboolean gdk_pixbuf__bmp_image_load_increment(gpointer data,
                                                      GError **error);
 
 
-
-/* Shared library entry point --> This should be removed when
-   generic_image_load enters gdk-pixbuf-io. */
-static GdkPixbuf *gdk_pixbuf__bmp_image_load(FILE * f, GError **error)
-{
-	guchar membuf[4096];
-	size_t length;
-	struct bmp_progressive_state *State;
-
-	GdkPixbuf *pb;
-
-	State =
-	    gdk_pixbuf__bmp_image_begin_load(NULL, NULL, NULL,
-                                             error);
-
-        if (State == NULL)
-          return NULL;
-
-	while (feof(f) == 0) {
-		length = fread(membuf, 1, sizeof (membuf), f);
-		if (length > 0)
-                  if (!gdk_pixbuf__bmp_image_load_increment(State,
-                                                            membuf,
-                                                            length,
-                                                            error)) {
-                          gdk_pixbuf__bmp_image_stop_load (State, NULL);
-                          return NULL;
-                  }
-
-	}
-	if (State->pixbuf != NULL)
-		g_object_ref(State->pixbuf);
-
-	pb = State->pixbuf;
-
-	gdk_pixbuf__bmp_image_stop_load(State, NULL);
-	return pb;
-}
-
 /* Picks up a 32-bit little-endian integer starting at the specified location.
  * Does it by hand instead of dereferencing a simple (gint *) cast due to
  * alignment constraints many platforms.
@@ -253,16 +214,17 @@ lsb_16 (guchar *src)
 static gboolean grow_buffer (struct bmp_progressive_state *State,
                              GError **error)
 {
-		State->buff = g_try_realloc (State->buff, State->BufferSize);
-		if (State->buff == NULL) {
-			g_set_error (error,
-				     GDK_PIXBUF_ERROR,
-                                     GDK_PIXBUF_ERROR_INSUFFICIENT_MEMORY,
-                                     _("Not enough memory to load bitmap image"));
-			State->read_state = READ_STATE_ERROR;
-			return FALSE;
-		}
-		return TRUE;
+  guchar *tmp = g_try_realloc (State->buff, State->BufferSize);
+  if (!tmp) {
+    g_set_error (error,
+		 GDK_PIXBUF_ERROR,
+		 GDK_PIXBUF_ERROR_INSUFFICIENT_MEMORY,
+		 _("Not enough memory to load bitmap image"));
+    State->read_state = READ_STATE_ERROR;
+    return FALSE;
+  }
+  State->buff = tmp;
+  return TRUE;
 }
 
 static gboolean DecodeHeader(unsigned char *BFH, unsigned char *BIH,
@@ -356,7 +318,9 @@ static gboolean DecodeHeader(unsigned char *BFH, unsigned char *BIH,
 		State->LineWidth = (State->LineWidth / 4) * 4 + 4;
 
 	if (State->pixbuf == NULL) {
-		if (State->Type == 32)
+		if (State->Type == 32 || 
+		    State->Compressed == BI_RLE4 || 
+		    State->Compressed == BI_RLE8)
 			State->pixbuf =
 			    gdk_pixbuf_new(GDK_COLORSPACE_RGB, TRUE, 8,
 					   (gint) State->Header.width,
@@ -381,21 +345,18 @@ static gboolean DecodeHeader(unsigned char *BFH, unsigned char *BIH,
 			(*State->prepared_func) (State->pixbuf, NULL, State->user_data);
 
 	}
-
-	if (!(State->Compressed == BI_RGB || State->Compressed == BI_BITFIELDS)) {
-		State->compr.linebuffdone = 0;
-		State->compr.linebuffsize = State->Header.width;
-		if (State->Type == 8)
-			State->compr.linebuffsize *= 2;
-		State->compr.linebuff = g_malloc ((State->compr.linebuffsize + 1) / 2);
+	
+	/* make all pixels initially transparent */
+	if (State->Compressed == BI_RLE4 || State->Compressed == BI_RLE8) {
+		memset (State->pixbuf->pixels, 0, State->pixbuf->rowstride * State->Header.height);
+		State->compr.p = State->pixbuf->pixels 
+			+ State->pixbuf->rowstride * (State->Header.height- 1);
 	}
 
 	State->BufferDone = 0;
 	if (State->Type <= 8) {
-		int biClrUsed = lsb_32 (&BIH[32]);
-		int n_colors = (biClrUsed ? biClrUsed : (1 << State->Header.depth)); 
 		State->read_state = READ_STATE_PALETTE;
-		State->BufferSize = lsb_32 (&BFH[10]) - 14 - State->Header.size;
+		State->BufferSize = lsb_32 (&BFH[10]) - 14 - State->Header.size; 
 	} else if (State->Compressed == BI_RGB) {
 		State->read_state = READ_STATE_DATA;
 		State->BufferSize = State->LineWidth;
@@ -418,8 +379,8 @@ static gboolean DecodeHeader(unsigned char *BFH, unsigned char *BIH,
 }
 
 static gboolean DecodeColormap (guchar *buff,
-			    struct bmp_progressive_state *State,
-			    GError **error)
+				struct bmp_progressive_state *State,
+				GError **error)
 {
 	gint i;
 
@@ -447,7 +408,7 @@ static gboolean DecodeColormap (guchar *buff,
 		State->BufferSize = 2;
 	else
 		State->BufferSize = State->LineWidth;
-
+	
 	if (!grow_buffer (State, error))
 		return FALSE;
 
@@ -514,7 +475,8 @@ decode_bitmasks (guchar *buf,
  */
 
 static gpointer
-gdk_pixbuf__bmp_image_begin_load(ModulePreparedNotifyFunc prepared_func,
+gdk_pixbuf__bmp_image_begin_load(ModuleSizeFunc size_func,
+                                 ModulePreparedNotifyFunc prepared_func,
 				 ModuleUpdatedNotifyFunc updated_func,
                                  gpointer user_data,
                                  GError **error)
@@ -565,9 +527,6 @@ static gboolean gdk_pixbuf__bmp_image_stop_load(gpointer data, GError **error)
 
 	g_return_val_if_fail(context != NULL, TRUE);
 
-	if (context->compr.linebuff != NULL)
-		g_free(context->compr.linebuff);
-
 	if (context->Colormap != NULL)
 		g_free(context->Colormap);
 
@@ -843,144 +802,168 @@ static void OneLine(struct bmp_progressive_state *context)
 	if (context->updated_func != NULL) {
 		(*context->updated_func) (context->pixbuf,
 					  0,
-					  context->Lines,
+					  (context->Header.Negative ?
+					   (context->Lines - 1) :
+					   (context->Header.height - context->Lines)),
 					  context->Header.width,
-					  2,
+					  1,
 					  context->user_data);
 
 	}
 }
 
-static gboolean
+#define NEUTRAL       0
+#define ENCODED       1
+#define ESCAPE        2   
+#define DELTA_X       3
+#define DELTA_Y       4
+#define ABSOLUTE      5
+#define SKIP          6
+
+#define END_OF_LINE   0
+#define END_OF_BITMAP 1
+#define DELTA         2
+
+static gboolean 
 DoCompressed(struct bmp_progressive_state *context, GError **error)
 {
-	gint count, pos;
-	switch (context->compr.phase) {
-	case 0:		/* Neutral state */
-		if (context->buff[0] != 0) {	/* run count */
-			context->compr.RunCount = context->buff[0];
-			if (context->Type == 8)
-				context->compr.RunCount *= 2;
-			while (context->compr.RunCount > 0) {
-				if (context->compr.linebuffdone & 1) {
-					guchar *ptr = context->compr.linebuff +
-					    context->compr.linebuffdone / 2;
+	gint i, j;
+	gint y;
+	guchar c;
+	gint idx;
 
-					*ptr = (*ptr & 0xF0) | (context->buff[1] >> 4);
-					context->buff[1] = (context->buff[1] << 4) |
-							   (context->buff[1] >> 4);
-					context->compr.linebuffdone++;
-					context->compr.RunCount--;
-				}
+	if (context->compr.y >= context->Header.height)
+		return TRUE;
 
-				if (context->compr.RunCount) {
-					count = context->compr.linebuffsize -
-					    context->compr.linebuffdone;
-					if (count > context->compr.RunCount)
-						count = context->compr.RunCount;
+	y = context->compr.y;
 
-					memset (context->compr.linebuff +
-						context->compr.linebuffdone / 2,
-						context->buff[1],
-						(count + 1) / 2);
-					context->compr.RunCount -= count;
-					context->compr.linebuffdone += count;
-				}
-				if (context->compr.linebuffdone == context->compr.linebuffsize) {
-					guchar *tmp = context->buff;
-					context->buff = context->compr.linebuff;
-					OneLine (context);
-					context->buff = tmp;
+ 	for (i = 0; i < context->BufferSize; i++) {
+		c = context->buff[i];
+		switch (context->compr.phase) {
+		    case NEUTRAL:
+			    if (c) {
+				    context->compr.run = c;
+				    context->compr.phase = ENCODED;
+			    }
+			    else
+				    context->compr.phase = ESCAPE;
+			    break;
+		    case ENCODED:
+			    for (j = 0; j < context->compr.run; j++) {
+				    if (context->Compressed == BI_RLE8)
+					    idx = c;
+				    else if (j & 1) 
+					    idx = c & 0x0f;
+				    else 
+					    idx = (c >> 4) & 0x0f;
+				    if (context->compr.x < context->Header.width) {
+					    *context->compr.p++ = context->Colormap[idx][2];
+					    *context->compr.p++ = context->Colormap[idx][1];
+					    *context->compr.p++ = context->Colormap[idx][0];
+					    *context->compr.p++ = 0xff;
+					    context->compr.x++;    
+				    }
+			    }
+			    context->compr.phase = NEUTRAL;
+			    break;
+		    case ESCAPE:
+			    switch (c) {
+				case END_OF_LINE:
+					context->compr.x = 0;
+					context->compr.y++;
+					context->compr.p = context->pixbuf->pixels 
+						+ (context->pixbuf->rowstride * (context->Header.height - context->compr.y - 1))
+						+ (4 * context->compr.x);
+					context->compr.phase = NEUTRAL;
+					break;
+				case END_OF_BITMAP:
+					context->compr.x = 0;
+					context->compr.y = context->Header.height;
+					context->compr.phase = NEUTRAL;
+					break;
+				case DELTA:
+					context->compr.phase = DELTA_X;
+					break;
+				default:
+					context->compr.run = c;
+					context->compr.count = 0;
+					context->compr.phase = ABSOLUTE;
+					break;
+			    }
+			    break;
+		    case DELTA_X:
+			    context->compr.x += c;
+			    context->compr.phase = DELTA_Y;
+			    break;
+		    case DELTA_Y:
+			    context->compr.y += c;
+			    context->compr.p = context->pixbuf->pixels 
+				    + (context->pixbuf->rowstride * (context->Header.height - context->compr.y - 1))
+				    + (4 * context->compr.x);
+			    context->compr.phase = NEUTRAL;
+			    break;
+		    case ABSOLUTE:
+			    if (context->Compressed == BI_RLE8) {
+				    idx = c;
+				    if (context->compr.x < context->Header.width) {
+					    *context->compr.p++ = context->Colormap[idx][2];
+					    *context->compr.p++ = context->Colormap[idx][1];
+					    *context->compr.p++ = context->Colormap[idx][0];
+					    *context->compr.p++ = 0xff;
+					    context->compr.x++;    
+				    }
+				    context->compr.count++;
 
-					if (context->compr.linebuffdone & 1)
-						context->buff[1] = (context->buff[1] << 4) |
-								   (context->buff[1] >> 4);
-					context->compr.linebuffdone = 0;
-				}
-			}
-		} else {	/* Escape */
-			if (context->buff[1] == 0) {	/* End of line */
-				if (context->compr.linebuffdone) {
-					guchar *tmp = context->buff;
-					context->buff = context->compr.linebuff;
-					OneLine (context);
-					context->buff = tmp;
+				    if (context->compr.count == context->compr.run) {
+					    if (context->compr.run & 1)
+						    context->compr.phase = SKIP;
+					    else
+						    context->compr.phase = NEUTRAL;
+				    }
+			    }
+			    else {
+				    for (j = 0; j < 2; j++) {
+					    if (context->compr.count & 1)
+						    idx = c & 0x0f;
+					    else 
+						    idx = (c >> 4) & 0x0f;
+					    if (context->compr.x < context->Header.width) {
+						    *context->compr.p++ = context->Colormap[idx][2];
+						    *context->compr.p++ = context->Colormap[idx][1];
+						    *context->compr.p++ = context->Colormap[idx][0];
+						    *context->compr.p++ = 0xff;
+						    context->compr.x++;    
+					    }
+					    context->compr.count++;
 
-					context->compr.linebuffdone = 0;
-				}
-			} else if (context->buff[1] == 1) {	/* End of image */
-				if (context->compr.linebuffdone) {
-					guchar *tmp = context->buff;
-					context->buff = context->compr.linebuff;
-					OneLine (context);
-					context->buff = tmp;
-				}
-
-				context->compr.phase = 2;
-			} else if (context->buff[1] == 2)	/* Cursor displacement */
-				;	/* not implemented */
-			else {
-				context->compr.phase = 1;
-				context->compr.RunCount = context->buff[1];
-				if (context->Type == 8)
-					context->compr.RunCount *= 2;
-				context->BufferSize = (context->compr.RunCount + 3) / 4 * 2;
-				if (!grow_buffer (context, error)) 
-					return FALSE;
-			}
+					    if (context->compr.count == context->compr.run) {
+						    if ((context->compr.run & 3) == 1
+							|| (context->compr.run & 3) == 2) 
+							    context->compr.phase = SKIP;
+						    else
+							    context->compr.phase = NEUTRAL;
+						    break;
+					    }
+				    }
+			    }
+			    break;
+		    case SKIP:
+			    context->compr.phase = NEUTRAL;
+			    break;
 		}
-		context->BufferDone = 0;
-		break;
-	case 1:
-		pos = 0;
-		while (pos < context->compr.RunCount) {
-			count = context->compr.linebuffsize - context->compr.linebuffdone;
-			if (count > context->compr.RunCount)
-				count = context->compr.RunCount;
-
-			if ((context->compr.linebuffdone & 1) || (pos & 1)) {
-				gint i, newval;
-				guchar *ptr;
-				for (i = 0; i < count; i++) {
-					ptr = context->compr.linebuff + (i +
-					      context->compr.linebuffdone) / 2;
-					newval = *(context->buff + (pos + i) / 2) & (0xf0 >> (((pos + i) % 2) * 4));
-					if (((pos + i) % 2) ^ ((context->compr.linebuffdone + i) % 2)) {
-						if ((pos + i) % 2)
-							newval <<= 4;
-						else
-							newval >>= 4;
-					}
-					*ptr = (*ptr & (0xf << (((i + context->compr.linebuffdone) % 2) * 4))) | newval;
-				}
-			} else {
-				memmove (context->compr.linebuff +
-					 context->compr.linebuffdone / 2,
-					 context->buff + pos / 2,
-					 (count + 1) / 2);
-			}
-			pos += count;
-			context->compr.linebuffdone += count;
-			if (context->compr.linebuffdone == context->compr.linebuffsize) {
-				guchar *tmp = context->buff;
-				context->buff = context->compr.linebuff;
-				OneLine (context);
-				context->buff = tmp;
-
-				context->compr.linebuffdone = 0;
-			}
-		}
-		context->compr.phase = 0;
-		context->BufferSize = 2;
-		if (!grow_buffer (context, error)) 
-			return FALSE;
-		context->BufferDone = 0;
-		break;
-	case 2:
-		context->BufferDone = 0;
-		break;
 	}
-	
+	if (context->updated_func != NULL) {
+		if (context->compr.y > y)
+			(*context->updated_func) (context->pixbuf,
+						  0,
+						  y,
+						  context->Header.width,
+						  context->compr.y - y,
+						  context->user_data);
+
+	}
+
+	context->BufferDone = 0;
 	return TRUE;
 }
 
@@ -1064,7 +1047,6 @@ gdk_pixbuf__bmp_image_load_increment(gpointer data,
 void
 gdk_pixbuf__bmp_fill_vtable (GdkPixbufModule *module)
 {
-  module->load = gdk_pixbuf__bmp_image_load;
   module->begin_load = gdk_pixbuf__bmp_image_begin_load;
   module->stop_load = gdk_pixbuf__bmp_image_stop_load;
   module->load_increment = gdk_pixbuf__bmp_image_load_increment;
