@@ -1,3 +1,4 @@
+/* -*- mode: C; c-file-style: "linux" -*- */
 /* GdkPixbuf library - JPEG image loader
  *
  * Copyright (C) 1999 Michael Zucchi
@@ -26,21 +27,6 @@
  */
 
 
-/*
-  Progressive file loading notes (11/03/1999) <drmike@redhat.com>...
-
-  These are issues I know of and will be dealing with shortly:
-
-    -  Currently does not handle progressive jpegs - this
-       requires a change in the way image_load_increment () calls
-       libjpeg. Progressive jpegs are rarer but I will add this
-       support asap.
-
-    - error handling is not as good as it should be
-
- */
-
-
 #include <config.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -65,7 +51,7 @@ typedef struct {
 
 	JOCTET buffer[JPEG_PROG_BUF_SIZE];              /* start of buffer */
 	long  skip_next;              /* number of bytes to skip next read */
-
+	
 } my_source_mgr;
 
 typedef my_source_mgr * my_src_ptr;
@@ -79,6 +65,7 @@ struct error_handler_data {
 
 /* progressive loader context */
 typedef struct {
+        ModuleSizeFunc           size_func;
 	ModuleUpdatedNotifyFunc  updated_func;
 	ModulePreparedNotifyFunc prepared_func;
 	gpointer                 user_data;
@@ -95,7 +82,8 @@ typedef struct {
 } JpegProgContext;
 
 static GdkPixbuf *gdk_pixbuf__jpeg_image_load (FILE *f, GError **error);
-static gpointer gdk_pixbuf__jpeg_image_begin_load (ModulePreparedNotifyFunc func, 
+static gpointer gdk_pixbuf__jpeg_image_begin_load (ModuleSizeFunc           func0,
+                                                   ModulePreparedNotifyFunc func1, 
                                                    ModuleUpdatedNotifyFunc func2,
                                                    gpointer user_data,
                                                    GError **error);
@@ -150,11 +138,12 @@ explode_gray_into_buf (struct jpeg_decompress_struct *cinfo,
 
 	g_return_if_fail (cinfo != NULL);
 	g_return_if_fail (cinfo->output_components == 1);
+	g_return_if_fail (cinfo->out_color_space == JCS_GRAYSCALE);
 
 	/* Expand grey->colour.  Expand from the end of the
 	 * memory down, so we can use the same buffer.
 	 */
-	w = cinfo->image_width;
+	w = cinfo->output_width;
 	for (i = cinfo->rec_outbuf_height - 1; i >= 0; i--) {
 		guchar *from, *to;
 		
@@ -170,6 +159,43 @@ explode_gray_into_buf (struct jpeg_decompress_struct *cinfo,
 	}
 }
 
+
+static void
+convert_cmyk_to_rgb (struct jpeg_decompress_struct *cinfo,
+		     guchar **lines) 
+{
+	gint i, j;
+
+	g_return_if_fail (cinfo != NULL);
+	g_return_if_fail (cinfo->output_components == 4);
+	g_return_if_fail (cinfo->out_color_space == JCS_CMYK);
+
+	for (i = cinfo->rec_outbuf_height - 1; i >= 0; i--) {
+		guchar *p;
+		
+		p = lines[i];
+		for (j = 0; j < cinfo->output_width; j++) {
+			int c, m, y, k;
+			c = p[0];
+			m = p[1];
+			y = p[2];
+			k = p[3];
+			if (cinfo->saw_Adobe_marker) {
+				p[0] = k*c / 255;
+				p[1] = k*m / 255;
+				p[2] = k*y / 255;
+			}
+			else {
+				p[0] = (255 - k)*(255 - c) / 255;
+				p[1] = (255 - k)*(255 - m) / 255;
+				p[2] = (255 - k)*(255 - y) / 255;
+			}
+			p[3] = 255;
+			p += 4;
+		}
+	}
+}
+
 typedef struct {
   struct jpeg_source_mgr pub;	/* public fields */
 
@@ -234,6 +260,20 @@ stdio_term_source (j_decompress_ptr cinfo)
 {
 }
 
+static gchar *
+colorspace_name (const J_COLOR_SPACE jpeg_color_space) 
+{
+	switch (jpeg_color_space) {
+	    case JCS_UNKNOWN: return "UNKNOWN"; 
+	    case JCS_GRAYSCALE: return "GRAYSCALE"; 
+	    case JCS_RGB: return "RGB"; 
+	    case JCS_YCbCr: return "YCbCr"; 
+	    case JCS_CMYK: return "CMYK"; 
+	    case JCS_YCCK: return "YCCK";
+	    default: return "invalid";
+	}
+}
+
 /* Shared library entry point */
 static GdkPixbuf *
 gdk_pixbuf__jpeg_image_load (FILE *f, GError **error)
@@ -292,8 +332,10 @@ gdk_pixbuf__jpeg_image_load (FILE *f, GError **error)
 	cinfo.do_fancy_upsampling = FALSE;
 	cinfo.do_block_smoothing = FALSE;
 
-	pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, FALSE, 8, cinfo.output_width, cinfo.output_height);
- 
+	pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, 
+				 cinfo.out_color_components == 4 ? TRUE : FALSE, 
+				 8, cinfo.output_width, cinfo.output_height);
+	      
 	if (!pixbuf) {
 		jpeg_destroy_decompress (&cinfo);
 
@@ -313,7 +355,6 @@ gdk_pixbuf__jpeg_image_load (FILE *f, GError **error)
 	dptr = pixbuf->pixels;
 
 	/* decompress all the lines, a few at a time */
-
 	while (cinfo.output_scanline < cinfo.output_height) {
 		lptr = lines;
 		for (i = 0; i < cinfo.rec_outbuf_height; i++) {
@@ -323,8 +364,29 @@ gdk_pixbuf__jpeg_image_load (FILE *f, GError **error)
 
 		jpeg_read_scanlines (&cinfo, lines, cinfo.rec_outbuf_height);
 
-		if (cinfo.output_components == 1)
-			explode_gray_into_buf (&cinfo, lines);
+		switch (cinfo.out_color_space) {
+		    case JCS_GRAYSCALE:
+		      explode_gray_into_buf (&cinfo, lines);
+		      break;
+		    case JCS_RGB:
+		      /* do nothing */
+		      break;
+		    case JCS_CMYK:
+		      convert_cmyk_to_rgb (&cinfo, lines);
+		      break;
+		    default:
+		      g_object_unref (pixbuf);
+		      if (error && *error == NULL) {
+                        g_set_error (error,
+                                     GDK_PIXBUF_ERROR,
+				     GDK_PIXBUF_ERROR_UNKNOWN_TYPE,
+				     _("Unsupported JPEG color space (%s)"),
+				     colorspace_name (cinfo.out_color_space)); 
+		      }
+                
+		      jpeg_destroy_decompress (&cinfo);
+		      return NULL;
+		}
 	}
 
 	jpeg_finish_decompress (&cinfo);
@@ -388,8 +450,9 @@ skip_input_data (j_decompress_ptr cinfo, long num_bytes)
  */
 
 gpointer
-gdk_pixbuf__jpeg_image_begin_load (ModulePreparedNotifyFunc prepared_func, 
-				   ModuleUpdatedNotifyFunc  updated_func,
+gdk_pixbuf__jpeg_image_begin_load (ModuleSizeFunc size_func,
+				   ModulePreparedNotifyFunc prepared_func, 
+				   ModuleUpdatedNotifyFunc updated_func,
 				   gpointer user_data,
                                    GError **error)
 {
@@ -397,6 +460,7 @@ gdk_pixbuf__jpeg_image_begin_load (ModulePreparedNotifyFunc prepared_func,
 	my_source_mgr   *src;
 
 	context = g_new0 (JpegProgContext, 1);
+	context->size_func = size_func;
 	context->prepared_func = prepared_func;
 	context->updated_func  = updated_func;
 	context->user_data = user_data;
@@ -411,11 +475,11 @@ gdk_pixbuf__jpeg_image_begin_load (ModulePreparedNotifyFunc prepared_func,
 
 	context->cinfo.src = (struct jpeg_source_mgr *) g_try_malloc (sizeof (my_source_mgr));
 	if (!context->cinfo.src) {
-	  g_set_error (error,
-		       GDK_PIXBUF_ERROR,
-		       GDK_PIXBUF_ERROR_INSUFFICIENT_MEMORY,
-		       _("Couldn't allocate memory for loading JPEG file"));
-	  return NULL;
+		g_set_error (error,
+			     GDK_PIXBUF_ERROR,
+			     GDK_PIXBUF_ERROR_INSUFFICIENT_MEMORY,
+			     _("Couldn't allocate memory for loading JPEG file"));
+		return NULL;
 	}
 	memset (context->cinfo.src, 0, sizeof (my_source_mgr));
        
@@ -451,14 +515,14 @@ gdk_pixbuf__jpeg_image_stop_load (gpointer data, GError **error)
 	JpegProgContext *context = (JpegProgContext *) data;
 
 	g_return_val_if_fail (context != NULL, TRUE);
-
+	
         /* FIXME this thing needs to report errors if
          * we have unused image data
          */
         
 	if (context->pixbuf)
 		g_object_unref (context->pixbuf);
-
+	
 	/* if we have an error? */
 	if (sigsetjmp (context->jerr.setjmp_buffer, 1)) {
 		jpeg_destroy_decompress (&context->cinfo);
@@ -501,6 +565,7 @@ gdk_pixbuf__jpeg_image_load_increment (gpointer data,
 	guint       spinguard;
 	gboolean    first;
 	const guchar *bufhd;
+	gint        width, height;
 
 	g_return_val_if_fail (context != NULL, FALSE);
 	g_return_val_if_fail (buf != NULL, FALSE);
@@ -511,10 +576,6 @@ gdk_pixbuf__jpeg_image_load_increment (gpointer data,
 
         context->jerr.error = error;
         
-	/* XXXXXXX (drmike) - loop(s) below need to be recoded now I
-         *                    have a grasp of what the flow needs to be!
-         */
-
 	/* check for fatal error */
 	if (sigsetjmp (context->jerr.setjmp_buffer, 1)) {
 		return FALSE;
@@ -578,20 +639,32 @@ gdk_pixbuf__jpeg_image_load_increment (gpointer data,
 		/* try to load jpeg header */
 		if (!context->got_header) {
 			int rc;
-
+			
 			rc = jpeg_read_header (cinfo, TRUE);
 			context->src_initialized = TRUE;
-
+			
 			if (rc == JPEG_SUSPENDED)
 				continue;
-
+			
 			context->got_header = TRUE;
-
-			context->pixbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB, 
-							 FALSE,
-							 8, 
-							 cinfo->image_width,
-							 cinfo->image_height);
+			
+			width = cinfo->image_width;
+			height = cinfo->image_height;
+			(* context->size_func) (&width, &height, context->user_data);
+			for (cinfo->scale_denom = 2; cinfo->scale_denom <= 8; cinfo->scale_denom *= 2) {
+				jpeg_calc_output_dimensions (cinfo);
+				if (cinfo->output_width < width || cinfo->output_height < height) {
+					cinfo->scale_denom /= 2;
+					break;
+				}
+			}
+			jpeg_calc_output_dimensions (cinfo);
+			
+			context->pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, 
+							  cinfo->output_components == 4 ? TRUE : FALSE,
+							  8, 
+							  cinfo->output_width,
+							  cinfo->output_height);
 
 			if (context->pixbuf == NULL) {
                                 g_set_error (error,
@@ -600,24 +673,24 @@ gdk_pixbuf__jpeg_image_load_increment (gpointer data,
                                              _("Couldn't allocate memory for loading JPEG file"));
                                 return FALSE;
 			}
-
+			
 			/* Use pixbuf buffer to store decompressed data */
 			context->dptr = context->pixbuf->pixels;
-
+			
 			/* Notify the client that we are ready to go */
 			(* context->prepared_func) (context->pixbuf,
                                                     NULL,
 						    context->user_data);
-
+			
 		} else if (!context->did_prescan) {
-			int rc;
+			int rc;			
 			
 			/* start decompression */
 			cinfo->buffered_image = TRUE;
 			rc = jpeg_start_decompress (cinfo);
 			cinfo->do_fancy_upsampling = FALSE;
 			cinfo->do_block_smoothing = FALSE;
-			
+
 			if (rc == JPEG_SUSPENDED)
 				continue;
 
@@ -653,10 +726,28 @@ gdk_pixbuf__jpeg_image_load_increment (gpointer data,
 					if (nlines == 0)
 						break;
 
-				        /* handle gray */
-					if (cinfo->output_components == 1)
-						explode_gray_into_buf (cinfo, lines);
-					
+					switch (cinfo->out_color_space) {
+					    case JCS_GRAYSCALE:
+						    explode_gray_into_buf (cinfo, lines);
+						    break;
+					    case JCS_RGB:
+						    /* do nothing */
+						    break;
+					    case JCS_CMYK:
+						    convert_cmyk_to_rgb (cinfo, lines);
+						    break;
+					    default:
+						    if (error && *error == NULL) {
+							    g_set_error (error,
+									 GDK_PIXBUF_ERROR,
+									 GDK_PIXBUF_ERROR_UNKNOWN_TYPE,
+									 _("Unsupported JPEG color space (%s)"),
+									 colorspace_name (cinfo->out_color_space)); 
+						    }
+						    
+						    return FALSE;
+					}
+
 					context->dptr += nlines * context->pixbuf->rowstride;
 					
 				        /* send updated signal */
