@@ -199,26 +199,24 @@ png_simple_warning_callback(png_structp png_save_ptr,
          */
 }
 
-/* Destroy notification function for the pixbuf */
-static void
-free_buffer (guchar *pixels, gpointer data)
-{
-	g_free (pixels);
-}
-
 static gboolean
 png_text_to_pixbuf_option (png_text   text_ptr,
                            gchar    **key,
                            gchar    **value)
 {
-        *value = g_convert (text_ptr.text, -1, 
-                            "UTF-8", "ISO-8859-1", 
-                            NULL, NULL, NULL);
+        if (text_ptr.text_length > 0) {
+                *value = g_convert (text_ptr.text, -1, 
+                                    "UTF-8", "ISO-8859-1", 
+                                    NULL, NULL, NULL);
+        }
+        else {
+                *value = g_strdup (text_ptr.text);
+        }
         if (*value) {
                 *key = g_strconcat ("tEXt::", text_ptr.key, NULL);
                 return TRUE;
         } else {
-                g_warning ("Couldn't convert tEXt chunk value to UTF-8.");
+                g_warning ("Couldn't convert text chunk value to UTF-8.");
                 *key = NULL;
                 return FALSE;
         }
@@ -240,16 +238,16 @@ png_free_callback (png_structp o, png_voidp x)
 static GdkPixbuf *
 gdk_pixbuf__png_image_load (FILE *f, GError **error)
 {
-        GdkPixbuf *pixbuf;
+        GdkPixbuf * volatile pixbuf = NULL;
 	png_structp png_ptr;
 	png_infop info_ptr;
         png_textp text_ptr;
-	gint i, ctype, bpp;
+	gint i, ctype;
 	png_uint_32 w, h;
 	png_bytepp volatile rows = NULL;
-	guchar * volatile pixels = NULL;
         gint    num_texts;
-        gchar **options = NULL;
+        gchar *key;
+        gchar *value;
 
 #ifdef PNG_USER_MEM_SUPPORTED
 	png_ptr = png_create_read_struct_2 (PNG_LIBPNG_VER_STRING,
@@ -278,8 +276,8 @@ gdk_pixbuf__png_image_load (FILE *f, GError **error)
 	    	if (rows)
 		  	g_free (rows);
 
-		if (pixels)
-			g_free (pixels);
+		if (pixbuf)
+			g_object_unref (pixbuf);
 
 		png_destroy_read_struct (&png_ptr, &info_ptr, NULL);
 		return NULL;
@@ -293,16 +291,9 @@ gdk_pixbuf__png_image_load (FILE *f, GError **error)
                 return NULL;
         }
         
-	if (ctype & PNG_COLOR_MASK_ALPHA)
-		bpp = 4;
-	else
-		bpp = 3;
+        pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, ctype & PNG_COLOR_MASK_ALPHA, 8, w, h);
 
-	pixels = g_try_malloc (w * h * bpp);
-	if (!pixels) {
-                /* Check error NULL, normally this would be broken,
-                 * but libpng makes me want to code defensively.
-                 */
+	if (!pixbuf) {
                 if (error && *error == NULL) {
                         g_set_error (error,
                                      GDK_PIXBUF_ERROR,
@@ -310,6 +301,7 @@ gdk_pixbuf__png_image_load (FILE *f, GError **error)
                                      _("Insufficient memory to load PNG file"));
                 }
                 
+
 		png_destroy_read_struct (&png_ptr, &info_ptr, NULL);
 		return NULL;
 	}
@@ -317,44 +309,22 @@ gdk_pixbuf__png_image_load (FILE *f, GError **error)
 	rows = g_new (png_bytep, h);
 
 	for (i = 0; i < h; i++)
-		rows[i] = pixels + i * w * bpp;
+		rows[i] = pixbuf->pixels + i * pixbuf->rowstride;
 
 	png_read_image (png_ptr, rows);
         png_read_end (png_ptr, info_ptr);
 
         if (png_get_text (png_ptr, info_ptr, &text_ptr, &num_texts)) {
-                options = g_new (gchar *, num_texts * 2);
                 for (i = 0; i < num_texts; i++) {
-                        png_text_to_pixbuf_option (text_ptr[i], 
-                                                   options + 2*i, 
-                                                   options + 2*i + 1);
+                        png_text_to_pixbuf_option (text_ptr[i], &key, &value);
+                        gdk_pixbuf_set_option (pixbuf, key, value);
+                        g_free (key);
+                        g_free (value);
                 }
         }
-	png_destroy_read_struct (&png_ptr, &info_ptr, NULL);
+
 	g_free (rows);
-
-	if (ctype & PNG_COLOR_MASK_ALPHA)
-		pixbuf = gdk_pixbuf_new_from_data (pixels, GDK_COLORSPACE_RGB, TRUE, 8,
-                                                   w, h, w * 4,
-                                                   free_buffer, NULL);
-	else
-		pixbuf = gdk_pixbuf_new_from_data (pixels, GDK_COLORSPACE_RGB, FALSE, 8,
-                                                   w, h, w * 3,
-                                                   free_buffer, NULL);
-
-        if (options) {
-                for (i = 0; i < num_texts; i++) {
-                        if (pixbuf) {
-                                if (!gdk_pixbuf_set_option (pixbuf, 
-                                                            options[2*i], 
-                                                            options[2*i+1]))
-                                        g_warning ("Got multiple tEXt chunks for the same key.");
-                        }
-                        g_free (options[2*i]);
-                        g_free (options[2*i+1]);
-                }
-                g_free (options);
-        }
+	png_destroy_read_struct (&png_ptr, &info_ptr, NULL);
 
         return pixbuf;
 }
@@ -416,7 +386,8 @@ struct _LoadContext {
 };
 
 static gpointer
-gdk_pixbuf__png_image_begin_load (ModulePreparedNotifyFunc prepare_func,
+gdk_pixbuf__png_image_begin_load (ModuleSizeFunc size_func,
+                                  ModulePreparedNotifyFunc prepare_func,
 				  ModuleUpdatedNotifyFunc update_func,
 				  gpointer user_data,
                                   GError **error)
@@ -637,7 +608,7 @@ png_info_callback   (png_structp png_read_ptr,
                 return;
         }
 
-        /* Extract tEXt chunks and attach them as pixbuf options */
+        /* Extract text chunks and attach them as pixbuf options */
         
         if (png_get_text (png_read_ptr, png_info_ptr, &png_text_ptr, &num_texts)) {
                 for (i = 0; i < num_texts; i++) {
@@ -788,8 +759,8 @@ gdk_pixbuf__png_image_save (FILE          *f,
 
                for (kiter = keys; *kiter; kiter++) {
                        if (strncmp (*kiter, "tEXt::", 6) != 0) {
-                                g_warning ("Bad option name '%s' passed to PNG saver", *kiter);
-                                return FALSE;
+                               g_warning ("Bad option name '%s' passed to PNG saver", *kiter);
+                               return FALSE;
                        }
                        key = *kiter + 6;
                        len = strlen (key);
@@ -797,7 +768,7 @@ gdk_pixbuf__png_image_save (FILE          *f,
                                g_set_error (error,
                                             GDK_PIXBUF_ERROR,
                                             GDK_PIXBUF_ERROR_BAD_OPTION,
-                                            _("Keys for PNG tEXt chunks must have at least 1 and at most 79 characters."));
+                                            _("Keys for PNG text chunks must have at least 1 and at most 79 characters."));
                                return FALSE;
                        }
                        for (i = 0; i < len; i++) {
@@ -805,7 +776,7 @@ gdk_pixbuf__png_image_save (FILE          *f,
                                        g_set_error (error,
                                                     GDK_PIXBUF_ERROR,
                                                     GDK_PIXBUF_ERROR_BAD_OPTION,
-                                                    _("Keys for PNG tEXt chunks must be ASCII characters."));
+                                                    _("Keys for PNG text chunks must be ASCII characters."));
                                        return FALSE;
                                }
                        }
@@ -822,11 +793,23 @@ gdk_pixbuf__png_image_save (FILE          *f,
                                                      "ISO-8859-1", "UTF-8", 
                                                      NULL, &text_ptr[i].text_length, 
                                                      NULL);
+
+#ifdef PNG_iTXt_SUPPORTED 
+                       if (!text_ptr[i].text) {
+                               text_ptr[i].compression = PNG_ITXT_COMPRESSION_NONE;
+                               text_ptr[i].text = g_strdup (values[i]);
+                               text_ptr[i].text_length = 0;
+                               text_ptr[i].itxt_length = strlen (text_ptr[i].text);
+                               text_ptr[i].lang = NULL;
+                               text_ptr[i].lang_key = NULL;
+                       }
+#endif
+
                        if (!text_ptr[i].text) {
                                g_set_error (error,
                                             GDK_PIXBUF_ERROR,
                                             GDK_PIXBUF_ERROR_BAD_OPTION,
-                                            _("Value for PNG tEXt chunk can not be converted to ISO-8859-1 encoding."));
+                                            _("Value for PNG text chunk %s can not be converted to ISO-8859-1 encoding."), keys[i] + 6);
                                num_keys = i;
                                for (i = 0; i < num_keys; i++)
                                        g_free (text_ptr[i].text);
@@ -894,7 +877,7 @@ gdk_pixbuf__png_image_save (FILE          *f,
        png_write_end (png_ptr, info_ptr);
 
 cleanup:
-       png_destroy_write_struct (&png_ptr, (png_infopp) NULL);
+       png_destroy_write_struct (&png_ptr, &info_ptr);
 
        if (num_keys > 0) {
                for (i = 0; i < num_keys; i++)
