@@ -244,6 +244,7 @@ read_bitmap_file_data (FILE *fstream,
 				*ptr=value;
 			}
 		}
+		break;
 	}
 
 	if (!bits)
@@ -294,6 +295,14 @@ gdk_pixbuf__xbm_image_load_real (FILE *f, XBMData *context, GError **error)
                 return NULL;
         }
         
+	if (x_hot != -1 && y_hot != -1) {
+		gchar hot[10];
+		g_snprintf (hot, 10, "%d", x_hot);
+		gdk_pixbuf_set_option (pixbuf, "x_hot", hot);
+		g_snprintf (hot, 10, "%d", y_hot);
+		gdk_pixbuf_set_option (pixbuf, "y_hot", hot);
+	}
+
 	pixels = gdk_pixbuf_get_pixels (pixbuf);
 	row_stride = gdk_pixbuf_get_rowstride (pixbuf);
 
@@ -323,6 +332,7 @@ gdk_pixbuf__xbm_image_load_real (FILE *f, XBMData *context, GError **error)
 		}
 		pixels += row_stride;
 	}
+	g_free (data);
 
 	if (context) {
 		(* context->update_func) (pixbuf, 0, 0, w, h, context->user_data);
@@ -351,7 +361,8 @@ gdk_pixbuf__xbm_image_load (FILE *f, GError **error)
  */
 
 static gpointer
-gdk_pixbuf__xbm_image_begin_load (ModulePreparedNotifyFunc prepare_func,
+gdk_pixbuf__xbm_image_begin_load (ModuleSizeFunc size_func,
+                                  ModulePreparedNotifyFunc prepare_func,
 				  ModuleUpdatedNotifyFunc update_func,
 				  gpointer user_data,
 				  GError **error)
