@@ -1203,27 +1203,21 @@ mem_buffer (enum buf_op op, gpointer handle)
 	return NULL;
 }
 
-/* Destroy notification function for the pixbuf */
-static void
-free_buffer (guchar *pixels, gpointer data)
-{
-	g_free (pixels);
-}
-
 /* This function does all the work. */
 static GdkPixbuf *
 pixbuf_create_from_xpm (const gchar * (*get_buf) (enum buf_op op, gpointer handle), gpointer handle,
                         GError **error)
 {
-	gint w, h, n_col, cpp;
-	gint cnt, xcnt, ycnt, wbytes, n, ns;
+	gint w, h, n_col, cpp, x_hot, y_hot, items;
+	gint cnt, xcnt, ycnt, wbytes, n;
 	gint is_trans = FALSE;
 	const gchar *buffer;
         gchar *name_buf;
 	gchar pixel_str[32];
 	GHashTable *color_hash;
 	XPMColor *colors, *color, *fallbackcolor;
-	guchar *pixels, *pixtmp;
+	guchar *pixtmp;
+	GdkPixbuf *pixbuf;
 
 	fallbackcolor = NULL;
 
@@ -1235,7 +1229,7 @@ pixbuf_create_from_xpm (const gchar * (*get_buf) (enum buf_op op, gpointer handl
                              _("No XPM header found"));
 		return NULL;
 	}
-	sscanf (buffer, "%d %d %d %d", &w, &h, &n_col, &cpp);
+	items = sscanf (buffer, "%d %d %d %d %d %d", &w, &h, &n_col, &cpp, &x_hot, &y_hot);
 	if (w <= 0) {
                 g_set_error (error,
                              GDK_PIXBUF_ERROR,
@@ -1314,12 +1308,9 @@ pixbuf_create_from_xpm (const gchar * (*get_buf) (enum buf_op op, gpointer handl
 			fallbackcolor = color;
 	}
 
-	if (is_trans)
-		pixels = g_try_malloc (w * h * 4);
-	else
-		pixels = g_try_malloc (w * h * 3);
+	pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, is_trans, 8, w, h);
 
-	if (!pixels) {
+	if (!pixbuf) {
                 g_set_error (error,
                              GDK_PIXBUF_ERROR,
                              GDK_PIXBUF_ERROR_INSUFFICIENT_MEMORY,
@@ -1331,9 +1322,10 @@ pixbuf_create_from_xpm (const gchar * (*get_buf) (enum buf_op op, gpointer handl
 	}
 
 	wbytes = w * cpp;
-	pixtmp = pixels;
 
 	for (ycnt = 0; ycnt < h; ycnt++) {
+		pixtmp = pixbuf->pixels + ycnt * pixbuf->rowstride;
+
 		buffer = (*get_buf) (op_body, handle);
 		if ((!buffer) || (strlen (buffer) < wbytes))
 			continue;
@@ -1341,7 +1333,6 @@ pixbuf_create_from_xpm (const gchar * (*get_buf) (enum buf_op op, gpointer handl
 		for (n = 0, cnt = 0, xcnt = 0; n < wbytes; n += cpp, xcnt++) {
 			strncpy (pixel_str, &buffer[n], cpp);
 			pixel_str[cpp] = 0;
-			ns = 0;
 
 			color = g_hash_table_lookup (color_hash, pixel_str);
 
@@ -1364,9 +1355,16 @@ pixbuf_create_from_xpm (const gchar * (*get_buf) (enum buf_op op, gpointer handl
 	g_free (colors);
 	g_free (name_buf);
 
-	return gdk_pixbuf_new_from_data (pixels, GDK_COLORSPACE_RGB, is_trans, 8,
-					 w, h, is_trans ? (w * 4) : (w * 3),
-					 free_buffer, NULL);
+	if (items == 6) {
+		gchar hot[10];
+		g_snprintf (hot, 10, "%d", x_hot);
+		gdk_pixbuf_set_option (pixbuf, "x_hot", hot);
+		g_snprintf (hot, 10, "%d", y_hot);
+		gdk_pixbuf_set_option (pixbuf, "y_hot", hot);
+
+	}
+
+	return pixbuf;
 }
 
 /* Shared library entry point for file loading */
@@ -1427,7 +1425,8 @@ struct _XPMContext
  * in the future.
  */
 static gpointer
-gdk_pixbuf__xpm_image_begin_load (ModulePreparedNotifyFunc prepare_func,
+gdk_pixbuf__xpm_image_begin_load (ModuleSizeFunc size_func,
+                                  ModulePreparedNotifyFunc prepare_func,
                                   ModuleUpdatedNotifyFunc update_func,
                                   gpointer user_data,
                                   GError **error)
