@@ -25,12 +25,12 @@
 #include <config.h>
 #include <stdio.h>
 #include <string.h>
-#include "gdk.h"		/* For gdk_screen_width/gdk_screen_height */
 #include "gdkcolor.h"
 #include "gdkimage.h"
 #include "gdkvisual.h"
 #include "gdkwindow.h"
 #include "gdkpixbuf.h"
+#include "gdkpixmap.h"
 #include "gdk-pixbuf-private.h"
 #include "gdkinternals.h"
 
@@ -204,9 +204,9 @@ rgb1 (GdkImage    *image,
                                   (0x80 >> (xx & 7)) :
                                   (1 << (xx & 7)));
 
-	  *o++ = colormap->colors[data].red;
-	  *o++ = colormap->colors[data].green;
-	  *o++ = colormap->colors[data].blue;
+	  *o++ = colormap->colors[data].red   >> 8;
+	  *o++ = colormap->colors[data].green >> 8;
+	  *o++ = colormap->colors[data].blue  >> 8;
 	}
       srow += bpl;
       orow += rowstride;
@@ -251,9 +251,9 @@ rgb1a (GdkImage    *image,
                                   (0x80 >> (xx & 7)) :
                                   (1 << (xx & 7)));
 
-          *o++ = colormap->colors[data].red;
-	  *o++ = colormap->colors[data].green;
-	  *o++ = colormap->colors[data].blue;
+          *o++ = colormap->colors[data].red   >> 8;
+	  *o++ = colormap->colors[data].green >> 8;
+	  *o++ = colormap->colors[data].blue  >> 8;
 	  *o++ = 255;
 	}
       srow += bpl;
@@ -296,9 +296,9 @@ rgb8 (GdkImage    *image,
       for (xx = x1; xx < x2; xx++)
         {
           data = *s++ & mask;
-          *o++ = colormap->colors[data].red;
-          *o++ = colormap->colors[data].green;
-          *o++ = colormap->colors[data].blue;
+          *o++ = colormap->colors[data].red   >> 8;
+          *o++ = colormap->colors[data].green >> 8;
+          *o++ = colormap->colors[data].blue  >> 8;
         }
       srow += bpl;
       orow += rowstride;
@@ -338,14 +338,14 @@ rgb8a (GdkImage    *image,
     {
 #ifdef LITTLE
       remap[xx] = 0xff000000
-	| colormap->colors[xx].blue << 16
-	| colormap->colors[xx].green << 8
-	| colormap->colors[xx].red;
+	| (colormap->colors[xx].blue  & 0xff00) << 8
+	| (colormap->colors[xx].green & 0xff00)
+	| (colormap->colors[xx].red   >> 8);
 #else
       remap[xx] = 0xff
-	| colormap->colors[xx].red << 24
-	| colormap->colors[xx].green << 16
-	| colormap->colors[xx].blue << 8;
+	| (colormap->colors[xx].red   & 0xff00) << 16
+	| (colormap->colors[xx].green & 0xff00) << 8
+	| (colormap->colors[xx].blue  & 0xff00);
 #endif
     }
 
@@ -437,14 +437,15 @@ rgb565lsb (GdkImage    *image,
 	    | (data & 0x1f0000) >> 5 | (data & 0x1c0000) >> 10;
 #else
 	  /* swap endianness first */
-	  data = s[1] | s[0] << 8 | s[3] << 16 | s[2] << 24;
+	  data = s[1] << 24 | s[0] << 16 | s[3] << 8 | s[2];
 	  s += 4;
-	  *o++ = (data & 0xf800) | (data & 0xe000) >> 5
-	    | (data & 0x7e0) >> 3 | (data & 0x600) >> 9;
-	  *o++ = (data & 0x1f) << 11 | (data & 0x1c) << 6
-	    | (data & 0xf8000000) >> 24 | (data & 0xe0000000) >> 29;
-	  *o++ = (data & 0x7e00000) >> 11 | (data & 0x6000000) >> 17
-	    | (data & 0x1f0000) >> 13 | (data & 0x1c0000) >> 18;
+
+	  *o++ = (data & 0xf8000000) >> 16 | (data & 0xe0000000) >> 21
+	    | (data & 0x7e00000) >> 19 | (data & 0x6000000) >> 25;
+	  *o++ = (data & 0x1f0000) >> 5 | (data & 0x1c0000) >> 10
+	    | (data & 0xf800) >> 8 | (data & 0xe000) >> 13;
+	  *o++ = (data & 0x7e0) << 5 | (data & 0x600) >> 1
+	    | (data & 0x1f) << 3 | (data & 0x1c) >> 2;
 #endif
 	}
       /* check for last remaining pixel */
@@ -1217,6 +1218,14 @@ convert_real_slow (GdkImage    *image,
   bpl = image->bpl;
   v = gdk_colormap_get_visual(cmap);
 
+  if (image->depth != v->depth)
+    {
+      g_warning ("%s: The depth of the source image (%d) doesn't "
+                 "match the depth of the colormap passed in (%d).",
+                 G_STRLOC, image->depth, v->depth);
+      return;
+    } 
+ 
   d(printf("rgb  mask/shift/prec = %x:%x:%x %d:%d:%d  %d:%d:%d\n",
 	   v->red_mask, v->green_mask, v->blue_mask,
 	   v->red_shift, v->green_shift, v->blue_shift,
@@ -1235,9 +1244,9 @@ convert_real_slow (GdkImage    *image,
 	    case GDK_VISUAL_GRAYSCALE:
 	    case GDK_VISUAL_STATIC_COLOR:
 	    case GDK_VISUAL_PSEUDO_COLOR:
-	      *o++ = cmap->colors[pixel].red;
-	      *o++ = cmap->colors[pixel].green;
-	      *o++ = cmap->colors[pixel].blue;
+	      *o++ = cmap->colors[pixel].red   >> 8; 
+	      *o++ = cmap->colors[pixel].green >> 8;
+	      *o++ = cmap->colors[pixel].blue  >> 8;
 	      break;
 	    case GDK_VISUAL_TRUE_COLOR:
 				/* This is odd because it must sometimes shift left (otherwise
@@ -1258,9 +1267,9 @@ convert_real_slow (GdkImage    *image,
 	      *o++ = component;
 	      break;
 	    case GDK_VISUAL_DIRECT_COLOR:
-	      *o++ = cmap->colors[((pixel & v->red_mask) << (32 - v->red_shift - v->red_prec)) >> 24].red;
-	      *o++ = cmap->colors[((pixel & v->green_mask) << (32 - v->green_shift - v->green_prec)) >> 24].green;
-	      *o++ = cmap->colors[((pixel & v->blue_mask) << (32 - v->blue_shift - v->blue_prec)) >> 24].blue;
+	      *o++ = cmap->colors[((pixel & v->red_mask) << (32 - v->red_shift - v->red_prec)) >> 24].red >> 8;
+	      *o++ = cmap->colors[((pixel & v->green_mask) << (32 - v->green_shift - v->green_prec)) >> 24].green >> 8;
+	      *o++ = cmap->colors[((pixel & v->blue_mask) << (32 - v->blue_shift - v->blue_prec)) >> 24].blue >> 8;
 	      break;
 	    }
 	  if (alpha)
@@ -1328,6 +1337,15 @@ rgbconvert (GdkImage    *image,
     }
   
   v = gdk_colormap_get_visual (cmap);
+
+  if (image->depth != v->depth)
+    {
+      g_warning ("%s: The depth of the source image (%d) doesn't "
+                 "match the depth of the colormap passed in (%d).",
+                 G_STRLOC, image->depth, v->depth);
+      return;
+    } 
+ 
   bank = 5; /* default fallback converter */
   index = (image->byte_order == GDK_MSB_FIRST) | (alpha != 0) << 1;
   
@@ -1514,6 +1532,14 @@ gdk_pixbuf_get_from_drawable (GdkPixbuf   *dest,
       return NULL;
     }
   
+  if (cmap != NULL && depth != cmap->visual->depth)
+    {
+      g_warning ("%s: Depth of the source drawable is %d where as "
+                 "the visual depth of the colormap passed is %d",
+                 G_STRLOC, depth, cmap->visual->depth);
+      return NULL;
+    } 
+ 
   /* Coordinate sanity checks */
   
   if (GDK_IS_PIXMAP (src))
@@ -1552,7 +1578,8 @@ gdk_pixbuf_get_from_drawable (GdkPixbuf   *dest,
 	  
 	  gint width1 = MIN (width - x0, GDK_SCRATCH_IMAGE_WIDTH);
 	  
-	  image = _gdk_image_get_scratch (width1, height1, depth, &xs0, &ys0);
+	  image = _gdk_image_get_scratch (gdk_drawable_get_screen (src), 
+					  width1, height1, depth, &xs0, &ys0);
 
 	  _gdk_drawable_copy_to_image (src, image,
 				       src_x + x0, src_y + y0,
@@ -1621,6 +1648,14 @@ gdk_pixbuf_get_from_image (GdkPixbuf   *dest,
       return NULL;
     }
   
+  if (cmap != NULL && src->depth != cmap->visual->depth)
+    {
+      g_warning ("%s: Depth of the Source image is %d where as "
+                 "the visual depth of the colormap passed is %d",
+                 G_STRLOC, src->depth, cmap->visual->depth);
+      return NULL;
+    } 
+ 
   /* Coordinate sanity checks */
 
   g_return_val_if_fail (src_x >= 0 && src_y >= 0, NULL);
