@@ -25,6 +25,7 @@
 #include "gdkinternals.h" /* _gdk_draw_pixbuf() */
 #include "gdk-pixbuf-private.h"
 #include "gdkpixbuf.h"
+#include "gdkscreen.h"
 
 
 
@@ -88,7 +89,7 @@ gdk_pixbuf_render_threshold_alpha (GdkPixbuf *pixbuf,
       color.pixel = (alpha_threshold == 255) ? 0 : 1;
       gdk_gc_set_foreground (gc, &color);
       gdk_draw_rectangle (bitmap, gc, TRUE, dest_x, dest_y, width, height);
-      gdk_gc_unref (gc);
+      g_object_unref (gc);
       return;
     }
 
@@ -131,7 +132,7 @@ gdk_pixbuf_render_threshold_alpha (GdkPixbuf *pixbuf,
 		       x - 1 + dest_x, y + dest_y);
     }
 	
-  gdk_gc_unref (gc);
+  g_object_unref (gc);
 }
 
 
@@ -243,8 +244,10 @@ gdk_pixbuf_render_to_drawable_alpha (GdkPixbuf   *pixbuf,
 /**
  * gdk_pixbuf_render_pixmap_and_mask:
  * @pixbuf: A pixbuf.
- * @pixmap_return: Return value for the created pixmap.
- * @mask_return: Return value for the created mask.
+ * @pixmap_return: Location to store a pointer to the created pixmap,
+ *   or %NULL if the pixmap is not needed.
+ * @mask_return: Location to store a pointer to the created mask,
+ *   or %NULL if the mask is not needed.
  * @alpha_threshold: Threshold value for opacity values.
  *
  * Creates a pixmap and a mask bitmap which are returned in the @pixmap_return
@@ -278,8 +281,10 @@ gdk_pixbuf_render_pixmap_and_mask (GdkPixbuf  *pixbuf,
  * gdk_pixbuf_render_pixmap_and_mask_for_colormap:
  * @pixbuf: A pixbuf.
  * @colormap: A #GdkColormap
- * @pixmap_return: Return value for the created pixmap.
- * @mask_return: Return value for the created mask.
+ * @pixmap_return: Location to store a pointer to the created pixmap,
+ *   or %NULL if the pixmap is not needed.
+ * @mask_return: Location to store a pointer to the created mask,
+ *   or %NULL if the mask is not needed.
  * @alpha_threshold: Threshold value for opacity values.
  *
  * Creates a pixmap and a mask bitmap which are returned in the @pixmap_return
@@ -303,31 +308,37 @@ gdk_pixbuf_render_pixmap_and_mask_for_colormap (GdkPixbuf   *pixbuf,
 						GdkBitmap  **mask_return,
 						int          alpha_threshold)
 {
-  g_return_if_fail (pixbuf != NULL);
+  GdkScreen *screen;
+
+  g_return_if_fail (GDK_IS_PIXBUF (pixbuf));
+  g_return_if_fail (GDK_IS_COLORMAP (colormap));
+
+  screen = gdk_colormap_get_screen (colormap);
   
   if (pixmap_return)
     {
       GdkGC *gc;
-      
-      *pixmap_return = gdk_pixmap_new (NULL, gdk_pixbuf_get_width (pixbuf), gdk_pixbuf_get_height (pixbuf),
+      *pixmap_return = gdk_pixmap_new (gdk_screen_get_root_window (screen),
+				       gdk_pixbuf_get_width (pixbuf), gdk_pixbuf_get_height (pixbuf),
 				       gdk_colormap_get_visual (colormap)->depth);
-      gdk_drawable_set_colormap (GDK_DRAWABLE (*pixmap_return),
-                                 colormap);
+
+      gdk_drawable_set_colormap (GDK_DRAWABLE (*pixmap_return), colormap);
       gc = gdk_gc_new (*pixmap_return);
       gdk_pixbuf_render_to_drawable (pixbuf, *pixmap_return, gc,
 				     0, 0, 0, 0,
 				     gdk_pixbuf_get_width (pixbuf), gdk_pixbuf_get_height (pixbuf),
 				     GDK_RGB_DITHER_NORMAL,
 				     0, 0);
-      gdk_gc_unref (gc);
+      g_object_unref (gc);
     }
   
   if (mask_return)
     {
       if (gdk_pixbuf_get_has_alpha (pixbuf))
 	{
-	  *mask_return = gdk_pixmap_new (NULL, gdk_pixbuf_get_width (pixbuf), gdk_pixbuf_get_height (pixbuf), 1);
-          
+	  *mask_return = gdk_pixmap_new (gdk_screen_get_root_window (screen),
+					 gdk_pixbuf_get_width (pixbuf), gdk_pixbuf_get_height (pixbuf), 1);
+
 	  gdk_pixbuf_render_threshold_alpha (pixbuf, *mask_return,
 					     0, 0, 0, 0,
 					     gdk_pixbuf_get_width (pixbuf), gdk_pixbuf_get_height (pixbuf),
