@@ -30,6 +30,7 @@
 #include "gdkpixmap.h"
 #include "gdkdrawable.h"
 #include "gdkpixmap.h"
+#include "gdkscreen.h"
 
 #define USE_BACKING_STORE	/* Appears to work on Win32, too, now. */
 
@@ -48,7 +49,7 @@ static const GdkPointerHooks default_pointer_hooks = {
   _gdk_windowing_window_at_pointer
 };
 
-static const GdkPointerHooks *current_pointer_hooks = &default_pointer_hooks;
+const GdkPointerHooks *_gdk_current_pointer_hooks = &default_pointer_hooks;
 
 static GdkGC *gdk_window_create_gc      (GdkDrawable     *drawable,
                                          GdkGCValues     *values,
@@ -154,8 +155,9 @@ static void   gdk_window_real_get_size  (GdkDrawable     *drawable,
 
 static GdkVisual*   gdk_window_real_get_visual   (GdkDrawable *drawable);
 static gint         gdk_window_real_get_depth    (GdkDrawable *drawable);
+static GdkScreen*   gdk_window_real_get_screen   (GdkDrawable *drawable);
 static void         gdk_window_real_set_colormap (GdkDrawable *drawable,
-                                             GdkColormap *cmap);
+						  GdkColormap *cmap);
 static GdkColormap* gdk_window_real_get_colormap (GdkDrawable *drawable);
 
 static GdkDrawable* gdk_window_get_composite_drawable (GdkDrawable *drawable,
@@ -240,6 +242,7 @@ gdk_window_class_init (GdkWindowObjectClass *klass)
   drawable_class->draw_image = gdk_window_draw_image;
   drawable_class->_draw_pixbuf = gdk_window_draw_pixbuf;
   drawable_class->get_depth = gdk_window_real_get_depth;
+  drawable_class->get_screen = gdk_window_real_get_screen;
   drawable_class->get_size = gdk_window_real_get_size;
   drawable_class->set_colormap = gdk_window_real_set_colormap;
   drawable_class->get_colormap = gdk_window_real_get_colormap;
@@ -270,7 +273,7 @@ gdk_window_finalize (GObject *object)
 	_gdk_window_destroy (window, TRUE);
     }
 
-  g_object_unref (G_OBJECT (obj->impl));
+  g_object_unref (obj->impl);
   obj->impl = NULL;
   
   G_OBJECT_CLASS (parent_class)->finalize (object);
@@ -332,7 +335,7 @@ _gdk_window_destroy_hierarchy (GdkWindow *window,
               private->bg_pixmap != GDK_PARENT_RELATIVE_BG &&
               private->bg_pixmap != GDK_NO_BG)
 	    {
-	      gdk_pixmap_unref (private->bg_pixmap);
+	      g_object_unref (private->bg_pixmap);
 	      private->bg_pixmap = NULL;
 	    }
 	  
@@ -419,7 +422,7 @@ void
 gdk_window_destroy (GdkWindow *window)
 {
   _gdk_window_destroy_hierarchy (window, FALSE, FALSE);
-  gdk_drawable_unref (window);
+  g_object_unref (window);
 }
 
 /**
@@ -697,10 +700,46 @@ gdk_window_remove_filter (GdkWindow     *window,
     }
 }
 
+/**
+ * gdk_screen_get_toplevel_windows:
+ * @screen : The #GdkScreen where the toplevels are located.
+ * 
+ * Obtains a list of all toplevel windows known to GDK on the screen @screen.
+ * A toplevel window is a child of the root window (see
+ * gdk_get_default_root_window()).
+ *
+ * The returned list should be freed with g_list_free(), but
+ * its elements need not be freed.
+ * 
+ * Return value: list of toplevel windows, free with g_list_free()
+ **/
+GList *
+gdk_screen_get_toplevel_windows (GdkScreen *screen)
+{
+  GdkWindow * root_window;
+  GList *new_list = NULL;
+  GList *tmp_list;
+  
+  g_return_val_if_fail (GDK_IS_SCREEN (screen), NULL);
+  
+  root_window = gdk_screen_get_root_window (screen);
+
+  tmp_list = ((GdkWindowObject *)root_window)->children;
+  while (tmp_list)
+    {
+      if (GDK_WINDOW_TYPE (tmp_list->data) != GDK_WINDOW_FOREIGN)
+	new_list = g_list_prepend (new_list, tmp_list->data);
+      tmp_list = tmp_list->next;
+    }
+  
+  return new_list;
+}
+
 /**
  * gdk_window_get_toplevels:
  * 
- * Obtains a list of all toplevel windows known to GDK.
+ * Obtains a list of all toplevel windows known to GDK on the default
+ * screen (see gdk_window_get_toplevels_for_screen()).
  * A toplevel window is a child of the root window (see
  * gdk_get_default_root_window()).
  *
@@ -712,18 +751,7 @@ gdk_window_remove_filter (GdkWindow     *window,
 GList *
 gdk_window_get_toplevels (void)
 {
-  GList *new_list = NULL;
-  GList *tmp_list;
-  
-  tmp_list = ((GdkWindowObject *)_gdk_parent_root)->children;
-  while (tmp_list)
-    {
-      if (GDK_WINDOW_TYPE (tmp_list->data) != GDK_WINDOW_FOREIGN)
-	new_list = g_list_prepend (new_list, tmp_list->data);
-      tmp_list = tmp_list->next;
-    }
-  
-  return new_list;
+  return gdk_screen_get_toplevel_windows (gdk_screen_get_default ());
 }
 
 /**
@@ -758,12 +786,14 @@ gboolean
 gdk_window_is_viewable (GdkWindow *window)
 {
   GdkWindowObject *private = (GdkWindowObject *)window;
+  GdkScreen *screen = gdk_drawable_get_screen (window);
+  GdkWindow *root_window = gdk_screen_get_root_window (screen);
   
   g_return_val_if_fail (window != NULL, FALSE);
   g_return_val_if_fail (GDK_IS_WINDOW (window), FALSE);
   
   while (private && 
-	 (private != (GdkWindowObject *)_gdk_parent_root) &&
+	 (private != (GdkWindowObject *)root_window) &&
 	 (GDK_WINDOW_TYPE (private) != GDK_WINDOW_FOREIGN))
     {
       if (!GDK_WINDOW_IS_MAPPED (window))
@@ -870,7 +900,7 @@ gdk_window_paint_init_bg (GdkWindow      *window,
   gdk_gc_set_clip_region (tmp_gc, init_region);
 
   gdk_draw_rectangle (paint->pixmap, tmp_gc, TRUE, 0, 0, -1, -1);
-  gdk_gc_unref (tmp_gc);
+  g_object_unref (tmp_gc);
 }
 
 #ifdef GDK_WINDOWING_X11
@@ -970,8 +1000,8 @@ gdk_window_begin_paint_region (GdkWindow *window,
 			     0, 0,
 			     old_rect.x - new_rect.x, old_rect.y - new_rect.y,
                              old_rect.width, old_rect.height);
-          gdk_gc_unref (tmp_gc);
-	  gdk_drawable_unref (tmp_paint->pixmap);
+          g_object_unref (tmp_gc);
+	  g_object_unref (tmp_paint->pixmap);
 
 	  paint->x_offset = new_rect.x;
 	  paint->y_offset = new_rect.y;
@@ -1075,7 +1105,7 @@ gdk_window_end_paint (GdkWindow *window)
                      clip_box.y - paint->y_offset,
                      clip_box.x - x_offset, clip_box.y - y_offset,
                      clip_box.width, clip_box.height);
-  gdk_gc_unref (tmp_gc);
+  g_object_unref (tmp_gc);
 
   if (private->paint_stack)
     {
@@ -1089,7 +1119,7 @@ gdk_window_end_paint (GdkWindow *window)
 	}
     }
   else
-    gdk_drawable_unref (paint->pixmap);
+    g_object_unref (paint->pixmap);
 
   gdk_region_destroy (paint->region);
   g_free (paint);
@@ -1110,7 +1140,7 @@ gdk_window_free_paint_stack (GdkWindow *window)
 	  GdkWindowPaint *paint = tmp_list->data;
 
 	  if (tmp_list == private->paint_stack)
-	    gdk_drawable_unref (paint->pixmap);
+	    g_object_unref (paint->pixmap);
 		  
 	  gdk_region_destroy (paint->region);
 	  g_free (paint);
@@ -1420,7 +1450,7 @@ gdk_window_get_composite_drawable (GdkDrawable *window,
                                          composite_x_offset,
                                          composite_y_offset);
       
-      return GDK_DRAWABLE (g_object_ref (G_OBJECT (window)));
+      return g_object_ref (window);
     }
   
   buffered_region = NULL;
@@ -1470,7 +1500,7 @@ gdk_window_get_composite_drawable (GdkDrawable *window,
                                          composite_x_offset,
                                          composite_y_offset);
 
-      return GDK_DRAWABLE (g_object_ref (G_OBJECT (window)));
+      return g_object_ref (window);
     }
   
   tmp_pixmap = gdk_pixmap_new (window,
@@ -1519,7 +1549,7 @@ gdk_window_get_composite_drawable (GdkDrawable *window,
   *composite_x_offset = x;
   *composite_y_offset = y;
 
-  g_object_unref (G_OBJECT (tmp_gc));
+  g_object_unref (tmp_gc);
   
   return tmp_pixmap;
 }
@@ -1770,7 +1800,7 @@ gdk_window_clear_backing_rect (GdkWindow *window,
   tmp_gc = gdk_window_get_bg_gc (window, paint);
   gdk_draw_rectangle (paint->pixmap, tmp_gc, TRUE,
 		      x - paint->x_offset, y - paint->y_offset, width, height);
-  gdk_gc_unref (tmp_gc);
+  g_object_unref (tmp_gc);
 }
 
 /**
@@ -1974,13 +2004,17 @@ gdk_window_real_get_visual (GdkDrawable *drawable)
 static gint
 gdk_window_real_get_depth (GdkDrawable *drawable)
 {
-  gint depth;
-  
   g_return_val_if_fail (GDK_IS_WINDOW (drawable), 0);
 
   return ((GdkWindowObject *)GDK_WINDOW (drawable))->depth;
 }
 
+static GdkScreen*
+gdk_window_real_get_screen (GdkDrawable *drawable)
+{
+  return gdk_drawable_get_screen (GDK_WINDOW_OBJECT (drawable)->impl);
+}
+
 static void
 gdk_window_real_set_colormap (GdkDrawable *drawable,
                               GdkColormap *cmap)
@@ -2069,7 +2103,7 @@ gdk_window_process_updates_internal (GdkWindow *window)
           if (debug_updates)
             {
               /* Make sure we see the red invalid area before redrawing. */
-              gdk_flush ();
+              gdk_display_sync (gdk_drawable_get_display (window));
               g_usleep (70000);
             }
           
@@ -2083,7 +2117,7 @@ gdk_window_process_updates_internal (GdkWindow *window)
 	  save_region = _gdk_windowing_window_queue_antiexpose (window, update_area);
       
 	  event.expose.type = GDK_EXPOSE;
-	  event.expose.window = gdk_window_ref (window);
+	  event.expose.window = g_object_ref (window);
 	  event.expose.count = 0;
 
 	  if (save_region)
@@ -2105,7 +2139,7 @@ gdk_window_process_updates_internal (GdkWindow *window)
 
 	  if (expose_region != update_area)
 	    gdk_region_destroy (expose_region);
-	  gdk_window_unref (window);
+	  g_object_unref (window);
 	}
       if (!save_region)
 	gdk_region_destroy (update_area);
@@ -2256,12 +2290,12 @@ draw_ugly_color (GdkWindow *window,
   gdk_region_get_clipbox (region, &clipbox);
   
   gdk_draw_rectangle (window,
-                     ugly_gc,
-                     TRUE,
-                     clipbox.x, clipbox.y,
-                     clipbox.width, clipbox.height);
+		      ugly_gc,
+		      TRUE,
+		      clipbox.x, clipbox.y,
+		      clipbox.width, clipbox.height);
   
-  g_object_unref (G_OBJECT (ugly_gc));
+  g_object_unref (ugly_gc);
 }
 
 /**
@@ -2681,12 +2715,12 @@ gdk_window_constrain_size (GdkGeometry *geometry,
 GdkPointerHooks *
 gdk_set_pointer_hooks (const GdkPointerHooks *new_hooks)
 {
-  const GdkPointerHooks *result = current_pointer_hooks;
+  const GdkPointerHooks *result = _gdk_current_pointer_hooks;
 
   if (new_hooks)
-    current_pointer_hooks = new_hooks;
+    _gdk_current_pointer_hooks = new_hooks;
   else
-    current_pointer_hooks = &default_pointer_hooks;
+    _gdk_current_pointer_hooks = &default_pointer_hooks;
 
   return (GdkPointerHooks *)result;
 }
@@ -2713,7 +2747,7 @@ gdk_window_get_pointer (GdkWindow	  *window,
 {
   g_return_val_if_fail (window == NULL || GDK_IS_WINDOW (window), NULL);
   
-  return current_pointer_hooks->get_pointer (window, x, y, mask); 
+  return _gdk_current_pointer_hooks->get_pointer (window, x, y, mask); 
 }
 
 /**
@@ -2725,6 +2759,9 @@ gdk_window_get_pointer (GdkWindow	  *window,
  * location of that window in @win_x, @win_y. Returns %NULL if the
  * window under the mouse pointer is not known to GDK (for example,
  * belongs to another application).
+ *
+ * NOTE: For multihead-aware widgets or applications use
+ * gdk_screen_get_window_at_pointer() instead.
  * 
  * Return value: window under the mouse pointer
  **/
@@ -2732,7 +2769,7 @@ GdkWindow*
 gdk_window_at_pointer (gint *win_x,
 		       gint *win_y)
 {
-  return current_pointer_hooks->window_at_pointer (NULL, win_x, win_y);
+  return gdk_screen_get_window_at_pointer (gdk_screen_get_default (), win_x, win_y);
 }
 
 /**
@@ -2746,6 +2783,25 @@ gdk_window_at_pointer (gint *win_x,
 GdkWindow *
 gdk_get_default_root_window (void)
 {
-  return _gdk_parent_root;
+  return gdk_screen_get_root_window (gdk_screen_get_default ());
+}
+
+/**
+ * gdk_window_foreign_new:
+ * @anid: a native window handle.
+ * 
+ * Wraps a native window for the default display in a #GdkWindow.
+ * This may fail if the window has been destroyed.
+ *
+ * For example in the X backend, a native window handle is an Xlib
+ * <type>XID</type>.
+ * 
+ * Return value: the newly-created #GdkWindow wrapper for the 
+ *    native window or %NULL if the window has been destroyed.
+ **/
+GdkWindow *
+gdk_window_foreign_new (GdkNativeWindow anid)
+{
+  return gdk_window_foreign_new_for_display (gdk_display_get_default (), anid);
 }
 
