@@ -96,7 +96,6 @@ gdk_window_impl_fb_init (GdkWindowFBData *impl)
 {
   impl->drawable_data.depth = gdk_display->modeinfo.bits_per_pixel;
   impl->drawable_data.colormap = gdk_colormap_get_system ();
-  impl->event_mask = GDK_STRUCTURE_MASK;
   impl->shape = NULL;
 }
 
@@ -1469,6 +1468,7 @@ gdk_window_set_back_pixmap (GdkWindow *window,
   g_return_if_fail (window != NULL);
   g_return_if_fail (GDK_IS_WINDOW (window));
   g_return_if_fail (pixmap == NULL || !parent_relative);
+  g_return_if_fail (pixmap == NULL || gdk_drawable_get_depth (window) == gdk_drawable_get_depth (pixmap));
 
   old_pixmap = private->bg_pixmap;
 
@@ -1733,7 +1733,7 @@ gdk_window_get_events (GdkWindow *window)
   if (GDK_WINDOW_DESTROYED (window))
     return 0;
   else
-    return GDK_WINDOW_IMPL_FBDATA (window)->event_mask;
+    return GDK_WINDOW_OBJECT (window)->event_mask;
 }
 
 void          
@@ -1749,10 +1749,7 @@ gdk_window_set_events (GdkWindow       *window,
       GDK_BUTTON1_MOTION_MASK | GDK_BUTTON2_MOTION_MASK |
       GDK_BUTTON3_MOTION_MASK;
   
-  if (!GDK_WINDOW_DESTROYED (window))
-    GDK_WINDOW_IMPL_FBDATA (window)->event_mask = event_mask;
-
-  GDK_WINDOW_OBJECT (window)->event_mask = event_mask;
+  GDK_WINDOW_OBJECT (window)->event_mask = GDK_STRUCTURE_MASK | event_mask;
 }
 
 void
@@ -1961,7 +1958,9 @@ gdk_window_shape_combine_mask (GdkWindow *window,
   _gdk_window_shape_combine_region (window,
 				    (mask == GDK_FB_USE_CHILD_SHAPE)?
 				    GDK_FB_USE_CHILD_SHAPE :
-				    gdk_fb_region_create_from_bitmap (mask),
+				    mask ?
+				    gdk_fb_region_create_from_bitmap (mask) :
+				    NULL,
 				    x, y);
 }
 
@@ -2184,6 +2183,22 @@ gdk_window_deiconify (GdkWindow *window)
   /*  g_warning ("gdk_window_deiconify() not implemented.\n");*/
 }
 
+void
+gdk_window_fullscreen (GdkWindow *window)
+{
+  g_return_if_fail (GDK_IS_WINDOW (window));
+
+  g_warning ("gdk_window_fullscreen() not implemented.\n");
+}
+
+void
+gdk_window_unfullscreen (GdkWindow *window)
+{
+  g_return_if_fail (GDK_IS_WINDOW (window));
+
+  /*  g_warning ("gdk_window_unfullscreen() not implemented.\n");*/
+}
+
 void
 gdk_window_focus (GdkWindow *window,
                   guint32    timestamp)
@@ -2208,6 +2223,20 @@ gdk_window_set_modal_hint (GdkWindow *window,
   g_return_if_fail (GDK_IS_WINDOW (window));
 }
 
+void
+gdk_window_set_skip_taskbar_hint (GdkWindow *window,
+				  gboolean   skips_taskbar)
+{
+  g_return_if_fail (GDK_IS_WINDOW (window));
+}
+
+void
+gdk_window_set_skip_pager_hint (GdkWindow *window,
+				gboolean   skips_pager)
+{
+  g_return_if_fail (GDK_IS_WINDOW (window));
+}
+
 void
 gdk_window_begin_resize_drag (GdkWindow     *window,
                               GdkWindowEdge  edge,
@@ -2251,7 +2280,8 @@ gdk_window_get_frame_extents (GdkWindow    *window,
 }
 
 GdkWindow*
-gdk_window_foreign_new (GdkNativeWindow anid)
+gdk_window_foreign_new_for_display (GdkDisplay      *display,
+				    GdkNativeWindow  anid)
 {
   return (GdkWindow*) gdk_drawable_ref (anid);
 }
@@ -2262,3 +2292,10 @@ gdk_window_lookup (GdkNativeWindow anid)
   return (GdkWindow*) (anid);
 }
 
+GdkWindow *
+gdk_window_lookup_for_display (GdkDisplay *display, GdkNativeWindow anid)
+{
+  g_return_val_if_fail (display == gdk_display_get_default(), NULL);
+
+  return (GdkWindow*) (anid);
+}
