@@ -33,6 +33,7 @@
 
 #include "gdkfont.h"
 #include "gdkpango.h" /* gdk_pango_context_get() */
+#include "gdkdisplay.h"
 #include "gdkprivate-win32.h"
 
 static GHashTable *font_name_hash = NULL;
@@ -123,332 +124,6 @@ charset_name (DWORD charset)
   return "unknown";
 }
 
-static gint num_fonts;
-static gint font_names_size;
-static gchar **xfontnames;
-
-static gchar *
-logfont_to_xlfd (const LOGFONT *lfp,
-		 int            size,
-		 int            res,
-		 int            avg_width)
-{
-  const gchar *weight;
-  const gchar *registry, *encoding;
-  int point_size;
-  static int logpixelsy = 0;
-  gchar facename[LF_FACESIZE*5];
-  gchar *utf8_facename;
-  gchar *p;
-  const gchar *q;
-
-  if (logpixelsy == 0)
-    {
-      logpixelsy = GetDeviceCaps (gdk_display_hdc, LOGPIXELSY);
-    }
-
-  if (lfp->lfWeight >= FW_HEAVY)
-    weight = "heavy";
-  else if (lfp->lfWeight >= FW_EXTRABOLD)
-    weight = "extrabold";
-  else if (lfp->lfWeight >= FW_BOLD)
-    weight = "bold";
-#ifdef FW_DEMIBOLD
-  else if (lfp->lfWeight >= FW_DEMIBOLD)
-    weight = "demibold";
-#endif
-  else if (lfp->lfWeight >= FW_MEDIUM)
-    weight = "medium";
-  else if (lfp->lfWeight >= FW_NORMAL)
-    weight = "normal";
-  else if (lfp->lfWeight >= FW_LIGHT)
-    weight = "light";
-  else if (lfp->lfWeight >= FW_EXTRALIGHT)
-    weight = "extralight";
-  else if (lfp->lfWeight >= FW_THIN)
-    weight = "thin";
-  else
-    weight = "regular";
-
-  switch (lfp->lfCharSet)
-    {
-    case ANSI_CHARSET:
-      registry = "iso8859";
-      encoding = "1";
-      break;
-    case SHIFTJIS_CHARSET:
-      registry = "jisx0208.1983";
-      encoding = "0";
-      break;
-    case HANGEUL_CHARSET:
-      registry = "ksc5601.1987";
-      encoding = "0";
-      break;
-    case GB2312_CHARSET:
-      registry = "gb2312.1980";
-      encoding = "0";
-      break;
-    case CHINESEBIG5_CHARSET:
-      registry = "big5";
-      encoding = "0";
-      break;
-    case GREEK_CHARSET:
-      registry = "iso8859";
-      encoding = "7";
-      break;
-    case TURKISH_CHARSET:
-      registry = "iso8859";
-      encoding = "9";
-      break;
-#if 0 /* Not a good idea, I think, to use ISO8859-8 and -6 for the Windows
-       * hebrew and arabic codepages, they differ too much.
-       */
-    case HEBREW_CHARSET:
-      registry = "iso8859";
-      encoding = "8";
-      break;
-    case ARABIC_CHARSET:
-      registry = "iso8859";
-      encoding = "6";
-      break;
-#endif
-    default:
-      registry = "microsoft";
-      encoding = charset_name (lfp->lfCharSet);
-    }
-  
-  point_size = (int) (((double) size/logpixelsy) * 720.);
-
-  if (res == -1)
-    res = logpixelsy;
-
-  /* Convert the facename Windows fives us from the locale-dependent
-   * codepage to UTF-8.
-   */
-  utf8_facename = g_filename_to_utf8 (lfp->lfFaceName, -1, NULL, NULL, NULL);
-
-  /* Replace characters illegal in an XLFD with hex escapes. */
-  p = facename;
-  q = utf8_facename;
-  while (*q)
-    {
-      if (*q == '-' || *q == '*' || *q == '?' || *q == '%')
-	p += sprintf (p, "%%%.02x", *q);
-      else
-	*p++ = *q;
-      q++;
-    }
-  *p = '\0';
-  g_free (utf8_facename);
-
-  return g_strdup_printf
-    ("-%s-%s-%s-%s-%s-%s-%d-%d-%d-%d-%s-%d-%s-%s",
-     "unknown", 
-     facename,
-     weight,
-     (lfp->lfItalic ?
-      ((lfp->lfPitchAndFamily & 0xF0) == FF_ROMAN
-       || (lfp->lfPitchAndFamily & 0xF0) == FF_SCRIPT ?
-       "i" : "o") : "r"),
-     "normal",
-     "",
-     size,
-     point_size,
-     res,
-     res,
-     ((lfp->lfPitchAndFamily & 0x03) == FIXED_PITCH ? "m" : "p"),
-     avg_width,
-     registry, encoding);
-}
-
-gchar *
-gdk_font_full_name_get (GdkFont *font)
-{
-  GdkFontPrivateWin32 *private;
-  GdkWin32SingleFont *singlefont;
-  GSList *list;
-  GString *string;
-  gchar *result;
-  gchar *xlfd;
-  LOGFONT logfont;
-
-  g_return_val_if_fail (font != NULL, NULL);
-
-  private = (GdkFontPrivateWin32 *) font;
-
-  list = private->fonts;
-  string = g_string_new ("");
-
-  while (list)
-    {
-      singlefont = (GdkWin32SingleFont *) list->data;
-
-      if (GetObject (singlefont->hfont, sizeof (LOGFONT), &logfont) == 0)
-	{
-	  WIN32_GDI_FAILED ("GetObject");
-	  return NULL;
-	}
-
-      xlfd = logfont_to_xlfd (&logfont, logfont.lfHeight, -1, 0);
-      string = g_string_append (string, xlfd);
-      g_free (xlfd);
-      list = list->next;
-      if (list)
-	string = g_string_append_c (string, ',');
-    }
-  result = string->str;
-  g_string_free (string, FALSE);
-  return result;
-}
-
-void
-gdk_font_full_name_free (gchar *name)
-{
-  g_free (name);
-}
-
-static gboolean
-pattern_match (const gchar *pattern,
-	       const gchar *string)
-{
-  const gchar *p = pattern, *n = string;
-  gchar c, c1;
-
-  /* Common case first */
-  if ((pattern[0] == '*'
-       && pattern[1] == '\0')
-      || (pattern[0] == '-'
-	  && pattern[1] == '*'
-	  && pattern[2] == '\0'))
-    return TRUE;
-
-  while ((c = *p++) != '\0')
-    {
-      c = tolower (c);
-
-      switch (c)
-	{
-	case '?':
-	  if (*n == '\0')
-	    return FALSE;
-	  break;
-
-	case '*':
-	  for (c = *p++; c == '?' || c == '*'; c = *p++, ++n)
-	    if (c == '?' && *n == '\0')
-	    return FALSE;
-
-	  if (c == '\0')
-	    return TRUE;
-
-	  c1 = tolower (c);
-	  for (--p; *n != '\0'; ++n)
-	    if (tolower (*n) == c1
-		&& pattern_match (p, n))
-	      return TRUE;
-	  return FALSE;
-
-	default:
-	  if (c != tolower (*n))
-	    return FALSE;
-	}
-
-      ++n;
-    }
-
-  if (*n == '\0')
-    return TRUE;
-
-  return FALSE;
-}
-
-static int CALLBACK
-InnerEnumFontFamExProc (const LOGFONT    *lfp,
-			const TEXTMETRIC *metrics,
-			DWORD	          fontType,
-			LPARAM            lParam)
-{
-  int size;
-  gchar *xlfd;
-
-  if (fontType == TRUETYPE_FONTTYPE)
-    {
-      size = 0;
-    }
-  else
-    {
-      size = lfp->lfHeight;
-    }
-
-  xlfd = logfont_to_xlfd (lfp, size, 0, 0);
-
-  if (!pattern_match ((gchar *) lParam, xlfd))
-    {
-      g_free (xlfd);
-      return 1;
-    }
-
-  num_fonts++;
-  if (num_fonts == font_names_size)
-    {
-      font_names_size *= 2;
-      xfontnames = g_realloc (xfontnames, font_names_size * sizeof (gchar *));
-    }
-  xfontnames[num_fonts-1] = xlfd;
-    
-  return 1;
-}
-
-static int CALLBACK
-EnumFontFamExProc (const LOGFONT    *lfp,
-		   const TEXTMETRIC *metrics,
-		   DWORD             fontType,
-		   LPARAM            lParam)
-{
-  if (fontType == TRUETYPE_FONTTYPE)
-    {
-      LOGFONT lf;
-
-      lf = *lfp;
-
-      EnumFontFamiliesEx (gdk_display_hdc, &lf, InnerEnumFontFamExProc, lParam, 0);
-    }
-  else
-    InnerEnumFontFamExProc (lfp, metrics, fontType, lParam);
-
-  return 1;
-}
-
-gchar **
-gdk_font_list_new (const gchar *font_pattern,
-		   gint        *n_returned)
-{
-  LOGFONT logfont;
-  gchar **result;
-
-  num_fonts = 0;
-  font_names_size = 100;
-  xfontnames = g_new (gchar *, font_names_size);
-  memset (&logfont, 0, sizeof (logfont));
-  logfont.lfCharSet = DEFAULT_CHARSET;
-  EnumFontFamiliesEx (gdk_display_hdc, &logfont, EnumFontFamExProc,
-		      (LPARAM) font_pattern, 0);
-
-  result = g_new (gchar *, num_fonts + 1);
-  memmove (result, xfontnames, num_fonts * sizeof (gchar *));
-  result[num_fonts] = NULL;
-  g_free (xfontnames);
-
-  *n_returned = num_fonts;
-  return result;
-}
-
-void
-gdk_font_list_free (gchar **font_list)
-{
-  g_strfreev (font_list);
-}
-
 /* This table classifies Unicode characters according to the Microsoft
  * Unicode subset numbering. This is based on the table in "Developing
  * International Software for Windows 95 and Windows NT". This is almost,
@@ -1515,7 +1190,8 @@ gdk_font_from_one_singlefont (GdkWin32SingleFont *singlefont)
 }
 
 GdkFont*
-gdk_font_load (const gchar *font_name)
+gdk_font_load_for_display (GdkDisplay  *display,
+                           const gchar *font_name)
 {
   GdkFont *font;
   GdkFontPrivateWin32 *private;
@@ -1524,6 +1200,7 @@ gdk_font_load (const gchar *font_name)
   TEXTMETRIC textmetric;
 
   g_return_val_if_fail (font_name != NULL, NULL);
+  g_return_val_if_fail (display == gdk_display_get_default (), NULL);
 
   font = gdk_font_hash_lookup (GDK_FONT_FONTSET, font_name);
   if (font)
@@ -1571,13 +1248,15 @@ gdk_font_load (const gchar *font_name)
  * cannot be loaded.
  **/
 GdkFont*
-gdk_font_from_description (PangoFontDescription *font_desc)
+gdk_font_from_description_for_display (GdkDisplay           *display,
+                                       PangoFontDescription *font_desc)
 {
   PangoFontMap *font_map;
   PangoFont *font;
   GdkFont *result = NULL;
 
   g_return_val_if_fail (font_desc != NULL, NULL);
+  g_return_val_if_fail (display == gdk_display_get_default (), NULL);
 
   font_map = pango_win32_font_map_for_display ();
   font = pango_font_map_load_font (font_map, gdk_pango_context_get (), font_desc);
@@ -1589,7 +1268,7 @@ gdk_font_from_description (PangoFontDescription *font_desc)
       result = gdk_font_from_one_singlefont (gdk_font_load_logfont (lfp));
       g_free (lfp);
 
-      g_object_unref (G_OBJECT (font));
+      g_object_unref (font);
     }
 
   return result;
@@ -1924,14 +1603,11 @@ gdk_text_width (GdkFont      *font,
 		const gchar  *text,
 		gint          text_length)
 {
-  gdk_text_size_arg arg;
+  gint width = -1;
 
-  arg.total.cx = arg.total.cy = 0;
+  gdk_text_extents (font, text, text_length, NULL, NULL, &width, NULL, NULL);
 
-  if (!gdk_text_size (font, text, text_length, &arg))
-    return -1;
-
-  return arg.total.cx;
+  return width;
 }
 
 gint
@@ -1939,36 +1615,10 @@ gdk_text_width_wc (GdkFont	  *font,
 		   const GdkWChar *text,
 		   gint		   text_length)
 {
-  gdk_text_size_arg arg;
-  wchar_t *wcstr;
-  gint i;
+  gint width = -1;
 
-  g_return_val_if_fail (font != NULL, -1);
-  g_return_val_if_fail (text != NULL, -1);
-
-  if (text_length == 0)
-    return 0;
-
-  g_assert (font->type == GDK_FONT_FONT || font->type == GDK_FONT_FONTSET);
-
-  if (sizeof (wchar_t) != sizeof (GdkWChar))
-    {
-      wcstr = g_new (wchar_t, text_length);
-      for (i = 0; i < text_length; i++)
-	wcstr[i] = text[i];
-    }
-  else
-    wcstr = (wchar_t *) text;
-
-  arg.total.cx = arg.total.cy = 0;
-
-  _gdk_wchar_text_handle (font, wcstr, text_length,
-			 gdk_text_size_handler, &arg);
-
-  if (sizeof (wchar_t) != sizeof (GdkWChar))
-    g_free (wcstr);
-
-  return arg.total.cx;
+  gdk_text_extents_wc (font, text, text_length, NULL, NULL, &width, NULL, NULL);
+  return width;
 }
 
 void
