@@ -246,7 +246,7 @@ gdk_input_wintab_init (void)
       wa.window_type = GDK_WINDOW_TOPLEVEL;
       if ((wintab_window = gdk_window_new (NULL, &wa, GDK_WA_X|GDK_WA_Y)) == NULL)
 	{
-	  g_warning ("_gdk_input_init: gdk_window_new failed");
+	  g_warning ("gdk_input_wintab_init: gdk_window_new failed");
 	  return;
 	}
       gdk_drawable_ref (wintab_window);
@@ -330,7 +330,7 @@ gdk_input_wintab_init (void)
 	  hctx = g_new (HCTX, 1);
           if ((*hctx = WTOpen (GDK_WINDOW_HWND (wintab_window), &lc, TRUE)) == NULL)
 	    {
-	      g_warning ("_gdk_input_init: WTOpen failed");
+	      g_warning ("gdk_input_wintab_init: WTOpen failed");
 	      return;
 	    }
 	  GDK_NOTE (INPUT, g_print ("opened Wintab device %d %p\n",
@@ -566,8 +566,8 @@ gdk_input_translate_coordinates (GdkDevicePrivate *gdkdev,
 
   if (gdkdev->info.mode == GDK_MODE_SCREEN) 
     {
-      x_scale = gdk_screen_width() / device_width;
-      y_scale = gdk_screen_height() / device_height;
+      x_scale = GetSystemMetrics (SM_CXSCREEN) / device_width;
+      y_scale = GetSystemMetrics (SM_CYSCREEN) / device_height;
 
       x_offset = - input_window->root_x;
       y_offset = - input_window->root_y;
@@ -661,7 +661,7 @@ _gdk_input_configure_event (GdkEventConfigure *event,
   GdkInputWindow *input_window;
   int root_x, root_y;
 
-  input_window = gdk_input_window_find (window);
+  input_window = _gdk_input_window_find (window);
   g_return_if_fail (window != NULL);
 
   gdk_input_get_root_relative_geometry (GDK_WINDOW_HWND (window),
@@ -678,7 +678,7 @@ _gdk_input_enter_event (GdkEventCrossing *event,
   GdkInputWindow *input_window;
   int root_x, root_y;
 
-  input_window = gdk_input_window_find (window);
+  input_window = _gdk_input_window_find (window);
   g_return_if_fail (window != NULL);
 
   gdk_input_get_root_relative_geometry (GDK_WINDOW_HWND (window), &root_x, &root_y);
@@ -696,6 +696,7 @@ _gdk_input_other_event (GdkEvent  *event,
 #if !USE_SYSCONTEXT
   GdkWindow *current_window;
 #endif
+  GdkDisplay *display;
   GdkWindowObject *obj;
   GdkWindowImplWin32 *impl;
   GdkInputWindow *input_window;
@@ -719,6 +720,7 @@ _gdk_input_other_event (GdkEvent  *event,
     window = _gdk_parent_root;
 
   gdk_drawable_ref (window);
+  display = gdk_drawable_get_display (window);
 
   GDK_NOTE (EVENTS_OR_INPUT,
 	    g_print ("gdk_input_win32_other_event: window=%p (%d,%d)\n",
@@ -833,7 +835,7 @@ _gdk_input_other_event (GdkEvent  *event,
 	  goto dijkstra;
 	}
 
-      input_window = gdk_input_window_find (window);
+      input_window = _gdk_input_window_find (window);
 
       g_assert (input_window != NULL);
 
@@ -928,7 +930,7 @@ _gdk_input_other_event (GdkEvent  *event,
 				 event2->button.button,
 				 event2->button.x,
 				 event2->button.y));
-	      _gdk_event_queue_append (event2);
+	      _gdk_event_queue_append (display, event2);
 	    }
 	}
       return TRUE;
@@ -1160,7 +1162,7 @@ gdk_device_get_state (GdkDevice       *device,
 	gdk_window_get_pointer (window, NULL, NULL, mask);
       
       gdkdev = (GdkDevicePrivate *)device;
-      input_window = gdk_input_window_find (window);
+      input_window = _gdk_input_window_find (window);
       g_return_if_fail (input_window != NULL);
 
 #if 0 /* FIXME */
@@ -1196,16 +1198,16 @@ gdk_device_get_state (GdkDevice       *device,
 }
 
 void 
-_gdk_input_init (void)
+_gdk_input_init (GdkDisplay *display)
 {
   _gdk_input_ignore_core = FALSE;
   _gdk_input_devices = NULL;
 
-  _gdk_init_input_core ();
+  _gdk_init_input_core (display);
 #ifdef HAVE_WINTAB
   gdk_input_wintab_init ();
 #endif /* HAVE_WINTAB */
 
-  _gdk_input_devices = g_list_append (_gdk_input_devices, _gdk_core_pointer);
+  _gdk_input_devices = g_list_append (_gdk_input_devices, display->core_pointer);
 }
 
