@@ -187,7 +187,7 @@ gtk_button_class_init (GtkButtonClass *klass)
                                    PROP_LABEL,
                                    g_param_spec_string ("label",
                                                         _("Label"),
-                                                        _("Text of the label widget inside the button, if the button contains a label widget."),
+                                                        _("Text of the label widget inside the button, if the button contains a label widget"),
                                                         NULL,
                                                         G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
   
@@ -211,7 +211,7 @@ gtk_button_class_init (GtkButtonClass *klass)
                                    PROP_RELIEF,
                                    g_param_spec_enum ("relief",
                                                       _("Border relief"),
-                                                      _("The border relief style."),
+                                                      _("The border relief style"),
                                                       GTK_TYPE_RELIEF_STYLE,
                                                       GTK_RELIEF_NORMAL,
                                                       G_PARAM_READABLE | G_PARAM_WRITABLE));
@@ -253,7 +253,7 @@ gtk_button_class_init (GtkButtonClass *klass)
 		    GTK_TYPE_NONE, 0);
   button_signals[ACTIVATE] =
     gtk_signal_new ("activate",
-                    GTK_RUN_FIRST,
+                    GTK_RUN_FIRST | GTK_RUN_ACTION,
                     GTK_CLASS_TYPE (object_class),
                     GTK_SIGNAL_OFFSET (GtkButtonClass, activate),
                     _gtk_marshal_VOID__VOID,
@@ -306,6 +306,7 @@ gtk_button_init (GtkButton *button)
   button->use_stock = FALSE;
   button->use_underline = FALSE;
   button->depressed = FALSE;
+  button->depress_on_activate = TRUE;
 }
 
 static void
@@ -1069,6 +1070,7 @@ gtk_real_button_activate (GtkButton *button)
 						    button);
 	  button->button_down = TRUE;
 	  gtk_button_update_state (button);
+	  gtk_widget_queue_draw (GTK_WIDGET (button));
 	}
     }
 }
@@ -1082,12 +1084,14 @@ gtk_button_finish_activate (GtkButton *button,
   g_source_remove (button->activate_timeout);
   button->activate_timeout = 0;
 
-  gdk_keyboard_ungrab (gtk_get_current_event_time ());
+  gdk_display_keyboard_ungrab (gtk_widget_get_display (widget),
+			       gtk_get_current_event_time ());
   gtk_grab_remove (widget);
 
   button->button_down = FALSE;
 
   gtk_button_update_state (button);
+  gtk_widget_queue_draw (GTK_WIDGET (button));
 
   if (do_it)
     gtk_button_clicked (button);
@@ -1123,13 +1127,13 @@ gtk_button_set_label (GtkButton   *button,
  * @button: a #GtkButton
  *
  * Fetches the text from the label of the button, as set by
- * gtk_button_set_label().  This string is owned by the widget 
- * and must not be modified or freed.  If the label text has not 
- * been set the return value  will be %NULL. This will be the 
+ * gtk_button_set_label(). If the label text has not 
+ * been set the return value will be %NULL. This will be the 
  * case if you create an empty button with gtk_button_new() to 
  * use as a container.
  *
- * Return value: the text of the label widget. 
+ * Return value: The text of the label widget. This string is owned
+ * by the widget and must not be modified or freed.
  **/
 G_CONST_RETURN gchar *
 gtk_button_get_label (GtkButton *button)
@@ -1257,12 +1261,15 @@ gtk_button_update_state (GtkButton *button)
   gboolean depressed;
   GtkStateType new_state;
 
-  depressed = button->button_down && (button->in_button || button->activate_timeout);
+  if (button->activate_timeout)
+    depressed = button->depress_on_activate;
+  else
+    depressed = button->in_button && button->button_down;
 
-  if (!button->button_down && button->in_button)
+  if (button->in_button && (!button->button_down || !depressed))
     new_state = GTK_STATE_PRELIGHT;
   else
-    new_state = depressed ? GTK_STATE_ACTIVE: GTK_STATE_NORMAL;
+    new_state = depressed ? GTK_STATE_ACTIVE : GTK_STATE_NORMAL;
 
   _gtk_button_set_depressed (button, depressed); 
   gtk_widget_set_state (GTK_WIDGET (button), new_state);
