@@ -24,6 +24,10 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
+/* Do NOT, I repeat, NOT, copy any of the code in this file.
+ * The code here relies on all sorts of internal details of GTK+
+ */
+
 #include <string.h>
 
 #include "gtkarrow.h"
@@ -58,6 +62,8 @@ enum {
 
 static void         gtk_combo_class_init         (GtkComboClass    *klass);
 static void         gtk_combo_init               (GtkCombo         *combo);
+static void         gtk_combo_realize		 (GtkWidget	   *widget);
+static void         gtk_combo_unrealize		 (GtkWidget	   *widget);
 static void         gtk_combo_destroy            (GtkObject        *combo);
 static GtkListItem *gtk_combo_find               (GtkCombo         *combo);
 static gchar *      gtk_combo_func               (GtkListItem      *li);
@@ -171,6 +177,8 @@ gtk_combo_class_init (GtkComboClass * klass)
   oclass->destroy = gtk_combo_destroy;
   
   widget_class->size_allocate = gtk_combo_size_allocate;
+  widget_class->realize = gtk_combo_realize;
+  widget_class->unrealize = gtk_combo_unrealize;
 }
 
 static void
@@ -195,7 +203,6 @@ gtk_combo_entry_key_press (GtkEntry * entry, GdkEventKey * event, GtkCombo * com
 
   /* completion */
   if ((event->keyval == GDK_Tab ||
-       event->keyval == GDK_ISO_Left_Tab ||
        event->keyval == GDK_KP_Tab) &&
       (event->state & GDK_MOD1_MASK)) 
     {
@@ -289,7 +296,8 @@ gtk_combo_window_key_press (GtkWidget   *window,
 	  if (GTK_WIDGET_HAS_GRAB (combo->popwin))
 	    {
 	      gtk_grab_remove (combo->popwin);
-	      gdk_pointer_ungrab (event->time);
+	      gdk_display_pointer_ungrab (gtk_widget_get_display (window),
+					  event->time);
 	    }
 	}
 
@@ -305,29 +313,39 @@ static GtkListItem *
 gtk_combo_find (GtkCombo * combo)
 {
   const gchar *text;
+  GtkListItem *found = NULL;
   gchar *ltext;
+  gchar *compare_text;
   GList *clist;
-  int (*string_compare) (const char *, const char *);
-
-  if (combo->case_sensitive)
-    string_compare = strcmp;
-  else
-    string_compare = g_strcasecmp;
 
   text = gtk_entry_get_text (GTK_ENTRY (combo->entry));
-  clist = GTK_LIST (combo->list)->children;
-
-  while (clist && clist->data)
+  if (combo->case_sensitive)
+    compare_text = (gchar *)text;
+  else
+    compare_text = g_utf8_casefold (text, -1);
+  
+  for (clist = GTK_LIST (combo->list)->children;
+       !found && clist;
+       clist = clist->next)
     {
       ltext = gtk_combo_func (GTK_LIST_ITEM (clist->data));
       if (!ltext)
 	continue;
-      if (!(*string_compare) (ltext, text))
-	return (GtkListItem *) clist->data;
-      clist = clist->next;
+
+      if (!combo->case_sensitive)
+	ltext = g_utf8_casefold (ltext, -1);
+
+      if (strcmp (ltext, compare_text) == 0)
+	found = clist->data;
+
+      if (!combo->case_sensitive)
+	g_free (ltext);
     }
 
-  return NULL;
+  if (!combo->case_sensitive)
+    g_free (compare_text);
+
+  return found;
 }
 
 static gchar *
@@ -365,6 +383,8 @@ gtk_combo_entry_focus_out (GtkEntry * entry, GdkEventFocus * event, GtkCombo * c
 
   if (combo->value_in_list && !gtk_combo_find (combo))
     {
+      GSource *focus_idle;
+      
       /* gdk_beep(); *//* this can be annoying */
       if (combo->ok_if_empty && !strcmp (gtk_entry_get_text (entry), ""))
 	return FALSE;
@@ -377,7 +397,12 @@ gtk_combo_entry_focus_out (GtkEntry * entry, GdkEventFocus * event, GtkCombo * c
          so the focus can be lost anyway...
          the signal_emit_stop doesn't seem to work either...
        */
-      gtk_idle_add ((GtkFunction) gtk_combo_focus_idle, combo);
+      focus_idle = g_idle_source_new ();
+      g_source_set_closure (focus_idle,
+			    g_cclosure_new_object (G_CALLBACK (gtk_combo_focus_idle),
+						   G_OBJECT (combo)));
+      g_source_attach (focus_idle, NULL);
+      
       /*gtk_signal_emit_stop_by_name (GTK_OBJECT (entry), "focus_out_event"); */
       return TRUE;
     }
@@ -410,7 +435,7 @@ gtk_combo_get_pos (GtkCombo * combo, gint * x, gint * y, gint * height, gint * w
   real_height = MIN (combo->entry->requisition.height, 
 		     combo->entry->allocation.height);
   *y += real_height;
-  avail_height = gdk_screen_height () - *y;
+  avail_height = gdk_screen_get_height (gtk_widget_get_screen (widget)) - *y;
   
   gtk_widget_size_request (combo->list, &list_requisition);
   min_height = MIN (list_requisition.height, 
@@ -493,6 +518,27 @@ gtk_combo_popup_list (GtkCombo * combo)
   gtk_widget_grab_focus (combo->popwin);
 }
 
+static void
+gtk_combo_popdown_list (GtkCombo *combo)
+{
+  combo->current_button = 0;
+      
+  if (GTK_BUTTON (combo->button)->in_button)
+    {
+      GTK_BUTTON (combo->button)->in_button = FALSE;
+      gtk_button_released (GTK_BUTTON (combo->button));
+    }
+
+  if (GTK_WIDGET_HAS_GRAB (combo->popwin))
+    {
+      gtk_grab_remove (combo->popwin);
+      gdk_display_pointer_ungrab (gtk_widget_get_display (GTK_WIDGET (combo)),
+				  GDK_CURRENT_TIME);
+    }
+  
+  gtk_widget_hide (combo->popwin);
+}
+
 static void        
 gtk_combo_activate (GtkWidget        *widget,
 		    GtkCombo         *combo)
@@ -544,10 +590,12 @@ gtk_combo_popup_button_leave (GtkWidget        *button,
 			      GdkEventCrossing *event,
 			      GtkCombo         *combo)
 {
+  /* The idea here is that we want to keep the button down if the
+   * popup is popped up.
+   */
   return combo->current_button != 0;
 }
 
-
 static void
 gtk_combo_update_entry (GtkList * list, GtkCombo * combo)
 {
@@ -607,9 +655,7 @@ gtk_combo_button_press (GtkWidget * widget, GdkEvent * event, GtkCombo * combo)
 	}
     }
 
-  gtk_widget_hide (combo->popwin);
-  gtk_grab_remove (combo->popwin);
-  gdk_pointer_ungrab (event->button.time);
+  gtk_combo_popdown_list (combo);
 
   return TRUE;
 }
@@ -628,23 +674,8 @@ gtk_combo_button_event_after (GtkWidget *widget,
     {
       /* This was the initial button press */
 
-      GdkEventCrossing tmp_event;
-
       combo->current_button = 0;
 
-      if (widget != combo->button)
-	gtk_widget_event (combo->button, event);
-
-      /* Un-pre-highlight */
-      
-      tmp_event.type = GDK_LEAVE_NOTIFY;
-      tmp_event.window = combo->button->window;
-      tmp_event.send_event = TRUE;
-      tmp_event.subwindow = NULL;
-      tmp_event.detail = GDK_NOTIFY_ANCESTOR;
-      
-      gtk_widget_event (combo->button, (GdkEvent *)&tmp_event);
-
       /* Check to see if we released inside the button */
       child = gtk_get_event_widget ((GdkEvent*) event);
 
@@ -662,18 +693,8 @@ gtk_combo_button_event_after (GtkWidget *widget,
 	  return;
 	}
     }
-  else
-    {
-      /* The user has clicked inside the popwin and released */
 
-      if (GTK_WIDGET_HAS_GRAB (combo->popwin))
-	{
-	  gtk_grab_remove (combo->popwin);
-	  gdk_pointer_ungrab (event->button.time);
-	}
-    }
-  
-  gtk_widget_hide (combo->popwin);
+  gtk_combo_popdown_list (combo);
 }
 
 static gint         
@@ -723,33 +744,31 @@ gtk_combo_list_key_press (GtkWidget * widget, GdkEventKey * event, GtkCombo * co
 {
   if (event->keyval == GDK_Escape)
     {
-      if (GTK_WIDGET_HAS_GRAB (combo->popwin))
-	{
-	  gtk_grab_remove (combo->popwin);
-	  gdk_pointer_ungrab (GDK_CURRENT_TIME);
-	}
-      else if (GTK_WIDGET_HAS_GRAB (combo->list))
+      if (GTK_WIDGET_HAS_GRAB (combo->list))
 	gtk_list_end_drag_selection (GTK_LIST (combo->list));
-      gtk_widget_hide (combo->popwin);
-      if (GTK_WIDGET_HAS_GRAB (combo->button))
-	{
-	  combo->current_button = 0;
-	  GTK_BUTTON (combo->button)->in_button = FALSE;
-	  gtk_button_released (GTK_BUTTON (combo->button));
-	  gtk_grab_remove (combo->button);
-	}
+
+      gtk_combo_popdown_list (combo);
+      
       return TRUE;
     }
   return FALSE;
 }
 
+static void
+combo_event_box_realize (GtkWidget *widget)
+{
+  GdkCursor *cursor = gdk_cursor_new_for_display (gtk_widget_get_display (widget),
+						  GDK_TOP_LEFT_ARROW);
+  gdk_window_set_cursor (widget->window, cursor);
+  gdk_cursor_destroy (cursor);
+}
+
 static void
 gtk_combo_init (GtkCombo * combo)
 {
   GtkWidget *arrow;
   GtkWidget *frame;
   GtkWidget *event_box;
-  GdkCursor *cursor;
 
   combo->case_sensitive = FALSE;
   combo->value_in_list = FALSE;
@@ -791,12 +810,10 @@ gtk_combo_init (GtkCombo * combo)
 
   event_box = gtk_event_box_new ();
   gtk_container_add (GTK_CONTAINER (combo->popwin), event_box);
+  g_signal_connect (event_box, "realize",
+		    G_CALLBACK (combo_event_box_realize), NULL);
   gtk_widget_show (event_box);
 
-  gtk_widget_realize (event_box);
-  cursor = gdk_cursor_new (GDK_TOP_LEFT_ARROW);
-  gdk_window_set_cursor (event_box->window, cursor);
-  gdk_cursor_destroy (cursor);
 
   frame = gtk_frame_new (NULL);
   gtk_container_add (GTK_CONTAINER (event_box), frame);
@@ -842,6 +859,28 @@ gtk_combo_init (GtkCombo * combo)
 		      GTK_SIGNAL_FUNC (gtk_combo_list_enter), combo);
 }
 
+static void
+gtk_combo_realize (GtkWidget *widget)
+{
+  GtkCombo *combo = GTK_COMBO (widget);
+
+  gtk_window_set_screen (GTK_WINDOW (combo->popwin), 
+			 gtk_widget_get_screen (widget));
+  
+  GTK_WIDGET_CLASS( parent_class )->realize (widget);  
+}
+
+static void        
+gtk_combo_unrealize (GtkWidget *widget)
+{
+  GtkCombo *combo = GTK_COMBO (widget);
+
+  gtk_combo_popdown_list (combo);
+  gtk_widget_unrealize (combo->popwin);
+  
+  GTK_WIDGET_CLASS (parent_class)->unrealize (widget);
+}
+
 GtkType
 gtk_combo_get_type (void)
 {
@@ -948,6 +987,8 @@ gtk_combo_set_popdown_strings (GtkCombo * combo, GList * strings)
   g_return_if_fail (GTK_IS_COMBO (combo));
   g_return_if_fail (strings != NULL);
 
+  gtk_combo_popdown_list (combo);
+
   gtk_list_clear_items (GTK_LIST (combo->list), 0, -1);
   list = strings;
   while (list)
