@@ -119,6 +119,8 @@ static void    gtk_font_selection_get_property       (GObject         *object,
 						      GParamSpec      *pspec);
 static void    gtk_font_selection_init		     (GtkFontSelection      *fontsel);
 static void    gtk_font_selection_finalize	     (GObject               *object);
+static void    gtk_font_selection_hierarchy_changed  (GtkWidget		    *widget,
+						      GtkWidget             *previous_toplevel);
 
 /* These are the callbacks & related functions. */
 static void     gtk_font_selection_select_font           (GtkTreeSelection *selection,
@@ -136,6 +138,9 @@ static void     gtk_font_selection_show_available_sizes  (GtkFontSelection *fs,
 							  gboolean          first_time);
 static void     gtk_font_selection_size_activate         (GtkWidget        *w,
 							  gpointer          data);
+static gboolean gtk_font_selection_size_focus_out        (GtkWidget        *w,
+							  GdkEventFocus    *event,
+							  gpointer          data);
 static void     gtk_font_selection_select_size           (GtkTreeSelection *selection,
 							  gpointer          data);
 
@@ -190,31 +195,34 @@ static void
 gtk_font_selection_class_init (GtkFontSelectionClass *klass)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
+  GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
   
   font_selection_parent_class = gtk_type_class (GTK_TYPE_VBOX);
   
   gobject_class->set_property = gtk_font_selection_set_property;
   gobject_class->get_property = gtk_font_selection_get_property;
+
+  widget_class->hierarchy_changed = gtk_font_selection_hierarchy_changed;
    
   g_object_class_install_property (gobject_class,
                                    PROP_FONT_NAME,
                                    g_param_spec_string ("font_name",
                                                         _("Font name"),
-                                                        _("The X string that represents this font."),
+                                                        _("The X string that represents this font"),
                                                         NULL,
                                                         G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class,
 				   PROP_FONT,
 				   g_param_spec_boxed ("font",
 						       _("Font"),
-						       _("The GdkFont that is currently selected."),
+						       _("The GdkFont that is currently selected"),
 						       GDK_TYPE_FONT,
 						       G_PARAM_READABLE));
   g_object_class_install_property (gobject_class,
                                    PROP_PREVIEW_TEXT,
                                    g_param_spec_string ("preview_text",
                                                         _("Preview text"),
-                                                        _("The text to display in order to demonstrate the selected font."),
+                                                        _("The text to display in order to demonstrate the selected font"),
                                                         PREVIEW_TEXT,
                                                         G_PARAM_READWRITE));
   gobject_class->finalize = gtk_font_selection_finalize;
@@ -317,6 +325,9 @@ gtk_font_selection_init (GtkFontSelection *fontsel)
   gtk_signal_connect (GTK_OBJECT (fontsel->size_entry), "activate",
 		      (GtkSignalFunc) gtk_font_selection_size_activate,
 		      fontsel);
+  gtk_signal_connect_after (GTK_OBJECT (fontsel->size_entry), "focus_out_event",
+			    (GtkSignalFunc) gtk_font_selection_size_focus_out,
+			    fontsel);
   
   font_label = gtk_label_new_with_mnemonic (_("_Family:"));
   gtk_misc_set_alignment (GTK_MISC (font_label), 0.0, 0.5);
@@ -439,8 +450,6 @@ gtk_font_selection_init (GtkFontSelection *fontsel)
   g_list_free (focus_chain);
   
   /* Insert the fonts. */
-  gtk_font_selection_show_available_fonts (fontsel);
-  
   g_signal_connect (gtk_tree_view_get_selection (GTK_TREE_VIEW (fontsel->family_list)), "changed",
 		    G_CALLBACK (gtk_font_selection_select_font), fontsel);
 
@@ -448,13 +457,9 @@ gtk_font_selection_init (GtkFontSelection *fontsel)
 			    GTK_SIGNAL_FUNC (gtk_font_selection_scroll_on_map),
 			    fontsel);
   
-  gtk_font_selection_show_available_styles (fontsel);
-  
   g_signal_connect (gtk_tree_view_get_selection (GTK_TREE_VIEW (fontsel->face_list)), "changed",
 		    G_CALLBACK (gtk_font_selection_select_style), fontsel);
 
-  gtk_font_selection_show_available_sizes (fontsel, TRUE);
-  
   g_signal_connect (gtk_tree_view_get_selection (GTK_TREE_VIEW (fontsel->size_list)), "changed",
 		    G_CALLBACK (gtk_font_selection_select_size), fontsel);
 
@@ -487,8 +492,6 @@ gtk_font_selection_init (GtkFontSelection *fontsel)
   gtk_box_pack_start (GTK_BOX (text_box), fontsel->preview_entry,
 		      TRUE, TRUE, 0);
 
-  gtk_font_selection_update_preview (fontsel);
-
   gtk_widget_pop_composite_child();
 }
 
@@ -518,6 +521,63 @@ gtk_font_selection_finalize (GObject *object)
     (* G_OBJECT_CLASS (font_selection_parent_class)->finalize) (object);
 }
 
+static void
+fontsel_screen_changed (GtkFontSelection *fontsel)
+{
+  GdkScreen *old_screen = g_object_get_data (G_OBJECT (fontsel), "gtk-font-selection-screen");
+  GdkScreen *screen;
+
+  if (gtk_widget_has_screen (GTK_WIDGET (fontsel)))
+    screen = gtk_widget_get_screen (GTK_WIDGET (fontsel));
+  else
+    screen = NULL;
+
+  if (screen == old_screen)
+    return;
+
+  if (fontsel->font)
+    {
+      gdk_font_unref (fontsel->font);
+      fontsel->font = NULL;
+    }
+
+  if (old_screen)
+    g_object_unref (old_screen);
+  
+  if (screen)
+    {
+      g_object_ref (screen);
+      g_object_set_data (G_OBJECT (fontsel), "gtk-font-selection-screen", screen);
+
+      gtk_font_selection_show_available_fonts (fontsel);
+      gtk_font_selection_show_available_sizes (fontsel, TRUE);
+      gtk_font_selection_show_available_styles (fontsel);
+    }
+  else
+    g_object_set_data (G_OBJECT (fontsel), "gtk-font-selection-screen", NULL);
+}
+
+static void
+gtk_font_selection_hierarchy_changed (GtkWidget *widget,
+				      GtkWidget *previous_toplevel)
+{
+  GtkWidget *toplevel;
+  
+  if (previous_toplevel)
+    g_signal_handlers_disconnect_by_func (previous_toplevel,
+					  (gpointer) fontsel_screen_changed,
+					  widget);
+  
+  toplevel = gtk_widget_get_toplevel (widget);
+  if (GTK_WIDGET_TOPLEVEL (toplevel))
+    g_signal_connect_swapped (toplevel,
+			      "notify::screen",
+			      G_CALLBACK (fontsel_screen_changed),
+			      widget);
+  
+  fontsel_screen_changed (GTK_FONT_SELECTION (widget));
+}
+
 static void
 gtk_font_selection_preview_changed (GtkWidget        *entry,
 				    GtkFontSelection *fontsel)
@@ -616,7 +676,7 @@ cmp_families (const void *a, const void *b)
   const char *a_name = pango_font_family_get_name (*(PangoFontFamily **)a);
   const char *b_name = pango_font_family_get_name (*(PangoFontFamily **)b);
   
-  return strcmp (a_name, b_name);
+  return g_utf8_collate (a_name, b_name);
 }
 
 static void
@@ -941,6 +1001,16 @@ gtk_font_selection_size_activate (GtkWidget   *w,
   gtk_font_selection_set_size (fontsel, new_size);
 }
 
+static gboolean
+gtk_font_selection_size_focus_out (GtkWidget     *w,
+				   GdkEventFocus *event,
+				   gpointer       data)
+{
+  gtk_font_selection_size_activate (w, data);
+  
+  return TRUE;
+}
+
 /* This is called when a size is selected in the list. */
 static void
 gtk_font_selection_select_size (GtkTreeSelection *selection,
@@ -1027,7 +1097,7 @@ gtk_font_selection_get_font (GtkFontSelection *fontsel)
   if (!fontsel->font)
     {
       PangoFontDescription *font_desc = gtk_font_selection_get_font_description (fontsel);
-      fontsel->font = gdk_font_from_description (font_desc);
+      fontsel->font = gdk_font_from_description_for_display (gtk_widget_get_display (GTK_WIDGET (fontsel)), font_desc);
       pango_font_description_free (font_desc);
     }
   
