@@ -182,7 +182,7 @@ gtk_handle_box_class_init (GtkHandleBoxClass *class)
   g_object_class_install_property (gobject_class,
                                    PROP_SHADOW,
                                    g_param_spec_enum ("shadow", NULL,
-                                                      _("Deprecated property, use shadow_type instead."),
+                                                      _("Deprecated property, use shadow_type instead"),
 						      GTK_TYPE_SHADOW_TYPE,
 						      GTK_SHADOW_ETCHED_OUT,
                                                       G_PARAM_READABLE | G_PARAM_WRITABLE));
@@ -190,7 +190,7 @@ gtk_handle_box_class_init (GtkHandleBoxClass *class)
                                    PROP_SHADOW_TYPE,
                                    g_param_spec_enum ("shadow_type",
                                                       _("Shadow type"),
-                                                      _("Appearance of the shadow that surrounds the container."),
+                                                      _("Appearance of the shadow that surrounds the container"),
 						      GTK_TYPE_SHADOW_TYPE,
 						      GTK_SHADOW_ETCHED_OUT,
                                                       G_PARAM_READABLE | G_PARAM_WRITABLE));
@@ -199,7 +199,7 @@ gtk_handle_box_class_init (GtkHandleBoxClass *class)
                                    PROP_HANDLE_POSITION,
                                    g_param_spec_enum ("handle_position",
                                                       _("Handle position"),
-                                                      _("Position of the handle relative to the child widget."),
+                                                      _("Position of the handle relative to the child widget"),
 						      GTK_TYPE_POSITION_TYPE,
 						      GTK_POS_LEFT,
                                                       G_PARAM_READABLE | G_PARAM_WRITABLE));
@@ -208,7 +208,7 @@ gtk_handle_box_class_init (GtkHandleBoxClass *class)
                                    PROP_SNAP_EDGE,
                                    g_param_spec_enum ("snap_edge",
                                                       _("Snap edge"),
-                                                      _("Side of the handlebox that's lined up with the docking point to dock the handlebox."),
+                                                      _("Side of the handlebox that's lined up with the docking point to dock the handlebox"),
 						      GTK_TYPE_POSITION_TYPE,
 						      GTK_POS_TOP,
                                                       G_PARAM_READABLE | G_PARAM_WRITABLE));
@@ -328,12 +328,6 @@ gtk_handle_box_new (void)
 static void
 gtk_handle_box_destroy (GtkObject *object)
 {
-  GtkHandleBox *hb;
-
-  g_return_if_fail (GTK_IS_HANDLE_BOX (object));
-
-  hb = GTK_HANDLE_BOX (object);
-
   if (GTK_OBJECT_CLASS (parent_class)->destroy)
     (* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
 }
@@ -344,8 +338,6 @@ gtk_handle_box_map (GtkWidget *widget)
   GtkBin *bin;
   GtkHandleBox *hb;
 
-  g_return_if_fail (GTK_IS_HANDLE_BOX (widget));
-
   GTK_WIDGET_SET_FLAGS (widget, GTK_MAPPED);
 
   bin = GTK_BIN (widget);
@@ -371,8 +363,6 @@ gtk_handle_box_unmap (GtkWidget *widget)
 {
   GtkHandleBox *hb;
 
-  g_return_if_fail (GTK_IS_HANDLE_BOX (widget));
-
   GTK_WIDGET_UNSET_FLAGS (widget, GTK_MAPPED);
 
   hb = GTK_HANDLE_BOX (widget);
@@ -392,8 +382,6 @@ gtk_handle_box_realize (GtkWidget *widget)
   gint attributes_mask;
   GtkHandleBox *hb;
 
-  g_return_if_fail (GTK_IS_HANDLE_BOX (widget));
-
   hb = GTK_HANDLE_BOX (widget);
 
   GTK_WIDGET_SET_FLAGS (widget, GTK_REALIZED);
@@ -417,11 +405,11 @@ gtk_handle_box_realize (GtkWidget *widget)
   attributes.width = widget->allocation.width;
   attributes.height = widget->allocation.height;
   attributes.window_type = GDK_WINDOW_CHILD;
-  attributes.event_mask |= (gtk_widget_get_events (widget) |
-			    GDK_EXPOSURE_MASK |
-			    GDK_BUTTON1_MOTION_MASK |
-			    GDK_POINTER_MOTION_HINT_MASK |
-			    GDK_BUTTON_PRESS_MASK |
+  attributes.event_mask = (gtk_widget_get_events (widget) |
+			   GDK_EXPOSURE_MASK |
+			   GDK_BUTTON1_MOTION_MASK |
+			   GDK_POINTER_MOTION_HINT_MASK |
+			   GDK_BUTTON_PRESS_MASK |
 			    GDK_BUTTON_RELEASE_MASK);
   attributes_mask = GDK_WA_X | GDK_WA_Y | GDK_WA_VISUAL | GDK_WA_COLORMAP;
   hb->bin_window = gdk_window_new (widget->window, &attributes, attributes_mask);
@@ -444,7 +432,8 @@ gtk_handle_box_realize (GtkWidget *widget)
 			   GDK_FOCUS_CHANGE_MASK |
 			   GDK_STRUCTURE_MASK);
   attributes_mask = GDK_WA_X | GDK_WA_Y | GDK_WA_VISUAL | GDK_WA_COLORMAP;
-  hb->float_window = gdk_window_new (NULL, &attributes, attributes_mask);
+  hb->float_window = gdk_window_new (gtk_widget_get_root_window (widget),
+				     &attributes, attributes_mask);
   gdk_window_set_user_data (hb->float_window, widget);
   gdk_window_set_decorations (hb->float_window, 0);
   gdk_window_set_type_hint (hb->float_window, GDK_WINDOW_TYPE_HINT_TOOLBAR);
@@ -459,11 +448,7 @@ gtk_handle_box_realize (GtkWidget *widget)
 static void
 gtk_handle_box_unrealize (GtkWidget *widget)
 {
-  GtkHandleBox *hb;
-
-  g_return_if_fail (GTK_IS_HANDLE_BOX (widget));
-
-  hb = GTK_HANDLE_BOX (widget);
+  GtkHandleBox *hb = GTK_HANDLE_BOX (widget);
 
   gdk_window_set_user_data (hb->bin_window, NULL);
   gdk_window_destroy (hb->bin_window);
@@ -480,17 +465,13 @@ static void
 gtk_handle_box_style_set (GtkWidget *widget,
 			  GtkStyle  *previous_style)
 {
-  GtkHandleBox *hb;
-
-  g_return_if_fail (GTK_IS_HANDLE_BOX (widget));
-
-  hb = GTK_HANDLE_BOX (widget);
+  GtkHandleBox *hb = GTK_HANDLE_BOX (widget);
 
   if (GTK_WIDGET_REALIZED (widget) &&
       !GTK_WIDGET_NO_WINDOW (widget))
     {
       gtk_style_set_background (widget->style, widget->window,
-widget->state);
+				widget->state);
       gtk_style_set_background (widget->style, hb->bin_window, widget->state);
       gtk_style_set_background (widget->style, hb->float_window, widget->state);
     }
@@ -504,9 +485,6 @@ gtk_handle_box_size_request (GtkWidget      *widget,
   GtkHandleBox *hb;
   GtkRequisition child_requisition;
 
-  g_return_if_fail (GTK_IS_HANDLE_BOX (widget));
-  g_return_if_fail (requisition != NULL);
-
   bin = GTK_BIN (widget);
   hb = GTK_HANDLE_BOX (widget);
 
@@ -579,9 +557,6 @@ gtk_handle_box_size_allocate (GtkWidget     *widget,
   GtkHandleBox *hb;
   GtkRequisition child_requisition;
   
-  g_return_if_fail (GTK_IS_HANDLE_BOX (widget));
-  g_return_if_fail (allocation != NULL);
-  
   bin = GTK_BIN (widget);
   hb = GTK_HANDLE_BOX (widget);
   
@@ -770,6 +745,8 @@ void
 gtk_handle_box_set_handle_position  (GtkHandleBox    *handle_box,
 				     GtkPositionType  position)
 {
+  g_return_if_fail (GTK_IS_HANDLE_BOX (handle_box));
+
   if ((GtkPositionType) handle_box->handle_position != position)
     {
       handle_box->handle_position = position;
@@ -919,9 +896,6 @@ gtk_handle_box_expose (GtkWidget      *widget,
 {
   GtkHandleBox *hb;
 
-  g_return_val_if_fail (GTK_IS_HANDLE_BOX (widget), FALSE);
-  g_return_val_if_fail (event != NULL, FALSE);
-
   if (GTK_WIDGET_DRAWABLE (widget))
     {
       hb = GTK_HANDLE_BOX (widget);
@@ -946,9 +920,6 @@ gtk_handle_box_button_changed (GtkWidget      *widget,
   gboolean event_handled;
   GdkCursor *fleur;
 
-  g_return_val_if_fail (GTK_IS_HANDLE_BOX (widget), FALSE);
-  g_return_val_if_fail (event != NULL, FALSE);
-
   hb = GTK_HANDLE_BOX (widget);
 
   event_handled = FALSE;
@@ -1019,7 +990,8 @@ gtk_handle_box_button_changed (GtkWidget      *widget,
 	      hb->attach_allocation.height = height;
 
 	      hb->in_drag = TRUE;
-	      fleur = gdk_cursor_new (GDK_FLEUR);
+	      fleur = gdk_cursor_new_for_display (gtk_widget_get_display (widget),
+						  GDK_FLEUR);
 	      if (gdk_pointer_grab (widget->window,
 				    FALSE,
 				    (GDK_BUTTON1_MOTION_MASK |
@@ -1047,7 +1019,8 @@ gtk_handle_box_button_changed (GtkWidget      *widget,
       if (event->window != widget->window)
 	return FALSE;
       
-      gdk_pointer_ungrab (GDK_CURRENT_TIME);
+      gdk_display_pointer_ungrab (gtk_widget_get_display (widget),
+				  GDK_CURRENT_TIME);
       hb->in_drag = FALSE;
       event_handled = TRUE;
     }
@@ -1064,9 +1037,6 @@ gtk_handle_box_motion (GtkWidget      *widget,
   gint snap_edge;
   gboolean is_snapped = FALSE;
 
-  g_return_val_if_fail (GTK_IS_HANDLE_BOX (widget), FALSE);
-  g_return_val_if_fail (event != NULL, FALSE);
-
   hb = GTK_HANDLE_BOX (widget);
   if (!hb->in_drag)
     return FALSE;
@@ -1079,7 +1049,8 @@ gtk_handle_box_motion (GtkWidget      *widget,
    */
   new_x = 0;
   new_y = 0;
-  gdk_window_get_pointer (NULL, &new_x, &new_y, NULL);
+  gdk_window_get_pointer (gtk_widget_get_root_window (widget), 
+			  &new_x, &new_y, NULL);
   new_x += hb->float_allocation.x;
   new_y += hb->float_allocation.y;
 
@@ -1222,9 +1193,9 @@ gtk_handle_box_motion (GtkWidget      *widget,
 	  /* this extra move is neccessary if we use decorations, or our
 	   * window manager insists on decorations.
 	   */
-	  gdk_flush ();
+	  gdk_display_sync (gtk_widget_get_display (widget));
 	  gdk_window_move (hb->float_window, new_x, new_y);
-	  gdk_flush ();
+	  gdk_display_sync (gtk_widget_get_display (widget));
 #endif	/* 0 */
 	  gtk_signal_emit (GTK_OBJECT (hb),
 			   handle_box_signals[SIGNAL_CHILD_DETACHED],
@@ -1242,10 +1213,6 @@ static void
 gtk_handle_box_add (GtkContainer *container,
 		    GtkWidget    *widget)
 {
-  g_return_if_fail (GTK_IS_HANDLE_BOX (container));
-  g_return_if_fail (GTK_BIN (container)->child == NULL);
-  g_return_if_fail (widget->parent == NULL);
-
   gtk_widget_set_parent_window (widget, GTK_HANDLE_BOX (container)->bin_window);
   GTK_CONTAINER_CLASS (parent_class)->add (container, widget);
 }
@@ -1254,9 +1221,6 @@ static void
 gtk_handle_box_remove (GtkContainer *container,
 		       GtkWidget    *widget)
 {
-  g_return_if_fail (GTK_IS_HANDLE_BOX (container));
-  g_return_if_fail (GTK_BIN (container)->child == widget);
-
   GTK_CONTAINER_CLASS (parent_class)->remove (container, widget);
 
   gtk_handle_box_reattach (GTK_HANDLE_BOX (container));
@@ -1266,12 +1230,7 @@ static gint
 gtk_handle_box_delete_event (GtkWidget *widget,
 			     GdkEventAny  *event)
 {
-  GtkHandleBox *hb;
-
-  g_return_val_if_fail (GTK_IS_HANDLE_BOX (widget), FALSE);
-  g_return_val_if_fail (event != NULL, FALSE);
-
-  hb = GTK_HANDLE_BOX (widget);
+  GtkHandleBox *hb = GTK_HANDLE_BOX (widget);
 
   if (event->window == hb->float_window)
     {
@@ -1286,13 +1245,15 @@ gtk_handle_box_delete_event (GtkWidget *widget,
 static void
 gtk_handle_box_reattach (GtkHandleBox *hb)
 {
+  GtkWidget *widget = GTK_WIDGET (hb);
+  
   if (hb->child_detached)
     {
       hb->child_detached = FALSE;
       if (GTK_WIDGET_REALIZED (hb))
 	{
 	  gdk_window_hide (hb->float_window);
-	  gdk_window_reparent (hb->bin_window, GTK_WIDGET (hb)->window, 0, 0);
+	  gdk_window_reparent (hb->bin_window, widget->window, 0, 0);
 
 	  if (GTK_BIN (hb)->child)
 	    gtk_signal_emit (GTK_OBJECT (hb),
@@ -1304,7 +1265,8 @@ gtk_handle_box_reattach (GtkHandleBox *hb)
     }
   if (hb->in_drag)
     {
-      gdk_pointer_ungrab (GDK_CURRENT_TIME);
+      gdk_display_pointer_ungrab (gtk_widget_get_display (GTK_WIDGET (hb)),
+				  GDK_CURRENT_TIME);
       hb->in_drag = FALSE;
     }
 
