@@ -60,6 +60,12 @@ struct _GtkListStore
 struct _GtkListStoreClass
 {
   GObjectClass parent_class;
+
+  /* Padding for future expansion */
+  void (*_gtk_reserved1) (void);
+  void (*_gtk_reserved2) (void);
+  void (*_gtk_reserved3) (void);
+  void (*_gtk_reserved4) (void);
 };
 
 
@@ -71,6 +77,9 @@ GtkListStore *gtk_list_store_newv             (gint          n_columns,
 void          gtk_list_store_set_column_types (GtkListStore *list_store,
 					       gint          n_columns,
 					       GType        *types);
+
+/* NOTE: use gtk_tree_model_get to get values from a GtkListStore */
+
 void          gtk_list_store_set_value        (GtkListStore *list_store,
 					       GtkTreeIter  *iter,
 					       gint          column,
@@ -81,7 +90,7 @@ void          gtk_list_store_set              (GtkListStore *list_store,
 void          gtk_list_store_set_valist       (GtkListStore *list_store,
 					       GtkTreeIter  *iter,
 					       va_list       var_args);
-void          gtk_list_store_remove           (GtkListStore *list_store,
+gboolean      gtk_list_store_remove           (GtkListStore *list_store,
 					       GtkTreeIter  *iter);
 void          gtk_list_store_insert           (GtkListStore *list_store,
 					       GtkTreeIter  *iter,
@@ -97,6 +106,16 @@ void          gtk_list_store_prepend          (GtkListStore *list_store,
 void          gtk_list_store_append           (GtkListStore *list_store,
 					       GtkTreeIter  *iter);
 void          gtk_list_store_clear            (GtkListStore *list_store);
+gboolean      gtk_list_store_iter_is_valid    (GtkListStore *list_store,
+                                               GtkTreeIter  *iter);
+void          gtk_list_store_reorder          (GtkListStore *store,
+                                               gint         *new_order);
+void          gtk_list_store_swap             (GtkListStore *store,
+                                               GtkTreeIter  *a,
+                                               GtkTreeIter  *b);
+void          gtk_list_store_move             (GtkListStore *store,
+                                               GtkTreeIter  *iter,
+                                               GtkTreePath  *position);
 
 
 #ifdef __cplusplus
