@@ -30,15 +30,23 @@ extern void _gtk_marshal_BOOLEAN__ENUM (GClosure     *closure,
                                         gpointer      invocation_hint,
                                         gpointer      marshal_data);
 
-/* BOOLEAN:OBJECT,UINT,UINT (gtkmarshalers.list:27) */
-extern void _gtk_marshal_BOOLEAN__OBJECT_UINT_UINT (GClosure     *closure,
-                                                    GValue       *return_value,
-                                                    guint         n_param_values,
-                                                    const GValue *param_values,
-                                                    gpointer      invocation_hint,
-                                                    gpointer      marshal_data);
+/* BOOLEAN:ENUM,INT (gtkmarshalers.list:27) */
+extern void _gtk_marshal_BOOLEAN__ENUM_INT (GClosure     *closure,
+                                            GValue       *return_value,
+                                            guint         n_param_values,
+                                            const GValue *param_values,
+                                            gpointer      invocation_hint,
+                                            gpointer      marshal_data);
 
-/* BOOLEAN:OBJECT,INT,INT,UINT (gtkmarshalers.list:28) */
+/* BOOLEAN:OBJECT,UINT,FLAGS (gtkmarshalers.list:28) */
+extern void _gtk_marshal_BOOLEAN__OBJECT_UINT_FLAGS (GClosure     *closure,
+                                                     GValue       *return_value,
+                                                     guint         n_param_values,
+                                                     const GValue *param_values,
+                                                     gpointer      invocation_hint,
+                                                     gpointer      marshal_data);
+
+/* BOOLEAN:OBJECT,INT,INT,UINT (gtkmarshalers.list:29) */
 extern void _gtk_marshal_BOOLEAN__OBJECT_INT_INT_UINT (GClosure     *closure,
                                                        GValue       *return_value,
                                                        guint         n_param_values,
@@ -46,7 +54,7 @@ extern void _gtk_marshal_BOOLEAN__OBJECT_INT_INT_UINT (GClosure     *closure,
                                                        gpointer      invocation_hint,
                                                        gpointer      marshal_data);
 
-/* BOOLEAN:OBJECT,STRING,STRING,BOXED (gtkmarshalers.list:29) */
+/* BOOLEAN:OBJECT,STRING,STRING,BOXED (gtkmarshalers.list:30) */
 extern void _gtk_marshal_BOOLEAN__OBJECT_STRING_STRING_BOXED (GClosure     *closure,
                                                               GValue       *return_value,
                                                               guint         n_param_values,
@@ -54,7 +62,7 @@ extern void _gtk_marshal_BOOLEAN__OBJECT_STRING_STRING_BOXED (GClosure     *clos
                                                               gpointer      invocation_hint,
                                                               gpointer      marshal_data);
 
-/* BOOLEAN:OBJECT,BOXED,BOXED (gtkmarshalers.list:30) */
+/* BOOLEAN:OBJECT,BOXED,BOXED (gtkmarshalers.list:31) */
 extern void _gtk_marshal_BOOLEAN__OBJECT_BOXED_BOXED (GClosure     *closure,
                                                       GValue       *return_value,
                                                       guint         n_param_values,
@@ -62,7 +70,7 @@ extern void _gtk_marshal_BOOLEAN__OBJECT_BOXED_BOXED (GClosure     *closure,
                                                       gpointer      invocation_hint,
                                                       gpointer      marshal_data);
 
-/* BOOLEAN:INT,INT (gtkmarshalers.list:31) */
+/* BOOLEAN:INT,INT (gtkmarshalers.list:32) */
 extern void _gtk_marshal_BOOLEAN__INT_INT (GClosure     *closure,
                                            GValue       *return_value,
                                            guint         n_param_values,
@@ -70,7 +78,7 @@ extern void _gtk_marshal_BOOLEAN__INT_INT (GClosure     *closure,
                                            gpointer      invocation_hint,
                                            gpointer      marshal_data);
 
-/* BOOLEAN:VOID (gtkmarshalers.list:32) */
+/* BOOLEAN:VOID (gtkmarshalers.list:33) */
 extern void _gtk_marshal_BOOLEAN__VOID (GClosure     *closure,
                                         GValue       *return_value,
                                         guint         n_param_values,
@@ -78,7 +86,7 @@ extern void _gtk_marshal_BOOLEAN__VOID (GClosure     *closure,
                                         gpointer      invocation_hint,
                                         gpointer      marshal_data);
 
-/* BOOLEAN:BOOLEAN (gtkmarshalers.list:33) */
+/* BOOLEAN:BOOLEAN (gtkmarshalers.list:34) */
 extern void _gtk_marshal_BOOLEAN__BOOLEAN (GClosure     *closure,
                                            GValue       *return_value,
                                            guint         n_param_values,
@@ -86,7 +94,7 @@ extern void _gtk_marshal_BOOLEAN__BOOLEAN (GClosure     *closure,
                                            gpointer      invocation_hint,
                                            gpointer      marshal_data);
 
-/* ENUM:ENUM (gtkmarshalers.list:34) */
+/* ENUM:ENUM (gtkmarshalers.list:35) */
 extern void _gtk_marshal_ENUM__ENUM (GClosure     *closure,
                                      GValue       *return_value,
                                      guint         n_param_values,
@@ -94,7 +102,7 @@ extern void _gtk_marshal_ENUM__ENUM (GClosure     *closure,
                                      gpointer      invocation_hint,
                                      gpointer      marshal_data);
 
-/* INT:POINTER (gtkmarshalers.list:35) */
+/* INT:POINTER (gtkmarshalers.list:36) */
 extern void _gtk_marshal_INT__POINTER (GClosure     *closure,
                                        GValue       *return_value,
                                        guint         n_param_values,
@@ -102,19 +110,19 @@ extern void _gtk_marshal_INT__POINTER (GClosure     *closure,
                                        gpointer      invocation_hint,
                                        gpointer      marshal_data);
 
-/* NONE:BOOLEAN (gtkmarshalers.list:36) */
+/* NONE:BOOLEAN (gtkmarshalers.list:37) */
 #define _gtk_marshal_VOID__BOOLEAN	g_cclosure_marshal_VOID__BOOLEAN
 #define _gtk_marshal_NONE__BOOLEAN	_gtk_marshal_VOID__BOOLEAN
 
-/* NONE:ENUM (gtkmarshalers.list:37) */
+/* NONE:ENUM (gtkmarshalers.list:38) */
 #define _gtk_marshal_VOID__ENUM	g_cclosure_marshal_VOID__ENUM
 #define _gtk_marshal_NONE__ENUM	_gtk_marshal_VOID__ENUM
 
-/* NONE:INT (gtkmarshalers.list:38) */
+/* NONE:INT (gtkmarshalers.list:39) */
 #define _gtk_marshal_VOID__INT	g_cclosure_marshal_VOID__INT
 #define _gtk_marshal_NONE__INT	_gtk_marshal_VOID__INT
 
-/* NONE:INT,BOOL (gtkmarshalers.list:39) */
+/* NONE:INT,BOOL (gtkmarshalers.list:40) */
 extern void _gtk_marshal_VOID__INT_BOOLEAN (GClosure     *closure,
                                             GValue       *return_value,
                                             guint         n_param_values,
@@ -123,7 +131,7 @@ extern void _gtk_marshal_VOID__INT_BOOLEAN (GClosure     *closure,
                                             gpointer      marshal_data);
 #define _gtk_marshal_NONE__INT_BOOL	_gtk_marshal_VOID__INT_BOOLEAN
 
-/* NONE:INT,INT (gtkmarshalers.list:40) */
+/* NONE:INT,INT (gtkmarshalers.list:41) */
 extern void _gtk_marshal_VOID__INT_INT (GClosure     *closure,
                                         GValue       *return_value,
                                         guint         n_param_values,
@@ -132,11 +140,11 @@ extern void _gtk_marshal_VOID__INT_INT (GClosure     *closure,
                                         gpointer      marshal_data);
 #define _gtk_marshal_NONE__INT_INT	_gtk_marshal_VOID__INT_INT
 
-/* NONE:NONE (gtkmarshalers.list:41) */
+/* NONE:NONE (gtkmarshalers.list:42) */
 #define _gtk_marshal_VOID__VOID	g_cclosure_marshal_VOID__VOID
 #define _gtk_marshal_NONE__NONE	_gtk_marshal_VOID__VOID
 
-/* NONE:STRING,INT,POINTER (gtkmarshalers.list:42) */
+/* NONE:STRING,INT,POINTER (gtkmarshalers.list:43) */
 extern void _gtk_marshal_VOID__STRING_INT_POINTER (GClosure     *closure,
                                                    GValue       *return_value,
                                                    guint         n_param_values,
@@ -145,7 +153,7 @@ extern void _gtk_marshal_VOID__STRING_INT_POINTER (GClosure     *closure,
                                                    gpointer      marshal_data);
 #define _gtk_marshal_NONE__STRING_INT_POINTER	_gtk_marshal_VOID__STRING_INT_POINTER
 
-/* STRING:DOUBLE (gtkmarshalers.list:43) */
+/* STRING:DOUBLE (gtkmarshalers.list:44) */
 extern void _gtk_marshal_STRING__DOUBLE (GClosure     *closure,
                                          GValue       *return_value,
                                          guint         n_param_values,
@@ -153,12 +161,12 @@ extern void _gtk_marshal_STRING__DOUBLE (GClosure     *closure,
                                          gpointer      invocation_hint,
                                          gpointer      marshal_data);
 
-/* VOID:DOUBLE (gtkmarshalers.list:44) */
+/* VOID:DOUBLE (gtkmarshalers.list:45) */
 #define _gtk_marshal_VOID__DOUBLE	g_cclosure_marshal_VOID__DOUBLE
 
-/* VOID:BOOLEAN (gtkmarshalers.list:45) */
+/* VOID:BOOLEAN (gtkmarshalers.list:46) */
 
-/* VOID:BOOLEAN,BOOLEAN,BOOLEAN (gtkmarshalers.list:46) */
+/* VOID:BOOLEAN,BOOLEAN,BOOLEAN (gtkmarshalers.list:47) */
 extern void _gtk_marshal_VOID__BOOLEAN_BOOLEAN_BOOLEAN (GClosure     *closure,
                                                         GValue       *return_value,
                                                         guint         n_param_values,
@@ -166,10 +174,10 @@ extern void _gtk_marshal_VOID__BOOLEAN_BOOLEAN_BOOLEAN (GClosure     *closure,
                                                         gpointer      invocation_hint,
                                                         gpointer      marshal_data);
 
-/* VOID:BOXED (gtkmarshalers.list:47) */
+/* VOID:BOXED (gtkmarshalers.list:48) */
 #define _gtk_marshal_VOID__BOXED	g_cclosure_marshal_VOID__BOXED
 
-/* VOID:BOXED,BOXED (gtkmarshalers.list:48) */
+/* VOID:BOXED,BOXED (gtkmarshalers.list:49) */
 extern void _gtk_marshal_VOID__BOXED_BOXED (GClosure     *closure,
                                             GValue       *return_value,
                                             guint         n_param_values,
@@ -177,7 +185,7 @@ extern void _gtk_marshal_VOID__BOXED_BOXED (GClosure     *closure,
                                             gpointer      invocation_hint,
                                             gpointer      marshal_data);
 
-/* VOID:BOXED,BOXED,POINTER (gtkmarshalers.list:49) */
+/* VOID:BOXED,BOXED,POINTER (gtkmarshalers.list:50) */
 extern void _gtk_marshal_VOID__BOXED_BOXED_POINTER (GClosure     *closure,
                                                     GValue       *return_value,
                                                     guint         n_param_values,
@@ -185,7 +193,7 @@ extern void _gtk_marshal_VOID__BOXED_BOXED_POINTER (GClosure     *closure,
                                                     gpointer      invocation_hint,
                                                     gpointer      marshal_data);
 
-/* VOID:BOXED,OBJECT (gtkmarshalers.list:50) */
+/* VOID:BOXED,OBJECT (gtkmarshalers.list:51) */
 extern void _gtk_marshal_VOID__BOXED_OBJECT (GClosure     *closure,
                                              GValue       *return_value,
                                              guint         n_param_values,
@@ -193,7 +201,7 @@ extern void _gtk_marshal_VOID__BOXED_OBJECT (GClosure     *closure,
                                              gpointer      invocation_hint,
                                              gpointer      marshal_data);
 
-/* VOID:BOXED,STRING,INT (gtkmarshalers.list:51) */
+/* VOID:BOXED,STRING,INT (gtkmarshalers.list:52) */
 extern void _gtk_marshal_VOID__BOXED_STRING_INT (GClosure     *closure,
                                                  GValue       *return_value,
                                                  guint         n_param_values,
@@ -201,7 +209,7 @@ extern void _gtk_marshal_VOID__BOXED_STRING_INT (GClosure     *closure,
                                                  gpointer      invocation_hint,
                                                  gpointer      marshal_data);
 
-/* VOID:BOXED,UINT (gtkmarshalers.list:52) */
+/* VOID:BOXED,UINT (gtkmarshalers.list:53) */
 extern void _gtk_marshal_VOID__BOXED_UINT (GClosure     *closure,
                                            GValue       *return_value,
                                            guint         n_param_values,
@@ -209,7 +217,7 @@ extern void _gtk_marshal_VOID__BOXED_UINT (GClosure     *closure,
                                            gpointer      invocation_hint,
                                            gpointer      marshal_data);
 
-/* VOID:BOXED,UINT,FLAGS (gtkmarshalers.list:53) */
+/* VOID:BOXED,UINT,FLAGS (gtkmarshalers.list:54) */
 extern void _gtk_marshal_VOID__BOXED_UINT_FLAGS (GClosure     *closure,
                                                  GValue       *return_value,
                                                  guint         n_param_values,
@@ -217,7 +225,7 @@ extern void _gtk_marshal_VOID__BOXED_UINT_FLAGS (GClosure     *closure,
                                                  gpointer      invocation_hint,
                                                  gpointer      marshal_data);
 
-/* VOID:BOXED,UINT,UINT (gtkmarshalers.list:54) */
+/* VOID:BOXED,UINT,UINT (gtkmarshalers.list:55) */
 extern void _gtk_marshal_VOID__BOXED_UINT_UINT (GClosure     *closure,
                                                 GValue       *return_value,
                                                 guint         n_param_values,
@@ -225,9 +233,9 @@ extern void _gtk_marshal_VOID__BOXED_UINT_UINT (GClosure     *closure,
                                                 gpointer      invocation_hint,
                                                 gpointer      marshal_data);
 
-/* VOID:ENUM (gtkmarshalers.list:55) */
+/* VOID:ENUM (gtkmarshalers.list:56) */
 
-/* VOID:ENUM,BOOLEAN (gtkmarshalers.list:56) */
+/* VOID:ENUM,BOOLEAN (gtkmarshalers.list:57) */
 extern void _gtk_marshal_VOID__ENUM_BOOLEAN (GClosure     *closure,
                                              GValue       *return_value,
                                              guint         n_param_values,
@@ -235,7 +243,7 @@ extern void _gtk_marshal_VOID__ENUM_BOOLEAN (GClosure     *closure,
                                              gpointer      invocation_hint,
                                              gpointer      marshal_data);
 
-/* VOID:ENUM,ENUM (gtkmarshalers.list:57) */
+/* VOID:ENUM,ENUM (gtkmarshalers.list:58) */
 extern void _gtk_marshal_VOID__ENUM_ENUM (GClosure     *closure,
                                           GValue       *return_value,
                                           guint         n_param_values,
@@ -243,7 +251,7 @@ extern void _gtk_marshal_VOID__ENUM_ENUM (GClosure     *closure,
                                           gpointer      invocation_hint,
                                           gpointer      marshal_data);
 
-/* VOID:ENUM,FLOAT (gtkmarshalers.list:58) */
+/* VOID:ENUM,FLOAT (gtkmarshalers.list:59) */
 extern void _gtk_marshal_VOID__ENUM_FLOAT (GClosure     *closure,
                                            GValue       *return_value,
                                            guint         n_param_values,
@@ -251,7 +259,7 @@ extern void _gtk_marshal_VOID__ENUM_FLOAT (GClosure     *closure,
                                            gpointer      invocation_hint,
                                            gpointer      marshal_data);
 
-/* VOID:ENUM,FLOAT,BOOLEAN (gtkmarshalers.list:59) */
+/* VOID:ENUM,FLOAT,BOOLEAN (gtkmarshalers.list:60) */
 extern void _gtk_marshal_VOID__ENUM_FLOAT_BOOLEAN (GClosure     *closure,
                                                    GValue       *return_value,
                                                    guint         n_param_values,
@@ -259,7 +267,7 @@ extern void _gtk_marshal_VOID__ENUM_FLOAT_BOOLEAN (GClosure     *closure,
                                                    gpointer      invocation_hint,
                                                    gpointer      marshal_data);
 
-/* VOID:ENUM,INT (gtkmarshalers.list:60) */
+/* VOID:ENUM,INT (gtkmarshalers.list:61) */
 extern void _gtk_marshal_VOID__ENUM_INT (GClosure     *closure,
                                          GValue       *return_value,
                                          guint         n_param_values,
@@ -267,7 +275,7 @@ extern void _gtk_marshal_VOID__ENUM_INT (GClosure     *closure,
                                          gpointer      invocation_hint,
                                          gpointer      marshal_data);
 
-/* VOID:ENUM,INT,BOOLEAN (gtkmarshalers.list:61) */
+/* VOID:ENUM,INT,BOOLEAN (gtkmarshalers.list:62) */
 extern void _gtk_marshal_VOID__ENUM_INT_BOOLEAN (GClosure     *closure,
                                                  GValue       *return_value,
                                                  guint         n_param_values,
@@ -275,11 +283,11 @@ extern void _gtk_marshal_VOID__ENUM_INT_BOOLEAN (GClosure     *closure,
                                                  gpointer      invocation_hint,
                                                  gpointer      marshal_data);
 
-/* VOID:INT (gtkmarshalers.list:62) */
+/* VOID:INT (gtkmarshalers.list:63) */
 
-/* VOID:INT,INT (gtkmarshalers.list:63) */
+/* VOID:INT,INT (gtkmarshalers.list:64) */
 
-/* VOID:INT,INT,BOXED (gtkmarshalers.list:64) */
+/* VOID:INT,INT,BOXED (gtkmarshalers.list:65) */
 extern void _gtk_marshal_VOID__INT_INT_BOXED (GClosure     *closure,
                                               GValue       *return_value,
                                               guint         n_param_values,
@@ -287,7 +295,7 @@ extern void _gtk_marshal_VOID__INT_INT_BOXED (GClosure     *closure,
                                               gpointer      invocation_hint,
                                               gpointer      marshal_data);
 
-/* VOID:INT,INT,INT (gtkmarshalers.list:65) */
+/* VOID:INT,INT,INT (gtkmarshalers.list:66) */
 extern void _gtk_marshal_VOID__INT_INT_INT (GClosure     *closure,
                                             GValue       *return_value,
                                             guint         n_param_values,
@@ -295,10 +303,10 @@ extern void _gtk_marshal_VOID__INT_INT_INT (GClosure     *closure,
                                             gpointer      invocation_hint,
                                             gpointer      marshal_data);
 
-/* VOID:OBJECT (gtkmarshalers.list:66) */
+/* VOID:OBJECT (gtkmarshalers.list:67) */
 #define _gtk_marshal_VOID__OBJECT	g_cclosure_marshal_VOID__OBJECT
 
-/* VOID:OBJECT,BOOLEAN (gtkmarshalers.list:67) */
+/* VOID:OBJECT,BOOLEAN (gtkmarshalers.list:68) */
 extern void _gtk_marshal_VOID__OBJECT_BOOLEAN (GClosure     *closure,
                                                GValue       *return_value,
                                                guint         n_param_values,
@@ -306,7 +314,7 @@ extern void _gtk_marshal_VOID__OBJECT_BOOLEAN (GClosure     *closure,
                                                gpointer      invocation_hint,
                                                gpointer      marshal_data);
 
-/* VOID:OBJECT,BOXED,BOXED (gtkmarshalers.list:68) */
+/* VOID:OBJECT,BOXED,BOXED (gtkmarshalers.list:69) */
 extern void _gtk_marshal_VOID__OBJECT_BOXED_BOXED (GClosure     *closure,
                                                    GValue       *return_value,
                                                    guint         n_param_values,
@@ -314,7 +322,7 @@ extern void _gtk_marshal_VOID__OBJECT_BOXED_BOXED (GClosure     *closure,
                                                    gpointer      invocation_hint,
                                                    gpointer      marshal_data);
 
-/* VOID:OBJECT,BOXED,UINT,UINT (gtkmarshalers.list:69) */
+/* VOID:OBJECT,BOXED,UINT,UINT (gtkmarshalers.list:70) */
 extern void _gtk_marshal_VOID__OBJECT_BOXED_UINT_UINT (GClosure     *closure,
                                                        GValue       *return_value,
                                                        guint         n_param_values,
@@ -322,7 +330,7 @@ extern void _gtk_marshal_VOID__OBJECT_BOXED_UINT_UINT (GClosure     *closure,
                                                        gpointer      invocation_hint,
                                                        gpointer      marshal_data);
 
-/* VOID:OBJECT,INT,INT (gtkmarshalers.list:70) */
+/* VOID:OBJECT,INT,INT (gtkmarshalers.list:71) */
 extern void _gtk_marshal_VOID__OBJECT_INT_INT (GClosure     *closure,
                                                GValue       *return_value,
                                                guint         n_param_values,
@@ -330,7 +338,7 @@ extern void _gtk_marshal_VOID__OBJECT_INT_INT (GClosure     *closure,
                                                gpointer      invocation_hint,
                                                gpointer      marshal_data);
 
-/* VOID:OBJECT,INT,INT,BOXED,UINT,UINT (gtkmarshalers.list:71) */
+/* VOID:OBJECT,INT,INT,BOXED,UINT,UINT (gtkmarshalers.list:72) */
 extern void _gtk_marshal_VOID__OBJECT_INT_INT_BOXED_UINT_UINT (GClosure     *closure,
                                                                GValue       *return_value,
                                                                guint         n_param_values,
@@ -338,7 +346,7 @@ extern void _gtk_marshal_VOID__OBJECT_INT_INT_BOXED_UINT_UINT (GClosure     *clo
                                                                gpointer      invocation_hint,
                                                                gpointer      marshal_data);
 
-/* VOID:OBJECT,OBJECT (gtkmarshalers.list:72) */
+/* VOID:OBJECT,OBJECT (gtkmarshalers.list:73) */
 extern void _gtk_marshal_VOID__OBJECT_OBJECT (GClosure     *closure,
                                               GValue       *return_value,
                                               guint         n_param_values,
@@ -346,7 +354,7 @@ extern void _gtk_marshal_VOID__OBJECT_OBJECT (GClosure     *closure,
                                               gpointer      invocation_hint,
                                               gpointer      marshal_data);
 
-/* VOID:OBJECT,STRING,STRING (gtkmarshalers.list:73) */
+/* VOID:OBJECT,STRING,STRING (gtkmarshalers.list:74) */
 extern void _gtk_marshal_VOID__OBJECT_STRING_STRING (GClosure     *closure,
                                                      GValue       *return_value,
                                                      guint         n_param_values,
@@ -354,7 +362,7 @@ extern void _gtk_marshal_VOID__OBJECT_STRING_STRING (GClosure     *closure,
                                                      gpointer      invocation_hint,
                                                      gpointer      marshal_data);
 
-/* VOID:OBJECT,UINT (gtkmarshalers.list:74) */
+/* VOID:OBJECT,UINT (gtkmarshalers.list:75) */
 extern void _gtk_marshal_VOID__OBJECT_UINT (GClosure     *closure,
                                             GValue       *return_value,
                                             guint         n_param_values,
@@ -362,7 +370,7 @@ extern void _gtk_marshal_VOID__OBJECT_UINT (GClosure     *closure,
                                             gpointer      invocation_hint,
                                             gpointer      marshal_data);
 
-/* VOID:OBJECT,UINT,FLAGS (gtkmarshalers.list:75) */
+/* VOID:OBJECT,UINT,FLAGS (gtkmarshalers.list:76) */
 extern void _gtk_marshal_VOID__OBJECT_UINT_FLAGS (GClosure     *closure,
                                                   GValue       *return_value,
                                                   guint         n_param_values,
@@ -370,10 +378,10 @@ extern void _gtk_marshal_VOID__OBJECT_UINT_FLAGS (GClosure     *closure,
                                                   gpointer      invocation_hint,
                                                   gpointer      marshal_data);
 
-/* VOID:POINTER (gtkmarshalers.list:76) */
+/* VOID:POINTER (gtkmarshalers.list:77) */
 #define _gtk_marshal_VOID__POINTER	g_cclosure_marshal_VOID__POINTER
 
-/* VOID:POINTER,INT (gtkmarshalers.list:77) */
+/* VOID:POINTER,INT (gtkmarshalers.list:78) */
 extern void _gtk_marshal_VOID__POINTER_INT (GClosure     *closure,
                                             GValue       *return_value,
                                             guint         n_param_values,
@@ -381,7 +389,7 @@ extern void _gtk_marshal_VOID__POINTER_INT (GClosure     *closure,
                                             gpointer      invocation_hint,
                                             gpointer      marshal_data);
 
-/* VOID:POINTER,POINTER,POINTER (gtkmarshalers.list:78) */
+/* VOID:POINTER,POINTER,POINTER (gtkmarshalers.list:79) */
 extern void _gtk_marshal_VOID__POINTER_POINTER_POINTER (GClosure     *closure,
                                                         GValue       *return_value,
                                                         guint         n_param_values,
@@ -389,7 +397,7 @@ extern void _gtk_marshal_VOID__POINTER_POINTER_POINTER (GClosure     *closure,
                                                         gpointer      invocation_hint,
                                                         gpointer      marshal_data);
 
-/* VOID:POINTER,UINT (gtkmarshalers.list:79) */
+/* VOID:POINTER,UINT (gtkmarshalers.list:80) */
 extern void _gtk_marshal_VOID__POINTER_UINT (GClosure     *closure,
                                              GValue       *return_value,
                                              guint         n_param_values,
@@ -397,10 +405,10 @@ extern void _gtk_marshal_VOID__POINTER_UINT (GClosure     *closure,
                                              gpointer      invocation_hint,
                                              gpointer      marshal_data);
 
-/* VOID:STRING (gtkmarshalers.list:80) */
+/* VOID:STRING (gtkmarshalers.list:81) */
 #define _gtk_marshal_VOID__STRING	g_cclosure_marshal_VOID__STRING
 
-/* VOID:STRING,STRING (gtkmarshalers.list:81) */
+/* VOID:STRING,STRING (gtkmarshalers.list:82) */
 extern void _gtk_marshal_VOID__STRING_STRING (GClosure     *closure,
                                               GValue       *return_value,
                                               guint         n_param_values,
@@ -408,9 +416,17 @@ extern void _gtk_marshal_VOID__STRING_STRING (GClosure     *closure,
                                               gpointer      invocation_hint,
                                               gpointer      marshal_data);
 
-/* VOID:STRING,INT,POINTER (gtkmarshalers.list:82) */
+/* VOID:STRING,INT,POINTER (gtkmarshalers.list:83) */
 
-/* VOID:UINT,UINT (gtkmarshalers.list:83) */
+/* VOID:UINT,FLAGS,BOXED (gtkmarshalers.list:84) */
+extern void _gtk_marshal_VOID__UINT_FLAGS_BOXED (GClosure     *closure,
+                                                 GValue       *return_value,
+                                                 guint         n_param_values,
+                                                 const GValue *param_values,
+                                                 gpointer      invocation_hint,
+                                                 gpointer      marshal_data);
+
+/* VOID:UINT,UINT (gtkmarshalers.list:85) */
 extern void _gtk_marshal_VOID__UINT_UINT (GClosure     *closure,
                                           GValue       *return_value,
                                           guint         n_param_values,
@@ -418,15 +434,7 @@ extern void _gtk_marshal_VOID__UINT_UINT (GClosure     *closure,
                                           gpointer      invocation_hint,
                                           gpointer      marshal_data);
 
-/* VOID:UINT,UINT,BOXED (gtkmarshalers.list:84) */
-extern void _gtk_marshal_VOID__UINT_UINT_BOXED (GClosure     *closure,
-                                                GValue       *return_value,
-                                                guint         n_param_values,
-                                                const GValue *param_values,
-                                                gpointer      invocation_hint,
-                                                gpointer      marshal_data);
-
-/* VOID:UINT,STRING (gtkmarshalers.list:85) */
+/* VOID:UINT,STRING (gtkmarshalers.list:86) */
 extern void _gtk_marshal_VOID__UINT_STRING (GClosure     *closure,
                                             GValue       *return_value,
                                             guint         n_param_values,
@@ -434,7 +442,7 @@ extern void _gtk_marshal_VOID__UINT_STRING (GClosure     *closure,
                                             gpointer      invocation_hint,
                                             gpointer      marshal_data);
 
-/* VOID:UINT,BOXED,UINT,FLAGS,FLAGS (gtkmarshalers.list:86) */
+/* VOID:UINT,BOXED,UINT,FLAGS,FLAGS (gtkmarshalers.list:87) */
 extern void _gtk_marshal_VOID__UINT_BOXED_UINT_FLAGS_FLAGS (GClosure     *closure,
                                                             GValue       *return_value,
                                                             guint         n_param_values,
@@ -442,7 +450,7 @@ extern void _gtk_marshal_VOID__UINT_BOXED_UINT_FLAGS_FLAGS (GClosure     *closur
                                                             gpointer      invocation_hint,
                                                             gpointer      marshal_data);
 
-/* VOID:UINT,OBJECT,UINT,FLAGS,FLAGS (gtkmarshalers.list:87) */
+/* VOID:UINT,OBJECT,UINT,FLAGS,FLAGS (gtkmarshalers.list:88) */
 extern void _gtk_marshal_VOID__UINT_OBJECT_UINT_FLAGS_FLAGS (GClosure     *closure,
                                                              GValue       *return_value,
                                                              guint         n_param_values,
@@ -450,7 +458,7 @@ extern void _gtk_marshal_VOID__UINT_OBJECT_UINT_FLAGS_FLAGS (GClosure     *closu
                                                              gpointer      invocation_hint,
                                                              gpointer      marshal_data);
 
-/* VOID:VOID (gtkmarshalers.list:88) */
+/* VOID:VOID (gtkmarshalers.list:89) */
 
 G_END_DECLS
 
