@@ -53,6 +53,7 @@
 #define MENU_SCROLL_TIMEOUT2 50
 
 typedef struct _GtkMenuAttachData	GtkMenuAttachData;
+typedef struct _GtkMenuPrivate  	GtkMenuPrivate;
 
 struct _GtkMenuAttachData
 {
@@ -60,6 +61,13 @@ struct _GtkMenuAttachData
   GtkMenuDetachFunc detacher;
 };
 
+struct _GtkMenuPrivate 
+{
+  gboolean have_position;
+  gint x;
+  gint y;
+};
+
 enum {
   PROP_0,
   PROP_TEAROFF_TITLE
@@ -113,6 +121,9 @@ static void     gtk_menu_handle_scrolling  (GtkMenu          *menu,
 					    gboolean         enter);
 static void     gtk_menu_set_tearoff_hints (GtkMenu          *menu,
 					    gint             width);
+static void     gtk_menu_style_set         (GtkWidget        *widget,
+					    GtkStyle         *previous_style);
+
 
 static void     gtk_menu_stop_navigating_submenu       (GtkMenu          *menu);
 static gboolean gtk_menu_stop_navigating_submenu_cb    (gpointer          user_data);
@@ -144,6 +155,29 @@ static void _gtk_menu_refresh_accel_paths (GtkMenu *menu,
 static GtkMenuShellClass *parent_class = NULL;
 static const gchar	 *attach_data_key = "gtk-menu-attach-data";
 
+GtkMenuPrivate *
+gtk_menu_get_private (GtkMenu *menu)
+{
+  GtkMenuPrivate *private;
+  static GQuark private_quark = 0;
+
+  if (!private_quark)
+    private_quark = g_quark_from_static_string ("gtk-menu-private");
+
+  private = g_object_get_qdata (G_OBJECT (menu), private_quark);
+
+  if (!private)
+    {
+      private = g_new0 (GtkMenuPrivate, 1);
+      private->have_position = FALSE;
+      
+      g_object_set_qdata_full (G_OBJECT (menu), private_quark,
+			       private, (GDestroyNotify) g_free);
+    }
+
+  return private;
+}
+
 GtkType
 gtk_menu_get_type (void)
 {
@@ -189,7 +223,7 @@ gtk_menu_class_init (GtkMenuClass *class)
                                    PROP_TEAROFF_TITLE,
                                    g_param_spec_string ("tearoff-title",
                                                         _("Tearoff Title"),
-                                                        _("A title that may be displayed by the window manager when this menu is torn-off."),
+                                                        _("A title that may be displayed by the window manager when this menu is torn-off"),
                                                         "",
                                                         G_PARAM_READABLE | G_PARAM_WRITABLE));
   object_class->destroy = gtk_menu_destroy;
@@ -206,6 +240,7 @@ gtk_menu_class_init (GtkMenuClass *class)
   widget_class->hide_all = gtk_menu_hide_all;
   widget_class->enter_notify_event = gtk_menu_enter_notify;
   widget_class->leave_notify_event = gtk_menu_leave_notify;
+  widget_class->style_set = gtk_menu_style_set;
 
   container_class->remove = gtk_menu_remove;
   
@@ -258,7 +293,7 @@ gtk_menu_class_init (GtkMenuClass *class)
 
   gtk_settings_install_property (g_param_spec_boolean ("gtk-can-change-accels",
 						       _("Can change accelerators"),
-						       _("Whether menu accelerators can be changed by pressing a key over the menu item."),
+						       _("Whether menu accelerators can be changed by pressing a key over the menu item"),
 						       FALSE,
 						       G_PARAM_READWRITE));
 }
@@ -332,6 +367,22 @@ gtk_menu_window_event (GtkWidget *window,
   return handled;
 }
 
+static void
+gtk_menu_window_size_request (GtkWidget      *window,
+			      GtkRequisition *requisition,
+			      GtkMenu        *menu)
+{
+  GtkMenuPrivate *private = gtk_menu_get_private (menu);
+
+  if (private->have_position)
+    {
+      gint screen_height = gdk_screen_height ();
+
+      if (private->y + requisition->height > screen_height)
+	requisition->height = screen_height - private->y;
+    }
+}
+
 static void
 gtk_menu_init (GtkMenu *menu)
 {
@@ -347,6 +398,7 @@ gtk_menu_init (GtkMenu *menu)
 						     "child", menu,
 						     NULL),
 				     "signal::event", gtk_menu_window_event, menu,
+				     "signal::size_request", gtk_menu_window_size_request, menu,
 				     "signal::destroy", gtk_widget_destroyed, &menu->toplevel,
 				     NULL);
   gtk_window_set_policy (GTK_WINDOW (menu->toplevel),
@@ -617,7 +669,8 @@ popup_grab_on_window (GdkWindow *window,
 	return TRUE;
       else
 	{
-	  gdk_pointer_ungrab (activate_time);
+	  gdk_display_pointer_ungrab (gdk_drawable_get_display (window),
+				      activate_time);
 	  return FALSE;
 	}
     }
@@ -639,6 +692,7 @@ gtk_menu_popup (GtkMenu		    *menu,
   GtkWidget *parent;
   GdkEvent *current_event;
   GtkMenuShell *menu_shell;
+  GtkMenuAttachData *attach_data;
 
   g_return_if_fail (GTK_IS_MENU (menu));
   
@@ -646,6 +700,21 @@ gtk_menu_popup (GtkMenu		    *menu,
   menu_shell = GTK_MENU_SHELL (menu);
   
   menu_shell->parent_menu_shell = parent_menu_shell;
+  
+  if (!g_object_get_data (G_OBJECT (menu), "gtk-menu-explicit-screen"))
+    {
+      /* The screen was not set explicitly, if the menu is
+       * attached to a widget, try to get screen from its 
+       * toplevel window else go with the default
+       */
+      attach_data = gtk_object_get_data (GTK_OBJECT (menu), attach_data_key);
+      if (attach_data)
+	{
+	  if (!GTK_WIDGET_REALIZED (menu))
+	  gtk_window_set_screen (GTK_WINDOW (menu->toplevel),
+				 gtk_widget_get_screen (attach_data->attach_widget));
+	}
+    }
 
   /* Find the last viewable ancestor, and make an X grab on it
    */
@@ -709,6 +778,7 @@ gtk_menu_popup (GtkMenu		    *menu,
        * try again.
        */
       menu_shell->parent_menu_shell = NULL;
+      menu_grab_transfer_window_destroy (menu);
       return;
     }
 
@@ -785,16 +855,20 @@ gtk_menu_popup (GtkMenu		    *menu,
 void
 gtk_menu_popdown (GtkMenu *menu)
 {
+  GtkMenuPrivate *private;
   GtkMenuShell *menu_shell;
 
   g_return_if_fail (GTK_IS_MENU (menu));
   
   menu_shell = GTK_MENU_SHELL (menu);
+  private = gtk_menu_get_private (menu);
   
   menu_shell->parent_menu_shell = NULL;
   menu_shell->active = FALSE;
   menu_shell->ignore_enter = FALSE;
 
+  private->have_position = FALSE;
+
   gtk_menu_stop_scrolling (menu);
   
   gtk_menu_stop_navigating_submenu (menu);
@@ -815,11 +889,7 @@ gtk_menu_popdown (GtkMenu *menu)
 
   if (menu->torn_off)
     {
-      gint width, height;
-      gdk_window_get_size (menu->tearoff_window->window, &width, &height);
-      gtk_widget_set_usize (menu->tearoff_window,
-			    -1,
-			    height);
+      gtk_widget_set_usize (menu->tearoff_window, -1, -1);
       
       if (GTK_BIN (menu->toplevel)->child) 
 	{
@@ -832,8 +902,10 @@ gtk_menu_popdown (GtkMenu *menu)
 	   */
 	  if (menu_shell->have_xgrab)
 	    {
-	      gdk_pointer_ungrab (GDK_CURRENT_TIME);
-	      gdk_keyboard_ungrab (GDK_CURRENT_TIME);
+	      GdkDisplay *display = gtk_widget_get_display (GTK_WIDGET (menu));
+	      
+	      gdk_display_pointer_ungrab (display, GDK_CURRENT_TIME);
+	      gdk_display_keyboard_ungrab (display, GDK_CURRENT_TIME);
 	    }
 	}
 
@@ -1043,6 +1115,7 @@ gtk_menu_set_tearoff_hints (GtkMenu *menu,
     
   geometry_hints.min_height = 0;
   geometry_hints.max_height = GTK_WIDGET (menu)->requisition.height;
+
   gtk_window_set_geometry_hints (GTK_WINDOW (menu->tearoff_window),
 				 NULL,
 				 &geometry_hints,
@@ -1094,31 +1167,24 @@ gtk_menu_set_tearoff_state (GtkMenu  *menu,
 
 	  if (!menu->tearoff_window)
 	    {
-	      menu->tearoff_window = g_object_connect (gtk_widget_new (GTK_TYPE_WINDOW,
-								       "type", GTK_WINDOW_TOPLEVEL,
-								       NULL),
-						       "signal::destroy", gtk_widget_destroyed, &menu->tearoff_window,
-						       NULL);
+	      menu->tearoff_window = gtk_widget_new (GTK_TYPE_WINDOW,
+						     "type", GTK_WINDOW_TOPLEVEL,
+						     "screen", gtk_widget_get_screen (menu->toplevel),
+						     "app_paintable", TRUE,
+						     NULL);
+
 	      gtk_window_set_type_hint (GTK_WINDOW (menu->tearoff_window),
 					GDK_WINDOW_TYPE_HINT_MENU);
 	      gtk_window_set_mnemonic_modifier (GTK_WINDOW (menu->tearoff_window), 0);
-	      gtk_widget_set_app_paintable (menu->tearoff_window, TRUE);
-	      gtk_signal_connect (GTK_OBJECT (menu->tearoff_window),  
-				  "event",
-				  GTK_SIGNAL_FUNC (gtk_menu_window_event), 
-				  GTK_OBJECT (menu));
+	      g_signal_connect (menu->tearoff_window, "destroy",
+				G_CALLBACK (gtk_widget_destroyed), &menu->tearoff_window);
+	      g_signal_connect (menu->tearoff_window, "event",
+				G_CALLBACK (gtk_menu_window_event), menu);
 
 	      gtk_menu_update_title (menu);
 
 	      gtk_widget_realize (menu->tearoff_window);
 	      
-	      gdk_window_set_decorations (menu->tearoff_window->window, 
-					  GDK_DECOR_ALL |
-					  GDK_DECOR_RESIZEH |
-					  GDK_DECOR_MINIMIZE |
-					  GDK_DECOR_MAXIMIZE);
-	      gtk_window_set_resizable (GTK_WINDOW (menu->tearoff_window), FALSE);
-
 	      menu->tearoff_hbox = gtk_hbox_new (FALSE, FALSE);
 	      gtk_container_add (GTK_CONTAINER (menu->tearoff_window), menu->tearoff_hbox);
 
@@ -1220,7 +1286,8 @@ gtk_menu_set_title (GtkMenu     *menu,
  * Returns the title of the menu. See gtk_menu_set_title().
  *
  * Return value: the title of the menu, or %NULL if the menu has no
- *               title set on it.
+ * title set on it. This string is owned by the widget and should
+ * not be modified or freed.
  **/
 G_CONST_RETURN gchar *
 gtk_menu_get_title (GtkMenu *menu)
@@ -1248,6 +1315,20 @@ gtk_menu_reorder_child (GtkMenu   *menu,
     }   
 }
 
+static void
+gtk_menu_style_set (GtkWidget *widget,
+		    GtkStyle  *previous_style)
+{
+  if (GTK_WIDGET_REALIZED (widget))
+    {
+      GtkMenu *menu = GTK_MENU (widget);
+      
+      gtk_style_set_background (widget->style, menu->bin_window, GTK_STATE_NORMAL);
+      gtk_style_set_background (widget->style, menu->view_window, GTK_STATE_NORMAL);
+      gtk_style_set_background (widget->style, widget->window, GTK_STATE_NORMAL);
+    }
+}
+
 static void
 gtk_menu_realize (GtkWidget *widget)
 {
@@ -1350,7 +1431,8 @@ menu_grab_transfer_window_get (GtkMenu *menu)
 
       attributes_mask = GDK_WA_X | GDK_WA_Y | GDK_WA_NOREDIR;
       
-      window = gdk_window_new (NULL, &attributes, attributes_mask);
+      window = gdk_window_new (gtk_widget_get_root_window (GTK_WIDGET (menu)),
+			       &attributes, attributes_mask);
       gdk_window_set_user_data (window, menu);
 
       gdk_window_show (window);
@@ -1455,11 +1537,9 @@ gtk_menu_size_request (GtkWidget      *widget,
   
   menu->toggle_size = max_toggle_size;
 
-  /* If the requested width was different than the allocated width, we need to change
-   * the geometry hints for the tear off window so that the window can actually be resized.
-   * Don't resize the tearoff if it is not active, because it won't redraw (it is only a background pixmap).
+  /* Don't resize the tearoff if it is not active, because it won't redraw (it is only a background pixmap).
    */
-  if ((requisition->width != GTK_WIDGET (menu)->allocation.width) && menu->tearoff_active)
+  if (menu->tearoff_active)
     gtk_menu_set_tearoff_hints (menu, requisition->width);
 }
 
@@ -1488,6 +1568,9 @@ gtk_menu_size_allocate (GtkWidget     *widget,
   
   width = MAX (1, allocation->width - x * 2);
   height = MAX (1, allocation->height - y * 2);
+
+  if (menu_shell->active)
+    gtk_menu_scroll_to (menu, menu->scroll_offset);
   
   if (menu->upper_arrow_visible && !menu->tearoff_active)
     {
@@ -1556,7 +1639,6 @@ gtk_menu_size_allocate (GtkWidget     *widget,
 		{
 		  gtk_widget_hide (menu->tearoff_scrollbar);
 		  gtk_menu_set_tearoff_hints (menu, allocation->width);
-		  gtk_widget_set_usize (menu->tearoff_window, -1, allocation->height);
 
 		  gtk_menu_scroll_to (menu, 0);
 		}
@@ -1582,7 +1664,6 @@ gtk_menu_size_allocate (GtkWidget     *widget,
 		{
 		  gtk_widget_show (menu->tearoff_scrollbar);
 		  gtk_menu_set_tearoff_hints (menu, allocation->width);
-		  gtk_widget_set_usize (menu->tearoff_window, -1, allocation->height);
 		}
 	    }
 	}
@@ -1667,35 +1748,6 @@ gtk_menu_paint (GtkWidget      *widget,
 			   MENU_SCROLL_ARROW_HEIGHT - 2 * border_y - 2);
 	}
     }
-  else if (event->window == menu->view_window)
-    {
-      gint menu_height;
-      gint top_pos;
-      
-      if (menu->scroll_offset < 0)
-	gtk_paint_box (widget->style,
-		       menu->view_window,
-		       GTK_STATE_ACTIVE,
-		       GTK_SHADOW_IN,
-		       NULL, widget, "menu",
-		       0, 0,
-		       -1,
-		       -menu->scroll_offset);
-
-      menu_height = widget->requisition.height - 2*border_y;
-      top_pos = height - 2*border_y - (menu->upper_arrow_visible ? MENU_SCROLL_ARROW_HEIGHT : 0);
-
-      if (menu_height - menu->scroll_offset < top_pos)
-	gtk_paint_box (widget->style,
-		       menu->view_window,
-		       GTK_STATE_ACTIVE,
-		       GTK_SHADOW_IN,
-		       NULL, widget, "menu",
-		       0,
-		       menu_height - menu->scroll_offset,
-		       -1,
-		       top_pos - (menu_height - menu->scroll_offset));
-    }
 }
 
 static gboolean
@@ -1736,6 +1788,7 @@ gtk_menu_key_press (GtkWidget	*widget,
   gchar *accel = NULL;
   guint accel_key, accel_mods;
   GdkModifierType consumed_modifiers;
+  GdkDisplay *display;
   
   g_return_val_if_fail (GTK_IS_MENU (widget), FALSE);
   g_return_val_if_fail (event != NULL, FALSE);
@@ -1747,10 +1800,12 @@ gtk_menu_key_press (GtkWidget	*widget,
 
   if (GTK_WIDGET_CLASS (parent_class)->key_press_event (widget, event))
     return TRUE;
+
+  display = gtk_widget_get_display (widget);
     
-  g_object_get (G_OBJECT (gtk_settings_get_default ()),
-                "gtk-menu-bar-accel",
-                &accel,
+  g_object_get (G_OBJECT (gtk_widget_get_settings (widget)),
+                "gtk-menu-bar-accel", &accel,
+		"gtk-can-change-accels", &can_change_accels,
                 NULL);
 
   if (accel)
@@ -1791,12 +1846,8 @@ gtk_menu_key_press (GtkWidget	*widget,
       break;
     }
 
-  g_object_get (G_OBJECT (gtk_settings_get_default ()),
-		"gtk-can-change-accels", &can_change_accels,
-		NULL);
-
   /* Figure out what modifiers went into determining the key symbol */
-  gdk_keymap_translate_keyboard_state (gdk_keymap_get_default (),
+  gdk_keymap_translate_keyboard_state (gdk_keymap_get_for_display (display),
 				       event->hardware_keycode, event->state, event->group,
 				       NULL, NULL, NULL, &consumed_modifiers);
 
@@ -1814,20 +1865,20 @@ gtk_menu_key_press (GtkWidget	*widget,
       menu_shell->active_menu_item &&
       GTK_BIN (menu_shell->active_menu_item)->child &&			/* no seperators */
       GTK_MENU_ITEM (menu_shell->active_menu_item)->submenu == NULL &&	/* no submenus */
-      (delete || gtk_accelerator_valid (event->keyval, event->state)))
+      (delete || gtk_accelerator_valid (accel_key, accel_mods)))
     {
       GtkWidget *menu_item = menu_shell->active_menu_item;
-      gboolean replace_accels = TRUE;
+      gboolean locked, replace_accels = TRUE;
       const gchar *path;
 
-      path = _gtk_widget_get_accel_path (menu_item);
-      if (!path)
+      path = _gtk_widget_get_accel_path (menu_item, &locked);
+      if (!path || locked)
 	{
 	  /* can't change accelerators on menu_items without paths
 	   * (basically, those items are accelerator-locked).
 	   */
-	  /* g_print("item has no path, menu prefix: %s\n", menu->accel_path); */
-	  gdk_beep ();
+	  /* g_print("item has no path or is locked, menu prefix: %s\n", menu->accel_path); */
+	  gdk_display_beep (display);
 	}
       else
 	{
@@ -1856,7 +1907,7 @@ gtk_menu_key_press (GtkWidget	*widget,
 	       * locked already
 	       */
 	      /* g_print("failed to change\n"); */
-	      gdk_beep ();
+	      gdk_display_beep (display);
 	    }
 	}
     }
@@ -2309,21 +2360,25 @@ gtk_menu_position (GtkMenu *menu)
 {
   GtkWidget *widget;
   GtkRequisition requisition;
+  GtkMenuPrivate *private;
   gint x, y;
-  gint screen_width;
-  gint screen_height;
   gint scroll_offset;
   gint menu_height;
   gboolean push_in;
-  
+  GdkScreen *screen;
+  GdkRectangle monitor;
+  gint monitor_num;
+
   g_return_if_fail (GTK_IS_MENU (menu));
 
   widget = GTK_WIDGET (menu);
 
-  screen_width = gdk_screen_width ();
-  screen_height = gdk_screen_height ();
+  gdk_window_get_pointer (gtk_widget_get_root_window (widget),
+  			  &x, &y, NULL);
 
-  gdk_window_get_pointer (NULL, &x, &y, NULL);
+  screen = gtk_widget_get_screen (widget);
+  monitor_num = gdk_screen_get_monitor_at_point (screen, x, y);
+  gdk_screen_get_monitor_geometry (screen, monitor_num, &monitor);
 
   /* We need the requisition to figure out the right place to
    * popup the menu. In fact, we always need to ask here, since
@@ -2338,8 +2393,8 @@ gtk_menu_position (GtkMenu *menu)
     (* menu->position_func) (menu, &x, &y, &push_in, menu->position_func_data);
   else
     {
-      x = CLAMP (x - 2, 0, MAX (0, screen_width - requisition.width));
-      y = CLAMP (y - 2, 0, MAX (0, screen_height - requisition.height));
+      x = CLAMP (x - 2, monitor.x, MAX (monitor.x, monitor.x + monitor.width - requisition.width));
+      y = CLAMP (y - 2, monitor.y, MAX (monitor.y, monitor.y + monitor.height - requisition.height));      
     }
 
   scroll_offset = 0;
@@ -2348,27 +2403,30 @@ gtk_menu_position (GtkMenu *menu)
     {
       menu_height = GTK_WIDGET (menu)->requisition.height;
 
-      if (y + menu_height > screen_height)
+      if (y + menu_height > monitor.y + monitor.height)
 	{
-	  scroll_offset -= y + menu_height - screen_height;
-	  y = screen_height - menu_height;
+	  scroll_offset -= y + menu_height - (monitor.y + monitor.height);
+	  y = (monitor.y + monitor.height) - menu_height;
 	}
   
-      if (y < 0)
+      if (y < monitor.y)
 	{
 	  scroll_offset -= y;
-	  y = 0;
+	  y = monitor.y;
 	}
     }
 
-  if (y + requisition.height > screen_height)
-    requisition.height = screen_height - y;
+  /* FIXME: should this be done in the various position_funcs ? */
+  x = CLAMP (x, monitor.x, MAX (monitor.x, monitor.x + monitor.width - requisition.width));
+ 
+  if (y + requisition.height > monitor.y + monitor.height)
+    requisition.height = (monitor.y + monitor.height) - y;
   
-  if (y < 0)
+  if (y < monitor.y)
     {
       scroll_offset -= y;
       requisition.height -= -y;
-      y = 0;
+      y = monitor.y;
     }
 
   if (scroll_offset > 0)
@@ -2376,9 +2434,22 @@ gtk_menu_position (GtkMenu *menu)
   
   gtk_window_move (GTK_WINDOW (GTK_MENU_SHELL (menu)->active ? menu->toplevel : menu->tearoff_window), 
 		   x, y);
-  gtk_widget_set_usize (GTK_MENU_SHELL (menu)->active ?
-			menu->toplevel : menu->tearoff_hbox,
-			-1, requisition.height);
+
+  if (GTK_MENU_SHELL (menu)->active)
+    {
+      private = gtk_menu_get_private (menu);
+      private->have_position = TRUE;
+      private->x = x;
+      private->y = y;
+
+      gtk_widget_queue_resize (menu->toplevel);
+    }
+  else
+    {
+      gtk_window_resize (GTK_WINDOW (menu->tearoff_window),
+			 requisition.width, requisition.height);
+    }
+  
   menu->scroll_offset = scroll_offset;
 }
 
@@ -2414,17 +2485,21 @@ gtk_menu_scroll_to (GtkMenu *menu,
       gtk_adjustment_value_changed (menu->tearoff_adjustment);
     }
   
-  /* Scroll the menu: */
-  gdk_window_move (menu->bin_window, 0, -offset);
-
   /* Move/resize the viewport according to arrows: */
-  gdk_window_get_size (widget->window, &view_width, &view_height);
+  view_width = widget->allocation.width;
+  view_height = widget->allocation.height;
 
   border_width = GTK_CONTAINER (menu)->border_width;
   view_width -= (border_width + widget->style->xthickness) * 2;
   view_height -= (border_width + widget->style->ythickness) * 2;
   menu_height = widget->requisition.height - (border_width + widget->style->ythickness) * 2;
 
+  offset = CLAMP (offset, 0, menu_height - view_height);
+
+  /* Scroll the menu: */
+  if (GTK_WIDGET_REALIZED (menu))
+    gdk_window_move (menu->bin_window, 0, -offset);
+
   x = border_width + widget->style->xthickness;
   y = border_width + widget->style->ythickness;
   
@@ -2432,7 +2507,7 @@ gtk_menu_scroll_to (GtkMenu *menu,
     {
       last_visible = menu->upper_arrow_visible;
       menu->upper_arrow_visible = (offset > 0);
-
+      
       if (menu->upper_arrow_visible)
 	view_height -= MENU_SCROLL_ARROW_HEIGHT;
       
@@ -2465,12 +2540,13 @@ gtk_menu_scroll_to (GtkMenu *menu,
       if (menu->upper_arrow_visible)
 	y += MENU_SCROLL_ARROW_HEIGHT;
     }
-  
-  gdk_window_move_resize (menu->view_window,
-			  x,
-			  y,
-			  view_width,
-			  view_height);
+
+  if (GTK_WIDGET_REALIZED (menu))
+    gdk_window_move_resize (menu->view_window,
+			    x,
+			    y,
+			    view_width,
+			    view_height);
 
   menu->scroll_offset = offset;
 }
@@ -2654,4 +2730,21 @@ gtk_menu_hide_all (GtkWidget *widget)
   gtk_container_foreach (GTK_CONTAINER (widget), (GtkCallback) gtk_widget_hide_all, NULL);
 }
 
+/**
+ * gtk_menu_set_screen:
+ * @menu: a #GtkMenu.
+ * @screen: a #GtkScreen.
+ *
+ * Sets the #GtkScreen on which the GtkMenu will be displayed.
+ * This function can only be called before @menu is realized.
+ **/
+void
+gtk_menu_set_screen (GtkMenu *menu, 
+		     GdkScreen *screen)
+{
+  g_return_if_fail (GTK_IS_MENU (menu));
+  gtk_window_set_screen (GTK_WINDOW (menu->toplevel), 
+			 screen);
+  g_object_set_data (G_OBJECT (menu), "gtk-menu-explicit-screen", screen);
+}
 
