@@ -164,19 +164,22 @@ gtk_menu_shell_get_type (void)
 
   if (!menu_shell_type)
     {
-      static const GtkTypeInfo menu_shell_info =
+      static const GTypeInfo menu_shell_info =
       {
-	"GtkMenuShell",
-	sizeof (GtkMenuShell),
 	sizeof (GtkMenuShellClass),
-	(GtkClassInitFunc) gtk_menu_shell_class_init,
-	(GtkObjectInitFunc) gtk_menu_shell_init,
-	/* reserved_1 */ NULL,
-        /* reserved_2 */ NULL,
-        (GtkClassInitFunc) NULL,
+	NULL,		/* base_init */
+	NULL,		/* base_finalize */
+        (GClassInitFunc) gtk_menu_shell_class_init,
+	NULL,		/* class_finalize */
+	NULL,		/* class_data */
+	sizeof (GtkMenuShell),
+	0,              /* n_preallocs */
+        (GInstanceInitFunc) gtk_menu_shell_init,
+	NULL            /* value_table */
       };
 
-      menu_shell_type = gtk_type_unique (gtk_container_get_type (), &menu_shell_info);
+      menu_shell_type = g_type_register_static (GTK_TYPE_CONTAINER, "GtkMenuShell",
+						&menu_shell_info, G_TYPE_FLAG_ABSTRACT);
     }
 
   return menu_shell_type;
@@ -794,9 +797,11 @@ gtk_real_menu_shell_deactivate (GtkMenuShell *menu_shell)
 	}
       if (menu_shell->have_xgrab)
 	{
+	  GdkDisplay *display = gtk_widget_get_display (GTK_WIDGET (menu_shell));
+	  
 	  menu_shell->have_xgrab = FALSE;
-	  gdk_pointer_ungrab (GDK_CURRENT_TIME);
-	  gdk_keyboard_ungrab (GDK_CURRENT_TIME);
+	  gdk_display_pointer_ungrab (display, GDK_CURRENT_TIME);
+	  gdk_display_keyboard_ungrab (display, GDK_CURRENT_TIME);
 	}
     }
 }
@@ -925,7 +930,7 @@ gtk_menu_shell_activate_item (GtkMenuShell      *menu_shell,
       /* flush the x-queue, so any grabs are removed and
        * the menu is actually taken down
        */
-      gdk_flush ();
+      gdk_display_sync (gtk_widget_get_display (menu_item));
     }
 
   gtk_widget_activate (menu_item);
@@ -982,7 +987,7 @@ gtk_menu_shell_move_selected (GtkMenuShell  *menu_shell,
 }
 
 /**
- * _gtk_menu_shell_select_first:
+ * gtk_menu_shell_select_first:
  * @menu_shell: a #GtkMenuShell
  * @search_sensitive: if %TRUE, search for the first selectable
  *                    menu item, otherwise select nothing if
@@ -995,8 +1000,8 @@ gtk_menu_shell_move_selected (GtkMenuShell  *menu_shell,
  * item.
  **/
 void
-_gtk_menu_shell_select_first (GtkMenuShell *menu_shell,
-			      gboolean      search_sensitive)
+gtk_menu_shell_select_first (GtkMenuShell *menu_shell,
+			     gboolean      search_sensitive)
 {
   GtkWidget *to_select = NULL;
   GList *tmp_list;
@@ -1056,7 +1061,7 @@ gtk_menu_shell_select_submenu_first (GtkMenuShell     *menu_shell)
   menu_item = GTK_MENU_ITEM (menu_shell->active_menu_item); 
   
   if (menu_item->submenu)
-    _gtk_menu_shell_select_first (GTK_MENU_SHELL (menu_item->submenu), TRUE);
+    gtk_menu_shell_select_first (GTK_MENU_SHELL (menu_item->submenu), TRUE);
 }
 
 static void
@@ -1129,7 +1134,7 @@ gtk_real_menu_shell_move_current (GtkMenuShell      *menu_shell,
       if (!had_selection &&
 	  !menu_shell->active_menu_item &&
 	  menu_shell->children)
-	_gtk_menu_shell_select_first (menu_shell, TRUE);
+	gtk_menu_shell_select_first (menu_shell, TRUE);
       break;
     }
   
