@@ -111,7 +111,9 @@ void    gtk_menu_shell_activate_item  (GtkMenuShell      *menu_shell,
 				       GtkWidget         *menu_item,
 				       gboolean           force_deactivate);
 
-void _gtk_menu_shell_select_first (GtkMenuShell *menu_shell);
+void gtk_menu_shell_select_first (GtkMenuShell *menu_shell,
+				  gboolean      search_sensitive);
+void _gtk_menu_shell_activate     (GtkMenuShell *menu_shell);
 
 #ifdef __cplusplus
 }
