@@ -30,6 +30,8 @@
 
 #include "gtklabel.h"
 #include "gtkmain.h"
+#include "gtkmenuitem.h"
+#include "gtkprivate.h"
 #include "gtkwidget.h"
 #include "gtkwindow.h"
 #include "gtksignal.h"
@@ -174,6 +176,8 @@ gtk_tooltips_force_window (GtkTooltips *tooltips)
   if (!tooltips->tip_window)
     {
       tooltips->tip_window = gtk_window_new (GTK_WINDOW_POPUP);
+      gtk_window_set_screen (GTK_WINDOW (tooltips->tip_window),
+			     gtk_widget_get_screen (tooltips->active_tips_data->widget));
       gtk_widget_set_app_paintable (tooltips->tip_window, TRUE);
       gtk_window_set_policy (GTK_WINDOW (tooltips->tip_window), FALSE, FALSE, TRUE);
       gtk_widget_set_name (tooltips->tip_window, "gtk-tooltips");
@@ -323,6 +327,7 @@ gtk_tooltips_draw_tips (GtkTooltips *tooltips)
   gint x, y, w, h, scr_w, scr_h;
   GtkTooltipsData *data;
   gboolean keyboard_mode;
+  GdkScreen *screen;
 
   if (!tooltips->tip_window)
     gtk_tooltips_force_window (tooltips);
@@ -335,9 +340,10 @@ gtk_tooltips_draw_tips (GtkTooltips *tooltips)
   widget = tooltips->active_tips_data->widget;
 
   keyboard_mode = get_keyboard_mode (widget);
-
-  scr_w = gdk_screen_width ();
-  scr_h = gdk_screen_height ();
+  
+  screen = gtk_widget_get_screen (widget);
+  scr_w = gdk_screen_get_width (screen);
+  scr_h = gdk_screen_get_height (screen);
 
   data = tooltips->active_tips_data;
 
@@ -357,7 +363,8 @@ gtk_tooltips_draw_tips (GtkTooltips *tooltips)
   x += widget->allocation.width / 2;
     
   if (!keyboard_mode)
-    gdk_window_get_pointer (NULL, &x, NULL, NULL);
+    gdk_window_get_pointer (gdk_screen_get_root_window (screen),
+			    &x, NULL, NULL);
 
   x -= (w / 2 + 4);
 
@@ -513,6 +520,31 @@ stop_keyboard_mode (GtkWidget *widget)
     }
 }
 
+static void
+gtk_tooltips_start_delay (GtkTooltips *tooltips,
+			  GtkWidget   *widget)
+{
+  GtkTooltipsData *old_tips_data;
+  
+  old_tips_data = tooltips->active_tips_data;
+  if (tooltips->enabled &&
+      (!old_tips_data || old_tips_data->widget != widget))
+    {
+      guint delay;
+      
+      gtk_tooltips_set_active_widget (tooltips, widget);
+      
+      if (tooltips->use_sticky_delay &&
+	  gtk_tooltips_recently_shown (tooltips))
+	delay = STICKY_DELAY;
+      else
+	delay = tooltips->delay;
+      tooltips->timer_tag = gtk_timeout_add (delay,
+					     gtk_tooltips_timeout,
+					     (gpointer) tooltips);
+    }
+}
+
 static void
 gtk_tooltips_event_handler (GtkWidget *widget,
                             GdkEvent  *event)
@@ -554,29 +586,12 @@ gtk_tooltips_event_handler (GtkWidget *widget,
   
       switch (event->type)
 	{
-	case GDK_MOTION_NOTIFY:
 	case GDK_EXPOSE:
 	  /* do nothing */
 	  break;
-	  
 	case GDK_ENTER_NOTIFY:
-	  old_tips_data = tooltips->active_tips_data;
-	  if (tooltips->enabled &&
-	      (!old_tips_data || old_tips_data->widget != widget))
-	    {
-	      guint delay;
-	      
-	      gtk_tooltips_set_active_widget (tooltips, widget);
-	      
-	      if (tooltips->use_sticky_delay  &&
-	      gtk_tooltips_recently_shown (tooltips))
-		delay = STICKY_DELAY;
-	      else
-		delay = tooltips->delay;
-	      tooltips->timer_tag = gtk_timeout_add (delay,
-						     gtk_tooltips_timeout,
-						     (gpointer) tooltips);
-	    }
+	  if (!(GTK_IS_MENU_ITEM (widget) && GTK_MENU_ITEM (widget)->submenu))
+	    gtk_tooltips_start_delay (tooltips, widget);
 	  break;
 	  
 	case GDK_LEAVE_NOTIFY:
@@ -590,6 +605,20 @@ gtk_tooltips_event_handler (GtkWidget *widget,
 	  }
 	  break;
 
+	case GDK_MOTION_NOTIFY:
+	  /* Handle menu items specially ... pend popup for each motion
+	   * on other widgets, we ignore motion.
+	   */
+	  if (GTK_IS_MENU_ITEM (widget) && !GTK_MENU_ITEM (widget)->submenu)
+	    {
+	      /* Completely evil hack to make sure we get the LEAVE_NOTIFY
+	       */
+	      GTK_PRIVATE_SET_FLAG (widget, GTK_LEAVE_PENDING);
+	      gtk_tooltips_set_active_widget (tooltips, NULL);
+	      gtk_tooltips_start_delay (tooltips, widget);
+	      break;
+	    }
+	  break;		/* ignore */
 	case GDK_BUTTON_PRESS:
 	case GDK_BUTTON_RELEASE:
 	case GDK_KEY_PRESS:
