@@ -2,35 +2,37 @@
  * Copyright (C) 1995-1997 Peter Mattis, Spencer Kimball and Josh MacDonald
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
+ * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
+ * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
 
 /*
- * Modified by the GTK+ Team and others 1997-1999.  See the AUTHORS
+ * Modified by the GTK+ Team and others 1997-2000.  See the AUTHORS
  * file for a list of people on the GTK+ Team.  See the ChangeLog
  * files for a list of changes.  These files are distributed with
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
 #include "gtklabel.h"
-#include "gtktree.h"
-#include "gtktreeitem.h"
 #include "gtkeventbox.h"
 #include "gtkpixmap.h"
 #include "gtkmain.h"
+#include "gtkmarshalers.h"
 #include "gtksignal.h"
+#define GTK_ENABLE_BROKEN
+#include "gtktree.h"
+#include "gtktreeitem.h"
 
 #include "tree_plus.xpm"
 #include "tree_minus.xpm"
@@ -64,19 +66,12 @@ static void gtk_tree_item_size_request  (GtkWidget        *widget,
 					 GtkRequisition   *requisition);
 static void gtk_tree_item_size_allocate (GtkWidget        *widget,
 					 GtkAllocation    *allocation);
-static void gtk_tree_item_draw          (GtkWidget        *widget,
-					 GdkRectangle     *area);
-static void gtk_tree_item_draw_focus    (GtkWidget        *widget);
 static void gtk_tree_item_paint         (GtkWidget        *widget,
 					 GdkRectangle     *area);
 static gint gtk_tree_item_button_press  (GtkWidget        *widget,
 					 GdkEventButton   *event);
 static gint gtk_tree_item_expose        (GtkWidget        *widget,
 					 GdkEventExpose   *event);
-static gint gtk_tree_item_focus_in      (GtkWidget        *widget,
-					 GdkEventFocus    *event);
-static gint gtk_tree_item_focus_out     (GtkWidget        *widget,
-					 GdkEventFocus    *event);
 static void gtk_tree_item_forall        (GtkContainer    *container,
 					 gboolean         include_internals,
 					 GtkCallback      callback,
@@ -90,7 +85,7 @@ static void gtk_real_tree_item_collapse (GtkTreeItem      *item);
 static void gtk_real_tree_item_expand   (GtkTreeItem      *item);
 static void gtk_real_tree_item_collapse (GtkTreeItem      *item);
 static void gtk_tree_item_destroy        (GtkObject *object);
-static void gtk_tree_item_subtree_button_click (GtkWidget *widget);
+static gint gtk_tree_item_subtree_button_click (GtkWidget *widget);
 static void gtk_tree_item_subtree_button_changed_state (GtkWidget *widget);
 
 static void gtk_tree_item_map(GtkWidget*);
@@ -135,41 +130,20 @@ gtk_tree_item_class_init (GtkTreeItemClass *class)
   GtkContainerClass *container_class;
   GtkItemClass *item_class;
 
+  parent_class = gtk_type_class (GTK_TYPE_ITEM);
+  
   object_class = (GtkObjectClass*) class;
   widget_class = (GtkWidgetClass*) class;
   item_class = (GtkItemClass*) class;
   container_class = (GtkContainerClass*) class;
 
-  parent_class = gtk_type_class (gtk_item_get_type ());
-  
-  tree_item_signals[EXPAND_TREE] =
-    gtk_signal_new ("expand",
-		    GTK_RUN_FIRST,
-		    object_class->type,
-		    GTK_SIGNAL_OFFSET (GtkTreeItemClass, expand),
-		    gtk_marshal_NONE__NONE,
-		    GTK_TYPE_NONE, 0);
-  tree_item_signals[COLLAPSE_TREE] =
-    gtk_signal_new ("collapse",
-		    GTK_RUN_FIRST,
-		    object_class->type,
-		    GTK_SIGNAL_OFFSET (GtkTreeItemClass, collapse),
-		    gtk_marshal_NONE__NONE,
-		    GTK_TYPE_NONE, 0);
-
-  gtk_object_class_add_signals (object_class, tree_item_signals, LAST_SIGNAL);
-
   object_class->destroy = gtk_tree_item_destroy;
 
   widget_class->realize = gtk_tree_item_realize;
   widget_class->size_request = gtk_tree_item_size_request;
   widget_class->size_allocate = gtk_tree_item_size_allocate;
-  widget_class->draw = gtk_tree_item_draw;
-  widget_class->draw_focus = gtk_tree_item_draw_focus;
   widget_class->button_press_event = gtk_tree_item_button_press;
   widget_class->expose_event = gtk_tree_item_expose;
-  widget_class->focus_in_event = gtk_tree_item_focus_in;
-  widget_class->focus_out_event = gtk_tree_item_focus_out;
   widget_class->map = gtk_tree_item_map;
   widget_class->unmap = gtk_tree_item_unmap;
 
@@ -181,32 +155,47 @@ gtk_tree_item_class_init (GtkTreeItemClass *class)
 
   class->expand = gtk_real_tree_item_expand;
   class->collapse = gtk_real_tree_item_collapse;
+
+  tree_item_signals[EXPAND_TREE] =
+    gtk_signal_new ("expand",
+		    GTK_RUN_FIRST,
+		    GTK_CLASS_TYPE (object_class),
+		    GTK_SIGNAL_OFFSET (GtkTreeItemClass, expand),
+		    _gtk_marshal_VOID__VOID,
+		    GTK_TYPE_NONE, 0);
+  tree_item_signals[COLLAPSE_TREE] =
+    gtk_signal_new ("collapse",
+		    GTK_RUN_FIRST,
+		    GTK_CLASS_TYPE (object_class),
+		    GTK_SIGNAL_OFFSET (GtkTreeItemClass, collapse),
+		    _gtk_marshal_VOID__VOID,
+		    GTK_TYPE_NONE, 0);
 }
 
 /* callback for event box mouse event */
-static void 
+static gint
 gtk_tree_item_subtree_button_click (GtkWidget *widget)
 {
   GtkTreeItem* item;
   
-  g_return_if_fail (widget != NULL);
-  g_return_if_fail (GTK_IS_EVENT_BOX (widget));
+  g_return_val_if_fail (GTK_IS_EVENT_BOX (widget), FALSE);
   
   item = (GtkTreeItem*) gtk_object_get_user_data (GTK_OBJECT (widget));
   if (!GTK_WIDGET_IS_SENSITIVE (item))
-    return;
+    return FALSE;
   
   if (item->expanded)
     gtk_tree_item_collapse (item);
   else
     gtk_tree_item_expand (item);
+
+  return TRUE;
 }
 
 /* callback for event box state changed */
 static void
 gtk_tree_item_subtree_button_changed_state (GtkWidget *widget)
 {
-  g_return_if_fail (widget != NULL);
   g_return_if_fail (GTK_IS_EVENT_BOX (widget));
   
   if (GTK_WIDGET_VISIBLE (widget))
@@ -228,7 +217,6 @@ gtk_tree_item_init (GtkTreeItem *tree_item)
 {
   GtkWidget *eventbox, *pixmapwid;
   
-  g_return_if_fail (tree_item != NULL);
   g_return_if_fail (GTK_IS_TREE_ITEM (tree_item));
 
   tree_item->expanded = FALSE;
@@ -303,9 +291,7 @@ void
 gtk_tree_item_set_subtree (GtkTreeItem *tree_item,
 			   GtkWidget   *subtree)
 {
-  g_return_if_fail (tree_item != NULL);
   g_return_if_fail (GTK_IS_TREE_ITEM (tree_item));
-  g_return_if_fail (subtree != NULL);
   g_return_if_fail (GTK_IS_TREE (subtree));
 
   if (tree_item->subtree)
@@ -327,23 +313,11 @@ gtk_tree_item_set_subtree (GtkTreeItem *tree_item,
     gtk_widget_hide (subtree);
 
   gtk_widget_set_parent (subtree, GTK_WIDGET (tree_item)->parent);
-
-  if (GTK_WIDGET_REALIZED (subtree->parent))
-    gtk_widget_realize (subtree);
-
-  if (GTK_WIDGET_VISIBLE (subtree->parent) && GTK_WIDGET_VISIBLE (subtree))
-    {
-      if (GTK_WIDGET_MAPPED (subtree->parent))
-	gtk_widget_map (subtree);
-
-      gtk_widget_queue_resize (subtree);
-    }
 }
 
 void
 gtk_tree_item_select (GtkTreeItem *tree_item)
 {
-  g_return_if_fail (tree_item != NULL);
   g_return_if_fail (GTK_IS_TREE_ITEM (tree_item));
 
   gtk_item_select (GTK_ITEM (tree_item));
@@ -352,7 +326,6 @@ gtk_tree_item_select (GtkTreeItem *tree_item)
 void
 gtk_tree_item_deselect (GtkTreeItem *tree_item)
 {
-  g_return_if_fail (tree_item != NULL);
   g_return_if_fail (GTK_IS_TREE_ITEM (tree_item));
 
   gtk_item_deselect (GTK_ITEM (tree_item));
@@ -361,7 +334,6 @@ gtk_tree_item_deselect (GtkTreeItem *tree_item)
 void
 gtk_tree_item_expand (GtkTreeItem *tree_item)
 {
-  g_return_if_fail (tree_item != NULL);
   g_return_if_fail (GTK_IS_TREE_ITEM (tree_item));
 
   gtk_signal_emit (GTK_OBJECT (tree_item), tree_item_signals[EXPAND_TREE], NULL);
@@ -370,7 +342,6 @@ gtk_tree_item_expand (GtkTreeItem *tree_item)
 void
 gtk_tree_item_collapse (GtkTreeItem *tree_item)
 {
-  g_return_if_fail (tree_item != NULL);
   g_return_if_fail (GTK_IS_TREE_ITEM (tree_item));
 
   gtk_signal_emit (GTK_OBJECT (tree_item), tree_item_signals[COLLAPSE_TREE], NULL);
@@ -383,7 +354,6 @@ gtk_tree_item_add_pixmaps (GtkTreeItem *tree_item)
   GdkColormap *colormap;
   GtkTreePixmaps *pixmap_node = NULL;
 
-  g_return_if_fail (tree_item != NULL);
   g_return_if_fail (GTK_IS_TREE_ITEM (tree_item));
 
   if (tree_item->pixmaps)
@@ -442,7 +412,6 @@ gtk_tree_item_add_pixmaps (GtkTreeItem *tree_item)
 static void
 gtk_tree_item_remove_pixmaps (GtkTreeItem *tree_item)
 {
-  g_return_if_fail (tree_item != NULL);
   g_return_if_fail (GTK_IS_TREE_ITEM (tree_item));
 
   if (tree_item->pixmaps)
@@ -471,7 +440,6 @@ gtk_tree_item_remove_pixmaps (GtkTreeItem *tree_item)
 static void
 gtk_tree_item_realize (GtkWidget *widget)
 {    
-  g_return_if_fail (widget != NULL);
   g_return_if_fail (GTK_IS_TREE_ITEM (widget));
 
   if (GTK_WIDGET_CLASS (parent_class)->realize)
@@ -491,7 +459,6 @@ gtk_tree_item_size_request (GtkWidget      *widget,
   GtkTreeItem* item;
   GtkRequisition child_requisition;
 
-  g_return_if_fail (widget != NULL);
   g_return_if_fail (GTK_IS_TREE_ITEM (widget));
   g_return_if_fail (requisition != NULL);
 
@@ -499,7 +466,7 @@ gtk_tree_item_size_request (GtkWidget      *widget,
   item = GTK_TREE_ITEM(widget);
 
   requisition->width = (GTK_CONTAINER (widget)->border_width +
-			widget->style->klass->xthickness) * 2;
+			widget->style->xthickness) * 2;
   requisition->height = GTK_CONTAINER (widget)->border_width * 2;
 
   if (bin->child && GTK_WIDGET_VISIBLE (bin->child))
@@ -530,7 +497,6 @@ gtk_tree_item_size_allocate (GtkWidget     *widget,
   gint border_width;
   int temp;
 
-  g_return_if_fail (widget != NULL);
   g_return_if_fail (GTK_IS_TREE_ITEM (widget));
   g_return_if_fail (allocation != NULL);
 
@@ -546,7 +512,7 @@ gtk_tree_item_size_allocate (GtkWidget     *widget,
   if (bin->child)
     {
       border_width = (GTK_CONTAINER (widget)->border_width +
-		      widget->style->klass->xthickness);
+		      widget->style->xthickness);
 
       child_allocation.x = border_width + GTK_TREE(widget->parent)->current_indent;
       child_allocation.y = GTK_CONTAINER (widget)->border_width;
@@ -578,7 +544,6 @@ gtk_tree_item_draw_lines (GtkWidget *widget)
   guint lx1, ly1, lx2, ly2;
   GdkGC* gc;
 
-  g_return_if_fail (widget != NULL);
   g_return_if_fail (GTK_IS_TREE_ITEM (widget));
 
   item = GTK_TREE_ITEM(widget);
@@ -646,7 +611,6 @@ gtk_tree_item_paint (GtkWidget    *widget,
   GdkRectangle child_area, item_area;
   GtkTreeItem* tree_item;
 
-  g_return_if_fail (widget != NULL);
   g_return_if_fail (GTK_IS_TREE_ITEM (widget));
   g_return_if_fail (area != NULL);
 
@@ -670,7 +634,7 @@ gtk_tree_item_paint (GtkWidget    *widget,
 	{
 	  if (!GTK_WIDGET_IS_SENSITIVE (widget)) 
 	    gtk_paint_flat_box(widget->style, widget->window,
-			       widget->state, GTK_STATE_INSENSITIVE,
+			       widget->state, GTK_SHADOW_NONE,
 			       area, widget, "treeitem",
 			       0, 0, -1, -1);
 	  else
@@ -700,54 +664,20 @@ gtk_tree_item_paint (GtkWidget    *widget,
 	}
 
       if (GTK_WIDGET_HAS_FOCUS (widget))
-	gtk_paint_focus (widget->style, widget->window,
+	gtk_paint_focus (widget->style, widget->window, GTK_WIDGET_STATE (widget),
 			 NULL, widget, "treeitem",
 			 0, 0,
-			 widget->allocation.width - 1,
-			 widget->allocation.height - 1);
+			 widget->allocation.width,
+			 widget->allocation.height);
       
     }
 }
 
-static void
-gtk_tree_item_draw (GtkWidget    *widget,
-		    GdkRectangle *area)
-{
-  GtkBin *bin;
-  GdkRectangle child_area;
-
-  g_return_if_fail (widget != NULL);
-  g_return_if_fail (GTK_IS_TREE_ITEM (widget));
-  g_return_if_fail (area != NULL);
-
-  if (GTK_WIDGET_DRAWABLE (widget))
-    {
-      bin = GTK_BIN (widget);
-
-      gtk_tree_item_paint (widget, area);
-     
-      if (bin->child && 
-	  gtk_widget_intersect (bin->child, area, &child_area))
-	gtk_widget_draw (bin->child, &child_area);
-
-    }
-}
-
-static void
-gtk_tree_item_draw_focus (GtkWidget *widget)
-{
-  g_return_if_fail (widget != NULL);
-  g_return_if_fail (GTK_IS_TREE_ITEM (widget));
-
-  gtk_widget_draw(widget, NULL);
-}
-
 static gint
 gtk_tree_item_button_press (GtkWidget      *widget,
 			    GdkEventButton *event)
 {
 
-  g_return_val_if_fail (widget != NULL, FALSE);
   g_return_val_if_fail (GTK_IS_TREE_ITEM (widget), FALSE);
   g_return_val_if_fail (event != NULL, FALSE);
 
@@ -756,72 +686,69 @@ gtk_tree_item_button_press (GtkWidget      *widget,
      	&& !GTK_WIDGET_HAS_FOCUS (widget))
       gtk_widget_grab_focus (widget);
 
-  return FALSE;
+  return (event->type == GDK_BUTTON_PRESS && GTK_WIDGET_IS_SENSITIVE(widget));
+}
+
+static void
+gtk_tree_item_expose_child (GtkWidget *child,
+                            gpointer   client_data)
+{
+  struct {
+    GtkWidget *container;
+    GdkEventExpose *event;
+  } *data = client_data;
+  GdkEventExpose child_event;
+
+  if (GTK_WIDGET_DRAWABLE (child) &&
+      GTK_WIDGET_NO_WINDOW (child) &&
+      (child->window == data->event->window))
+    {
+      child_event = *data->event;
+
+      child_event.region = gtk_widget_region_intersect (child,
+                                                        data->event->region);
+      if (!gdk_region_empty (child_event.region))
+        {
+          gdk_region_get_clipbox (child_event.region, &child_event.area);
+          gtk_widget_send_expose (child, (GdkEvent *) &child_event);
+	}
+      gdk_region_destroy (child_event.region);
+    }
 }
 
 static gint
 gtk_tree_item_expose (GtkWidget      *widget,
 		      GdkEventExpose *event)
 {
-  GdkEventExpose child_event;
-  GtkBin *bin;
-
-  g_return_val_if_fail (widget != NULL, FALSE);
+  struct {
+    GtkWidget *container;
+    GdkEventExpose *event;
+  } data;
+  
   g_return_val_if_fail (GTK_IS_TREE_ITEM (widget), FALSE);
   g_return_val_if_fail (event != NULL, FALSE);
 
   if (GTK_WIDGET_DRAWABLE (widget))
     {
-      bin = GTK_BIN (widget);
-      
       gtk_tree_item_paint (widget, &event->area);
 
-      child_event = *event;
-      if (bin->child && GTK_WIDGET_NO_WINDOW (bin->child) &&
-	  gtk_widget_intersect (bin->child, &event->area, &child_event.area))
-	gtk_widget_event (bin->child, (GdkEvent*) &child_event);
+      data.container = widget;
+      data.event = event;
+
+      gtk_container_forall (GTK_CONTAINER (widget),
+                            gtk_tree_item_expose_child,
+                            &data);
    }
 
   return FALSE;
 }
 
-static gint
-gtk_tree_item_focus_in (GtkWidget     *widget,
-			GdkEventFocus *event)
-{
-  g_return_val_if_fail (widget != NULL, FALSE);
-  g_return_val_if_fail (GTK_IS_TREE_ITEM (widget), FALSE);
-  g_return_val_if_fail (event != NULL, FALSE);
-
-  GTK_WIDGET_SET_FLAGS (widget, GTK_HAS_FOCUS);
-  gtk_widget_draw_focus (widget);
-
-
-  return FALSE;
-}
-
-static gint
-gtk_tree_item_focus_out (GtkWidget     *widget,
-			 GdkEventFocus *event)
-{
-  g_return_val_if_fail (widget != NULL, FALSE);
-  g_return_val_if_fail (GTK_IS_TREE_ITEM (widget), FALSE);
-  g_return_val_if_fail (event != NULL, FALSE);
-
-  GTK_WIDGET_UNSET_FLAGS (widget, GTK_HAS_FOCUS);
-  gtk_widget_draw_focus (widget);
-
-
-  return FALSE;
-}
-
 static void
 gtk_real_tree_item_select (GtkItem *item)
 {    
   GtkTreeItem *tree_item;
   GtkWidget *widget;
 
-  g_return_if_fail (item != NULL);
   g_return_if_fail (GTK_IS_TREE_ITEM (item));
 
   tree_item = GTK_TREE_ITEM (item);
@@ -839,7 +766,6 @@ gtk_real_tree_item_deselect (GtkItem *item)
   GtkTreeItem *tree_item;
   GtkWidget *widget;
 
-  g_return_if_fail (item != NULL);
   g_return_if_fail (GTK_IS_TREE_ITEM (item));
 
   tree_item = GTK_TREE_ITEM (item);
@@ -854,7 +780,6 @@ gtk_real_tree_item_deselect (GtkItem *item)
 static void
 gtk_real_tree_item_toggle (GtkItem *item)
 {
-  g_return_if_fail (item != NULL);
   g_return_if_fail (GTK_IS_TREE_ITEM (item));
 
   if(!GTK_WIDGET_IS_SENSITIVE(item))
@@ -882,7 +807,6 @@ gtk_real_tree_item_expand (GtkTreeItem *tree_item)
 {
   GtkTree* tree;
   
-  g_return_if_fail (tree_item != NULL);
   g_return_if_fail (GTK_IS_TREE_ITEM (tree_item));
   
   if (tree_item->subtree && !tree_item->expanded)
@@ -911,7 +835,6 @@ gtk_real_tree_item_collapse (GtkTreeItem *tree_item)
 {
   GtkTree* tree;
   
-  g_return_if_fail (tree_item != NULL);
   g_return_if_fail (GTK_IS_TREE_ITEM (tree_item));
   
   if (tree_item->subtree && tree_item->expanded) 
@@ -941,7 +864,6 @@ gtk_tree_item_destroy (GtkObject *object)
   GtkTreeItem* item;
   GtkWidget* child;
 
-  g_return_if_fail (object != NULL);
   g_return_if_fail (GTK_IS_TREE_ITEM (object));
 
 #ifdef TREE_DEBUG
@@ -1004,7 +926,6 @@ gtk_tree_item_destroy (GtkObject *object)
 void
 gtk_tree_item_remove_subtree (GtkTreeItem* item) 
 {
-  g_return_if_fail (item != NULL);
   g_return_if_fail (GTK_IS_TREE_ITEM(item));
   g_return_if_fail (item->subtree != NULL);
   
@@ -1047,7 +968,6 @@ gtk_tree_item_map (GtkWidget *widget)
   GtkBin *bin;
   GtkTreeItem* item;
 
-  g_return_if_fail (widget != NULL);
   g_return_if_fail (GTK_IS_TREE_ITEM (widget));
 
   bin = GTK_BIN (widget);
@@ -1074,7 +994,6 @@ gtk_tree_item_unmap (GtkWidget *widget)
   GtkBin *bin;
   GtkTreeItem* item;
 
-  g_return_if_fail (widget != NULL);
   g_return_if_fail (GTK_IS_TREE_ITEM (widget));
 
   GTK_WIDGET_UNSET_FLAGS (widget, GTK_MAPPED);
@@ -1103,7 +1022,6 @@ gtk_tree_item_forall (GtkContainer *container,
   GtkBin *bin;
   GtkTreeItem *tree_item;
 
-  g_return_if_fail (container != NULL);
   g_return_if_fail (GTK_IS_TREE_ITEM (container));
   g_return_if_fail (callback != NULL);
 
