@@ -199,6 +199,39 @@ gtk_tree_path_new_from_string (const gchar *path)
   return retval;
 }
 
+/**
+ * gtk_tree_path_new_from_indices:
+ * @first_index: first integer
+ * @varargs: list of integers terminated by -1
+ *
+ * Creates a new path with @first_index and @varargs as indices.
+ *
+ * Return value: A newly created GtkTreePath.
+ **/
+GtkTreePath *
+gtk_tree_path_new_from_indices (gint first_index,
+				...)
+{
+  int arg;
+  va_list args;
+  GtkTreePath *path;
+
+  path = gtk_tree_path_new ();
+
+  va_start (args, first_index);
+  arg = first_index;
+
+  while (arg != -1)
+    {
+      gtk_tree_path_append_index (path, arg);
+      arg = va_arg (args, gint);
+    }
+
+  va_end (args);
+
+  return path;
+}
+
 /**
  * gtk_tree_path_to_string:
  * @path: A #GtkTreePath
@@ -284,7 +317,11 @@ void
 gtk_tree_path_prepend_index (GtkTreePath *path,
 			     gint       index)
 {
-  gint *new_indices = g_new (gint, ++path->depth);
+  gint *new_indices;
+
+  (path->depth)++;
+  new_indices = g_new (gint, path->depth);
+
   if (path->indices == NULL)
     {
       path->indices = new_indices;
@@ -721,6 +758,36 @@ gtk_tree_model_get_iter_from_string (GtkTreeModel *tree_model,
   return retval;
 }
 
+/**
+ * gtk_tree_model_get_string_from_iter:
+ * @tree_model: A #GtkTreeModel.
+ * @iter: An #GtkTreeIter.
+ *
+ * Generates a string representation of the iter. This string is a ':'
+ * separated list of numbers. For example, "4:10:0:3" would be an
+ * acceptable return value for this string.
+ *
+ * Return value: A newly-allocated string. Must be freed with g_free().
+ **/
+gchar *
+gtk_tree_model_get_string_from_iter (GtkTreeModel *tree_model,
+                                     GtkTreeIter  *iter)
+{
+  GtkTreePath *path;
+  gchar *ret;
+
+  g_return_val_if_fail (GTK_IS_TREE_MODEL (tree_model), NULL);
+  g_return_val_if_fail (iter != NULL, NULL);
+
+  path = gtk_tree_model_get_path (tree_model, iter);
+
+  g_return_val_if_fail (path != NULL, NULL);
+
+  ret = gtk_tree_path_to_string (path);
+  gtk_tree_path_free (path);
+
+  return ret;
+}
 
 /**
  * gtk_tree_model_get_iter_first:
@@ -819,12 +886,15 @@ gtk_tree_model_iter_next (GtkTreeModel  *tree_model,
  * gtk_tree_model_iter_children:
  * @tree_model: A #GtkTreeModel.
  * @iter: The new #GtkTreeIter to be set to the child.
- * @parent: The #GtkTreeIter.
+ * @parent: The #GtkTreeIter, or %NULL
  *
  * Sets @iter to point to the first child of @parent.  If @parent has no children,
  * %FALSE is returned and @iter is set to be invalid.  @parent will remain a valid
  * node after this function has been called.
  *
+ * If @parent is %NULL returns the first node, equivalent to
+ * <literal>gtk_tree_model_get_iter_first (tree_model, iter);</literal>
+ *
  * Return value: %TRUE, if @child has been set to the first child.
  **/
 gboolean
@@ -892,9 +962,9 @@ gtk_tree_model_iter_n_children (GtkTreeModel *tree_model,
  * index is 0.  If @index is too big, or @parent has no children, @iter is set
  * to an invalid iterator and %FALSE is returned.  @parent will remain a valid
  * node after this function has been called.  As a special case, if @parent is
- * %NULL, then the @n<!>th root node is set.
+ * %NULL, then the @n<!-- -->th root node is set.
  *
- * Return value: %TRUE, if @parent has an @n<!>th child.
+ * Return value: %TRUE, if @parent has an @n<!-- -->th child.
  **/
 gboolean
 gtk_tree_model_iter_nth_child (GtkTreeModel *tree_model,
@@ -1252,6 +1322,20 @@ static void gtk_tree_row_reference_unref_path (GtkTreePath  *path,
 					       gint          depth);
 
 
+GType
+gtk_tree_row_reference_get_type (void)
+{
+  static GType our_type = 0;
+  
+  if (our_type == 0)
+    our_type = g_boxed_type_register_static ("GtkTreeRowReference",
+					     (GBoxedCopyFunc) gtk_tree_row_reference_copy,
+					     (GBoxedFreeFunc) gtk_tree_row_reference_free);
+
+  return our_type;
+}
+
+
 #define ROW_REF_DATA_STRING "gtk-tree-row-refs"
 
 struct _GtkTreeRowReference
@@ -1675,6 +1759,15 @@ gtk_tree_row_reference_valid (GtkTreeRowReference *reference)
   return TRUE;
 }
 
+
+GtkTreeRowReference *
+gtk_tree_row_reference_copy (GtkTreeRowReference *reference)
+{
+  return gtk_tree_row_reference_new_proxy (reference->proxy,
+					   reference->model,
+					   reference->path);
+}
+
 /**
  * gtk_tree_row_reference_free:
  * @reference: A #GtkTreeRowReference, or NULL
