@@ -31,6 +31,7 @@ G_BEGIN_DECLS
 
 #define GTK_TYPE_TREE_ITER             (gtk_tree_iter_get_type ())
 #define GTK_TYPE_TREE_PATH             (gtk_tree_path_get_type ())
+#define GTK_TYPE_TREE_ROW_REFERENCE    (gtk_tree_row_reference_get_type ())
 
 typedef struct _GtkTreeIter         GtkTreeIter;
 typedef struct _GtkTreePath         GtkTreePath;
@@ -116,6 +117,8 @@ struct _GtkTreeModelIface
 /* GtkTreePath operations */
 GtkTreePath *gtk_tree_path_new              (void);
 GtkTreePath *gtk_tree_path_new_from_string  (const gchar       *path);
+GtkTreePath *gtk_tree_path_new_from_indices (gint               first_index,
+					     ...);
 gchar       *gtk_tree_path_to_string        (GtkTreePath       *path);
 #define gtk_tree_path_new_root() gtk_tree_path_new_first()
 GtkTreePath *gtk_tree_path_new_first        (void);
@@ -152,6 +155,7 @@ GtkTreeRowReference *gtk_tree_row_reference_new_proxy (GObject             *prox
 						       GtkTreePath         *path);
 GtkTreePath         *gtk_tree_row_reference_get_path  (GtkTreeRowReference *reference);
 gboolean             gtk_tree_row_reference_valid     (GtkTreeRowReference *reference);
+GtkTreeRowReference *gtk_tree_row_reference_copy      (GtkTreeRowReference *reference);
 void                 gtk_tree_row_reference_free      (GtkTreeRowReference *reference);
 /* These two functions are only needed if you created the row reference with a
  * proxy object */
@@ -183,6 +187,8 @@ gboolean          gtk_tree_model_get_iter        (GtkTreeModel *tree_model,
 gboolean          gtk_tree_model_get_iter_from_string (GtkTreeModel *tree_model,
 						       GtkTreeIter  *iter,
 						       const gchar  *path_string);
+gchar *           gtk_tree_model_get_string_from_iter (GtkTreeModel *tree_model,
+                                                       GtkTreeIter  *iter);
 #define gtk_tree_model_get_iter_root(tree_model, iter) gtk_tree_model_get_iter_first(tree_model, iter)
 gboolean          gtk_tree_model_get_iter_first  (GtkTreeModel *tree_model,
 						  GtkTreeIter  *iter);
