@@ -101,6 +101,8 @@ enum {
 
 #define NO_SORT_FUNC ((GtkTreeIterCompareFunc) 0x1)
 
+#define VALID_ITER(iter, tree_model_sort) (iter != NULL && iter->user_data != NULL && iter->user_data2 != NULL && tree_model_sort->stamp == iter->stamp)
+
 /* general (object/interface init, etc) */
 static void gtk_tree_model_sort_init                  (GtkTreeModelSort      *tree_model_sort);
 static void gtk_tree_model_sort_class_init            (GtkTreeModelSortClass *tree_model_sort_class);
@@ -371,7 +373,7 @@ gtk_tree_model_sort_finalize (GObject *object)
   if (tree_model_sort->sort_list)
     {
       _gtk_tree_data_list_header_free (tree_model_sort->sort_list);
-      g_free (tree_model_sort->sort_list);
+      tree_model_sort->sort_list = NULL;
     }
 
   /* must chain up */
@@ -433,7 +435,7 @@ gtk_tree_model_sort_row_changed (GtkTreeModel *s_model,
 
   gboolean free_s_path = FALSE;
 
-  gint offset, index = 0, old_index, i;
+  gint offset, index = 0, i;
 
   g_return_if_fail (start_s_path != NULL || start_s_iter != NULL);
 
@@ -487,8 +489,6 @@ gtk_tree_model_sort_row_changed (GtkTreeModel *s_model,
   memcpy (&tmp, elt, sizeof (SortElt));
   g_array_remove_index (level->array, index);
 
-  old_index = index;
-
   if (GTK_TREE_MODEL_SORT_CACHE_CHILD_ITERS (tree_model_sort))
     index = gtk_tree_model_sort_level_find_insert (tree_model_sort,
 						   level,
@@ -510,66 +510,8 @@ gtk_tree_model_sort_row_changed (GtkTreeModel *s_model,
   gtk_tree_path_append_index (path, index);
 
   gtk_tree_model_sort_increment_stamp (tree_model_sort);
-
-  /* if the item moved, then emit rows_reordered */
-  if (old_index != index)
-    {
-      gint *new_order;
-      gint j;
-
-      GtkTreePath *tmppath;
-
-      new_order = g_new (gint, level->array->len);
-
-      for (j = 0; j < level->array->len; j++)
-        {
-	  if (index > old_index)
-	    {
-	      if (j == index)
-		new_order[j] = old_index;
-	      else if (j >= old_index && j < index)
-		new_order[j] = j + 1;
-	      else
-		new_order[j] = j;
-	    }
-	  else if (index < old_index)
-	    {
-	      if (j == index)
-		new_order[j] = old_index;
-	      else if (j > index && j <= old_index)
-		new_order[j] = j - 1;
-	      else
-		new_order[j] = j;
-	    }
-	  /* else? shouldn't really happen */
-	}
-
-      if (level->parent_elt)
-        {
-	  iter.stamp = tree_model_sort->stamp;
-	  iter.user_data = level->parent_level;
-	  iter.user_data2 = level->parent_elt;
-
-	  tmppath = gtk_tree_model_get_path (GTK_TREE_MODEL (tree_model_sort), &iter);
-
-	  gtk_tree_model_rows_reordered (GTK_TREE_MODEL (tree_model_sort),
-	                                 tmppath, &iter, new_order);
-	}
-      else
-        {
-	  /* toplevel */
-	  tmppath = gtk_tree_path_new ();
-
-          gtk_tree_model_rows_reordered (GTK_TREE_MODEL (tree_model_sort), tmppath,
-	                                 NULL, new_order);
-	}
-
-      gtk_tree_path_free (tmppath);
-      g_free (new_order);
-    }
-
-  /* emit row_changed signal (at new location) */
   gtk_tree_model_get_iter (GTK_TREE_MODEL (data), &iter, path);
+
   gtk_tree_model_row_changed (GTK_TREE_MODEL (data), path, &iter);
 
   gtk_tree_path_free (path);
@@ -1232,7 +1174,6 @@ gtk_tree_model_sort_real_unref_node (GtkTreeModel *tree_model,
 
   elt->ref_count--;
   level->ref_count--;
-
   if (level->ref_count == 0)
     {
       SortLevel *parent_level = level->parent_level;
@@ -1826,6 +1767,7 @@ gtk_tree_model_sort_set_model (GtkTreeModelSort *tree_model_sort,
       GType *types;
       gint i, n_columns;
 
+      g_object_ref (tree_model_sort->child_model);
       tree_model_sort->changed_id =
         g_signal_connect (child_model, "row_changed",
                           G_CALLBACK (gtk_tree_model_sort_row_changed),
@@ -2311,3 +2253,50 @@ gtk_tree_model_sort_clear_cache (GtkTreeModelSort *tree_model_sort)
   if (tree_model_sort->zero_ref_count)
     gtk_tree_model_sort_clear_cache_helper (tree_model_sort, (SortLevel *)tree_model_sort->root);
 }
+
+static gboolean
+gtk_tree_model_sort_iter_is_valid_helper (GtkTreeIter *iter,
+					  SortLevel   *level)
+{
+  gint i;
+
+  for (i = 0; i < level->array->len; i++)
+    {
+      SortElt *elt = &g_array_index (level->array, SortElt, i);
+
+      if (iter->user_data == level && iter->user_data2 == elt)
+	return TRUE;
+
+      if (elt->children)
+	if (gtk_tree_model_sort_iter_is_valid_helper (iter, elt->children))
+	  return TRUE;
+    }
+
+  return FALSE;
+}
+
+/**
+ * gtk_tree_model_sort_iter_is_valid:
+ * @tree_model_sort: A #GtkTreeModelSort.
+ * @iter: A #GtkTreeIter.
+ *
+ * WARNING: This function is slow. Only use it for debugging and/or testing
+ * purposes.
+ *
+ * Checks if the given iter is a valid iter for this #GtkTreeModelSort.
+ *
+ * Return value: %TRUE if the iter is valid, %FALSE if the iter is invalid.
+ **/
+gboolean
+gtk_tree_model_sort_iter_is_valid (GtkTreeModelSort *tree_model_sort,
+                                   GtkTreeIter      *iter)
+{
+  g_return_val_if_fail (GTK_IS_TREE_MODEL_SORT (tree_model_sort), FALSE);
+  g_return_val_if_fail (iter != NULL, FALSE);
+
+  if (!VALID_ITER (iter, tree_model_sort))
+    return FALSE;
+
+  return gtk_tree_model_sort_iter_is_valid_helper (iter,
+						   tree_model_sort->root);
+}
