@@ -334,7 +334,7 @@ gtk_tree_selection_get_tree_view (GtkTreeSelection *selection)
 /**
  * gtk_tree_selection_get_selected:
  * @selection: A #GtkTreeSelection.
- * @model: A pointer set to the #GtkTreeModel, or NULL.
+ * @model: A pointer to set to the #GtkTreeModel, or NULL.
  * @iter: The #GtkTreeIter, or NULL.
  *
  * Sets @iter to the currently selected node if @selection is set to
@@ -403,13 +403,179 @@ gtk_tree_selection_get_selected (GtkTreeSelection  *selection,
   return retval;
 }
 
-/* gtk_tree_selection_selected_foreach helper */
-static void
-model_changed (gpointer data)
+/**
+ * gtk_tree_selection_get_selected_rows:
+ * @selection: A #GtkTreeSelection.
+ * @model: A pointer to set to the #GtkTreeModel, or NULL.
+ *
+ * Creates a list of path of all selected rows. Additionally, if you are
+ * planning on modifying the model after calling this function, you may
+ * want to convert the returned list into a list of #GtkTreeRowReferences.
+ * To do this, you can use gtk_tree_row_reference_new_proxy().
+ *
+ * To free the return value, use:
+ * <informalexample><programlisting>
+ * g_list_foreach (list, gtk_tree_path_free, NULL);
+ * g_list_free (list);
+ * </programlisting></informalexample>
+ *
+ * Return value: A #GList containing a #GtkTreePath for each selected row.
+ **/
+GList *
+gtk_tree_selection_get_selected_rows (GtkTreeSelection   *selection,
+                                      GtkTreeModel      **model)
 {
-  gboolean *stop = (gboolean *)data;
+  GList *list = NULL;
+  GtkRBTree *tree = NULL;
+  GtkRBNode *node = NULL;
+  GtkTreePath *path;
 
-  *stop = TRUE;
+  g_return_val_if_fail (GTK_IS_TREE_SELECTION (selection), NULL);
+  g_return_val_if_fail (selection->tree_view != NULL, NULL);
+  g_return_val_if_fail (selection->tree_view->priv->model != NULL, NULL);
+
+  if (selection->tree_view->priv->tree == NULL ||
+      selection->tree_view->priv->tree->root == NULL)
+    return NULL;
+
+  if (model)
+    *model = selection->tree_view->priv->model;
+
+  if (selection->type == GTK_SELECTION_NONE)
+    return NULL;
+  else if (selection->type != GTK_SELECTION_MULTIPLE)
+    {
+      GtkTreeIter iter;
+
+      if (gtk_tree_selection_get_selected (selection, NULL, &iter))
+        {
+	  GtkTreePath *path;
+
+	  path = gtk_tree_model_get_path (*model, &iter);
+	  list = g_list_append (list, path);
+
+	  return list;
+	}
+
+      return NULL;
+    }
+
+  tree = selection->tree_view->priv->tree;
+  node = selection->tree_view->priv->tree->root;
+
+  while (node->left != tree->nil)
+    node = node->left;
+  path = gtk_tree_path_new_first ();
+
+  do
+    {
+      if (GTK_RBNODE_FLAG_SET (node, GTK_RBNODE_IS_SELECTED))
+	list = g_list_append (list, gtk_tree_path_copy (path));
+
+      if (node->children)
+        {
+	  tree = node->children;
+	  node = tree->root;
+
+	  while (node->left != tree->nil)
+	    node = node->left;
+
+	  gtk_tree_path_append_index (path, 0);
+	}
+      else
+        {
+	  gboolean done = FALSE;
+
+	  do
+	    {
+	      node = _gtk_rbtree_next (tree, node);
+	      if (node != NULL)
+	        {
+		  done = TRUE;
+		  gtk_tree_path_next (path);
+		}
+	      else
+	        {
+		  node = tree->parent_node;
+		  tree = tree->parent_tree;
+
+		  if (!tree)
+		    {
+		      gtk_tree_path_free (path);
+		      return list;
+		    }
+
+		  gtk_tree_path_up (path);
+		}
+	    }
+	  while (!done);
+	}
+    }
+  while (TRUE);
+
+  gtk_tree_path_free (path);
+
+  return list;
+}
+
+static void
+gtk_tree_selection_count_selected_rows_helper (GtkRBTree *tree,
+					       GtkRBNode *node,
+					       gpointer   data)
+{
+  gint *count = (gint *)data;
+
+  if (GTK_RBNODE_FLAG_SET (node, GTK_RBNODE_IS_SELECTED))
+    (*count)++;
+
+  if (node->children)
+    _gtk_rbtree_traverse (node->children, node->children->root,
+			  G_PRE_ORDER,
+			  gtk_tree_selection_count_selected_rows_helper, data);
+}
+
+/**
+ * gtk_tree_selection_count_selected_rows:
+ * @selection: A #GtkTreeSelection.
+ *
+ * Returns the number of rows that have been selected in @tree.
+ *
+ * Return value: The number of rows selected.
+ **/
+gint
+gtk_tree_selection_count_selected_rows (GtkTreeSelection *selection)
+{
+  gint count = 0;
+  GtkRBTree *tree;
+  GtkRBNode *node;
+
+  g_return_val_if_fail (GTK_IS_TREE_SELECTION (selection), 0);
+  g_return_val_if_fail (selection->tree_view != NULL, 0);
+  g_return_val_if_fail (selection->tree_view->priv->model != NULL, 0);
+
+  if (selection->tree_view->priv->tree == NULL ||
+      selection->tree_view->priv->tree->root == NULL)
+    return 0;
+
+  if (selection->type == GTK_SELECTION_SINGLE ||
+      selection->type == GTK_SELECTION_BROWSE)
+    {
+      if (gtk_tree_selection_get_selected (selection, NULL, NULL))
+	return 1;
+      else
+	return 0;
+    }
+
+  tree = selection->tree_view->priv->tree;
+  node = selection->tree_view->priv->tree->root;
+
+  _gtk_rbtree_traverse (selection->tree_view->priv->tree,
+                        selection->tree_view->priv->tree->root,
+			G_PRE_ORDER,
+			gtk_tree_selection_count_selected_rows_helper,
+			&count);
+
+  return count;
 }
 
 /**
@@ -418,7 +584,9 @@ model_changed (gpointer data)
  * @func: The function to call for each selected node.
  * @data: user data to pass to the function.
  *
- * Calls a function for each selected node.
+ * Calls a function for each selected node. Note that you cannot modify
+ * the tree or selection from within this function. As a result,
+ * gtk_tree_selection_get_selected_rows() might be more useful.
  **/
 void
 gtk_tree_selection_selected_foreach (GtkTreeSelection            *selection,
@@ -430,9 +598,6 @@ gtk_tree_selection_selected_foreach (GtkTreeSelection            *selection,
   GtkRBNode *node;
   GtkTreeIter iter;
 
-  guint inserted_id, deleted_id, reordered_id;
-  gboolean stop = FALSE, has_next = TRUE, has_parent = TRUE;
-
   g_return_if_fail (GTK_IS_TREE_SELECTION (selection));
   g_return_if_fail (selection->tree_view != NULL);
   g_return_if_fail (selection->tree_view->priv->model != NULL);
@@ -461,20 +626,6 @@ gtk_tree_selection_selected_foreach (GtkTreeSelection            *selection,
   while (node->left != tree->nil)
     node = node->left;
 
-  /* connect to signals to monitor changes in treemodel */
-  inserted_id = g_signal_connect_swapped (selection->tree_view->priv->model,
-                                          "row_inserted",
-					  G_CALLBACK (model_changed),
-				          &stop);
-  deleted_id = g_signal_connect_swapped (selection->tree_view->priv->model,
-                                         "row_deleted",
-					 G_CALLBACK (model_changed),
-				         &stop);
-  reordered_id = g_signal_connect_swapped (selection->tree_view->priv->model,
-                                           "rows_reordered",
-					   G_CALLBACK (model_changed),
-				           &stop);
-
   /* find the node internally */
   path = gtk_tree_path_new_first ();
   gtk_tree_model_get_iter (selection->tree_view->priv->model,
@@ -484,10 +635,6 @@ gtk_tree_selection_selected_foreach (GtkTreeSelection            *selection,
     {
       if (GTK_RBNODE_FLAG_SET (node, GTK_RBNODE_IS_SELECTED))
 	(* func) (selection->tree_view->priv->model, path, &iter, data);
-
-      if (stop)
-	goto out;
-
       if (node->children)
 	{
 	  gboolean has_child;
@@ -500,10 +647,8 @@ gtk_tree_selection_selected_foreach (GtkTreeSelection            *selection,
 	  tmp = iter;
 	  has_child = gtk_tree_model_iter_children (selection->tree_view->priv->model, &iter, &tmp);
 	  gtk_tree_path_append_index (path, 0);
-
-	  /* we do the sanity check at the bottom of this function */
-	  if (!has_child)
-	    goto out;
+	  /* Sanity Check! */
+	  TREE_VIEW_INTERNAL_ASSERT_VOID (has_child);
 	}
       else
 	{
@@ -519,9 +664,8 @@ gtk_tree_selection_selected_foreach (GtkTreeSelection            *selection,
 		  done = TRUE;
 		  gtk_tree_path_next (path);
 
-		  /* we do the sanity check at the bottom of this function */
-		  if (!has_next)
-		    goto out;
+		  /* Sanity Check! */
+		  TREE_VIEW_INTERNAL_ASSERT_VOID (has_next);
 		}
 	      else
 		{
@@ -532,47 +676,21 @@ gtk_tree_selection_selected_foreach (GtkTreeSelection            *selection,
 		  tree = tree->parent_tree;
 		  if (tree == NULL)
 		    {
+		      gtk_tree_path_free (path);
 		      /* we've run out of tree */
 		      /* We're done with this function */
-
-		      goto out;
+		      return;
 		    }
-
 		  has_parent = gtk_tree_model_iter_parent (selection->tree_view->priv->model, &iter, &tmp_iter);
 		  gtk_tree_path_up (path);
-
-		  /* we do the sanity check at the bottom of this function */
-		  if (!has_parent)
-		    goto out;
+		  /* Sanity check */
+		  TREE_VIEW_INTERNAL_ASSERT_VOID (has_parent);
 		}
 	    }
 	  while (!done);
 	}
     }
   while (TRUE);
-
-out:
-  if (path)
-    gtk_tree_path_free (path);
-
-  g_signal_handler_disconnect (selection->tree_view->priv->model,
-                               inserted_id);
-  g_signal_handler_disconnect (selection->tree_view->priv->model,
-                               deleted_id);
-  g_signal_handler_disconnect (selection->tree_view->priv->model,
-                               reordered_id);
-
-  /* check if we have to spew a scary message */
-  if (!has_next)
-    TREE_VIEW_INTERNAL_ASSERT_VOID (has_next);
-  if (!has_parent)
-    TREE_VIEW_INTERNAL_ASSERT_VOID (has_parent);
-  if (stop)
-    g_warning
-      ("The model has been modified from within gtk_tree_selection_foreach.\n"
-       "This function is for observing the selections of the tree only.  If\n"
-       "you are trying to get all selected items from the tree, try using\n"
-       "gtk_tree_selection_get_selected_rows instead.\n");
 }
 
 /**
@@ -748,7 +866,7 @@ gtk_tree_selection_path_is_selected (GtkTreeSelection *selection,
  * @selection: A #GtkTreeSelection
  * @iter: A valid #GtkTreeIter
  * 
- * Returns %TRUE if the row pointed to by @path is currently selected.
+ * Returns %TRUE if the row at @iter is currently selected.
  * 
  * Return value: %TRUE, if @iter is selected
  **/
@@ -954,14 +1072,20 @@ gtk_tree_selection_unselect_all (GtkTreeSelection *selection)
     g_signal_emit (G_OBJECT (selection), tree_selection_signals[CHANGED], 0);
 }
 
+enum
+{
+  RANGE_SELECT,
+  RANGE_UNSELECT
+};
+
 static gint
-gtk_tree_selection_real_select_range (GtkTreeSelection *selection,
+gtk_tree_selection_real_modify_range (GtkTreeSelection *selection,
+                                      gint              mode,
 				      GtkTreePath      *start_path,
 				      GtkTreePath      *end_path)
 {
   GtkRBNode *start_node, *end_node;
   GtkRBTree *start_tree, *end_tree;
-  GtkTreePath *anchor_path;
   gboolean dirty = FALSE;
 
   switch (gtk_tree_path_compare (start_path, end_path))
@@ -975,7 +1099,6 @@ gtk_tree_selection_real_select_range (GtkTreeSelection *selection,
 				start_path,
 				&end_tree,
 				&end_node);
-      anchor_path = end_path;
       break;
     case 0:
       _gtk_tree_view_find_node (selection->tree_view,
@@ -984,7 +1107,6 @@ gtk_tree_selection_real_select_range (GtkTreeSelection *selection,
 				&start_node);
       end_tree = start_tree;
       end_node = start_node;
-      anchor_path = start_path;
       break;
     case -1:
       _gtk_tree_view_find_node (selection->tree_view,
@@ -995,27 +1117,15 @@ gtk_tree_selection_real_select_range (GtkTreeSelection *selection,
 				end_path,
 				&end_tree,
 				&end_node);
-      anchor_path = start_path;
       break;
     }
 
   g_return_val_if_fail (start_node != NULL, FALSE);
   g_return_val_if_fail (end_node != NULL, FALSE);
 
-  if (anchor_path)
-    {
-      if (selection->tree_view->priv->anchor)
-	gtk_tree_row_reference_free (selection->tree_view->priv->anchor);
-
-      selection->tree_view->priv->anchor =
-	gtk_tree_row_reference_new_proxy (G_OBJECT (selection->tree_view),
-	                                  selection->tree_view->priv->model,
-					  anchor_path);
-    }
-
   do
     {
-      dirty |= gtk_tree_selection_real_select_node (selection, start_tree, start_node, TRUE);
+      dirty |= gtk_tree_selection_real_select_node (selection, start_tree, start_node, (mode == RANGE_SELECT)?TRUE:FALSE);
 
       if (start_node == end_node)
 	break;
@@ -1059,14 +1169,36 @@ gtk_tree_selection_select_range (GtkTreeSelection *selection,
   g_return_if_fail (GTK_IS_TREE_SELECTION (selection));
   g_return_if_fail (selection->tree_view != NULL);
 
-  if (gtk_tree_selection_real_select_range (selection, start_path, end_path))
+  if (gtk_tree_selection_real_modify_range (selection, RANGE_SELECT, start_path, end_path))
+    g_signal_emit (G_OBJECT (selection), tree_selection_signals[CHANGED], 0);
+}
+
+/**
+ * gtk_tree_selection_unselect_range:
+ * @selection: A #GtkTreeSelection.
+ * @start_path: The initial node of the range.
+ * @end_path: The initial node of the range.
+ *
+ * Unselects a range of nodes, determined by @start_path and @end_path
+ * inclusive.
+ **/
+void
+gtk_tree_selection_unselect_range (GtkTreeSelection *selection,
+                                   GtkTreePath      *start_path,
+				   GtkTreePath      *end_path)
+{
+  g_return_if_fail (GTK_IS_TREE_SELECTION (selection));
+  g_return_if_fail (selection->tree_view != NULL);
+
+  if (gtk_tree_selection_real_modify_range (selection, RANGE_UNSELECT, start_path, end_path))
     g_signal_emit (G_OBJECT (selection), tree_selection_signals[CHANGED], 0);
 }
 
 /* Called internally by gtktreeview.c It handles actually selecting the tree.
  */
 
-/* docs about the 'override_browse_mode', we set this flag when we want to
+/*
+ * docs about the 'override_browse_mode', we set this flag when we want to
  * unset select the node and override the select browse mode behaviour (that is
  * 'one node should *always* be selected').
  */
@@ -1147,9 +1279,6 @@ _gtk_tree_selection_internal_select_node (GtkTreeSelection *selection,
 	      if (gtk_tree_selection_real_select_node (selection, tree, node, TRUE))
 		{
 		  dirty = TRUE;
-		  if (selection->tree_view->priv->anchor)
-		    gtk_tree_row_reference_free (selection->tree_view->priv->anchor);
-
 		  selection->tree_view->priv->anchor =
 		    gtk_tree_row_reference_new_proxy (G_OBJECT (selection->tree_view), selection->tree_view->priv->model, path);
 		}
@@ -1190,7 +1319,8 @@ _gtk_tree_selection_internal_select_node (GtkTreeSelection *selection,
       else if ((state & GDK_SHIFT_MASK) == GDK_SHIFT_MASK)
 	{
 	  dirty = gtk_tree_selection_real_unselect_all (selection);
-	  dirty |= gtk_tree_selection_real_select_range (selection,
+	  dirty |= gtk_tree_selection_real_modify_range (selection,
+                                                         RANGE_SELECT,
 							 anchor_path,
 							 path);
 	}
