@@ -68,6 +68,7 @@ struct _GtkTreeViewColumnCellInfo
   guint expand : 1;
   guint pack : 1;
   guint has_focus : 1;
+  guint in_editing_mode : 1;
 };
 
 /* Type methods */
@@ -85,7 +86,7 @@ static void gtk_tree_view_column_get_property                  (GObject
 								GParamSpec              *pspec);
 static void gtk_tree_view_column_finalize                      (GObject                 *object);
 
-/* Button handling code */ 
+/* Button handling code */
 static void gtk_tree_view_column_create_button                 (GtkTreeViewColumn       *tree_column);
 static void gtk_tree_view_column_update_button                 (GtkTreeViewColumn       *tree_column);
 
@@ -112,10 +113,17 @@ static void gtk_tree_view_column_set_attributesv               (GtkTreeViewColum
 								va_list                  args);
 static GtkTreeViewColumnCellInfo *gtk_tree_view_column_get_cell_info (GtkTreeViewColumn *tree_column,
 								      GtkCellRenderer   *cell_renderer);
+
+/* cell list manipulation */
+static GList *gtk_tree_view_column_cell_first                  (GtkTreeViewColumn      *tree_column);
+static GList *gtk_tree_view_column_cell_last                   (GtkTreeViewColumn      *tree_column);
+static GList *gtk_tree_view_column_cell_next                   (GtkTreeViewColumn      *tree_column,
+								GList                  *current);
+static GList *gtk_tree_view_column_cell_prev                   (GtkTreeViewColumn      *tree_column,
+								GList                  *current);
 static void gtk_tree_view_column_clear_attributes_by_info      (GtkTreeViewColumn      *tree_column,
 					                        GtkTreeViewColumnCellInfo *info);
 
-
 static GtkObjectClass *parent_class = NULL;
 static guint tree_column_signals[LAST_SIGNAL] = { 0 };
 
@@ -919,7 +927,7 @@ gtk_tree_view_column_setup_sort_column_id_callback (GtkTreeViewColumn *tree_colu
       GtkSortType real_order;
 
       if (tree_column->sort_column_changed_signal == 0)
-	tree_column->sort_column_changed_signal =
+        tree_column->sort_column_changed_signal =
 	  g_signal_connect (G_OBJECT (model), "sort_column_changed",
                             GTK_SIGNAL_FUNC (gtk_tree_view_model_sort_column_changed),
                             tree_column);
@@ -972,8 +980,8 @@ _gtk_tree_view_column_realize_button (GtkTreeViewColumn *column)
 		     GDK_POINTER_MOTION_HINT_MASK |
 		     GDK_KEY_PRESS_MASK);
   attributes_mask = GDK_WA_CURSOR | GDK_WA_X | GDK_WA_Y;
-  attr.cursor = gdk_cursor_new (GDK_SB_H_DOUBLE_ARROW);
-
+  attr.cursor = gdk_cursor_new_for_display (gdk_drawable_get_display (tree_view->priv->header_window),
+					    GDK_SB_H_DOUBLE_ARROW);
   attr.y = 0;
   attr.width = TREE_VIEW_DRAG_WIDTH;
   attr.height = tree_view->priv->header_height;
@@ -1000,6 +1008,16 @@ _gtk_tree_view_column_unrealize_button (GtkTreeViewColumn *column)
   column->window = NULL;
 }
 
+void
+_gtk_tree_view_column_unset_model (GtkTreeViewColumn *column,
+				   GtkTreeModel      *old_model)
+{
+  if (column->sort_column_changed_signal)
+    g_signal_handler_disconnect (G_OBJECT (old_model),
+				 column->sort_column_changed_signal);
+    column->sort_column_changed_signal = 0;
+}
+
 void
 _gtk_tree_view_column_set_tree_view (GtkTreeViewColumn *column,
 				     GtkTreeView       *tree_view)
@@ -1055,14 +1073,14 @@ _gtk_tree_view_column_has_editable_cell (GtkTreeViewColumn *column)
   return FALSE;
 }
 
+/* gets cell being edited */
 GtkCellRenderer *
-_gtk_tree_view_column_get_editable_cell (GtkTreeViewColumn *column)
+_gtk_tree_view_column_get_edited_cell (GtkTreeViewColumn *column)
 {
   GList *list;
 
   for (list = column->cell_list; list; list = list->next)
-    if (((GtkTreeViewColumnCellInfo *)list->data)->cell->mode ==
-	GTK_CELL_RENDERER_MODE_EDITABLE)
+    if (((GtkTreeViewColumnCellInfo *)list->data)->in_editing_mode)
       return ((GtkTreeViewColumnCellInfo *)list->data)->cell;
 
   return NULL;
@@ -1086,6 +1104,25 @@ _gtk_tree_view_column_count_special_cells (GtkTreeViewColumn *column)
   return i;
 }
 
+GtkCellRenderer *
+_gtk_tree_view_column_get_cell_at_pos (GtkTreeViewColumn *column,
+				       gint               x)
+{
+  GList *list;
+  gint current_x = 0;
+
+  list = gtk_tree_view_column_cell_first (column);
+  for (; list; list = gtk_tree_view_column_cell_next (column, list))
+   {
+     GtkTreeViewColumnCellInfo *cellinfo = list->data;
+     if (current_x <= x && x <= current_x + cellinfo->real_width)
+       return cellinfo->cell;
+     current_x += cellinfo->real_width;
+   }
+
+  return NULL;
+}
+
 /* Public Functions */
 
 
@@ -1119,18 +1156,18 @@ gtk_tree_view_column_new (void)
  *
  * Here's a simple example:
  * <informalexample><programlisting>
- *  <!>enum { TEXT_COLUMN, COLOR_COLUMN, N_COLUMNS };
- *  <!>...
- *  <!>{
- *  <!>  GtkTreeViewColumn *column;
- *  <!>  GtkCellRenderer   *renderer = gtk_cell_renderer_text_new ();
- *  <!>
- *  <!>  column = gtk_tree_view_column_new_with_attributes ("Title",
- *  <!>                                                     renderer,
- *  <!>                                                     "text", TEXT_COLUMN,
- *  <!>                                                     "foreground", COLOR_COLUMN,
- *  <!>                                                     NULL);
- *  <!>}
+ *  enum { TEXT_COLUMN, COLOR_COLUMN, N_COLUMNS };
+ *  ...
+ *  {
+ *    GtkTreeViewColumn *column;
+ *    GtkCellRenderer   *renderer = gtk_cell_renderer_text_new (<!-- -->);
+ *  
+ *    column = gtk_tree_view_column_new_with_attributes ("Title",
+ *                                                       renderer,
+ *                                                       "text", TEXT_COLUMN,
+ *                                                       "foreground", COLOR_COLUMN,
+ *                                                       NULL);
+ *  }
  * </programlisting></informalexample>
  * 
  * Return value: A newly created #GtkTreeViewColumn.
@@ -1173,8 +1210,9 @@ gtk_tree_view_column_get_cell_info (GtkTreeViewColumn *tree_column,
  * @cell: The #GtkCellRenderer. 
  * @expand: %TRUE if @cell is to be given extra space allocated to box.
  *
- * Packs the @cell into the beginning column.  If @expand is %TRUE, then the
- * @cell is allocated a share of all available space that the @tree_column has.
+ * Packs the @cell into the beginning of the column. If @expand is FALSE, then
+ * the @cell is allocated no more space than it needs. Any unused space is divided
+ * evenly between cells for which @expand is TRUE.
  **/
 void
 gtk_tree_view_column_pack_start (GtkTreeViewColumn *tree_column,
@@ -1206,8 +1244,9 @@ gtk_tree_view_column_pack_start (GtkTreeViewColumn *tree_column,
  * @cell: The #GtkCellRenderer. 
  * @expand: %TRUE if @cell is to be given extra space allocated to box.
  *
- * Packs the @cell into the column.  If @expand is %TRUE, then the @cell is
- * allocated a share of all available space that the @tree_column has.
+ * Adds the @cell to end of the column. If @expand is FALSE, then the @cell
+ * is allocated no more space than it needs. Any unused space is divided
+ * evenly between cells for which @expand is TRUE.
  **/
 void
 gtk_tree_view_column_pack_end (GtkTreeViewColumn  *tree_column,
@@ -1868,9 +1907,10 @@ gtk_tree_view_column_set_title (GtkTreeViewColumn *tree_column,
  * gtk_tree_view_column_get_title:
  * @tree_column: A #GtkTreeViewColumn.
  * 
- * Returns the title of the widget.  This value should not be modified.
+ * Returns the title of the widget.
  * 
- * Return value: the title of the column.
+ * Return value: the title of the column. This string should not be
+ * modified or freed.
  **/
 G_CONST_RETURN gchar *
 gtk_tree_view_column_get_title (GtkTreeViewColumn *tree_column)
@@ -2216,9 +2256,9 @@ gtk_tree_view_column_get_sort_order      (GtkTreeViewColumn     *tree_column)
  * @is_expander: %TRUE, if the row has children
  * @is_expanded: %TRUE, if the row has visible children
  * 
- * Sets the cell renderer based on the @tree_model and @tree_node.  That is, for
+ * Sets the cell renderer based on the @tree_model and @iter.  That is, for
  * every attribute mapping in @tree_column, it will get a value from the set
- * column on the @tree_node, and use that value to set the attribute on the cell
+ * column on the @iter, and use that value to set the attribute on the cell
  * renderer.  This is used primarily by the #GtkTreeView.
  **/
 void
@@ -2268,9 +2308,9 @@ gtk_tree_view_column_cell_set_cell_data (GtkTreeViewColumn *tree_column,
 /**
  * gtk_tree_view_column_cell_get_size:
  * @tree_column: A #GtkTreeViewColumn.
- * @cell_area: The area a the column will be allocated, or %NULL
- * @x_offset: location to return x offset of cell relative to @cell_area, or %NULL
- * @y_offset: location to return y offset of cell relative to @cell_area, or %NULL
+ * @cell_area: The area a cell in the column will be allocated, or %NULL
+ * @x_offset: location to return x offset of a cell relative to @cell_area, or %NULL
+ * @y_offset: location to return y offset of a cell relative to @cell_area, or %NULL
  * @width: location to return width needed to render a cell, or %NULL
  * @height: location to return height needed to render a cell, or %NULL
  * 
@@ -2354,26 +2394,60 @@ gtk_tree_view_column_cell_process_action (GtkTreeViewColumn  *tree_column,
 {
   GList *list;
   GdkRectangle real_cell_area;
+  GdkRectangle real_background_area;
   gint expand_cell_count = 0;
   gint full_requested_width = 0;
   gint extra_space;
   gint min_x, min_y, max_x, max_y;
   gint focus_line_width;
+  gint dx;
+  gint special_cells;
 
   min_x = G_MAXINT;
   min_y = G_MAXINT;
   max_x = 0;
   max_y = 0;
 
-  real_cell_area = *cell_area;
+  special_cells = _gtk_tree_view_column_count_special_cells (tree_column);
+
+  if (special_cells > 1 && action == CELL_ACTION_FOCUS)
+    {
+      GtkTreeViewColumnCellInfo *info = NULL;
+      gboolean found_has_focus = FALSE;
+
+      /* one should have focus */
+      for (list = tree_column->cell_list; list; list = list->next)
+        {
+	  info = list->data;
+	  if (info && info->has_focus)
+	    {
+	      found_has_focus = TRUE;
+	      break;
+	    }
+	}
+
+      if (!found_has_focus)
+        {
+	  /* give the first one focus */
+	  info = gtk_tree_view_column_cell_first (tree_column)->data;
+	  info->has_focus = TRUE;
+	}
+    }
 
   gtk_widget_style_get (GTK_WIDGET (tree_column->tree_view),
 			"focus-line-width", &focus_line_width,
 			NULL);
-  /* Find out how my extra space we have to allocate */
+
+  real_cell_area = *cell_area;
+  real_background_area = *background_area;
+  dx = real_cell_area.x - real_background_area.x - focus_line_width;
+
+  real_cell_area.x += focus_line_width;
+
+  /* Find out how many extra space we have to allocate */
   for (list = tree_column->cell_list; list; list = list->next)
     {
-      GtkTreeViewColumnCellInfo *info = (GtkTreeViewColumnCellInfo *) list->data;
+      GtkTreeViewColumnCellInfo *info = (GtkTreeViewColumnCellInfo *)list->data;
 
       if (! info->cell->visible)
 	continue;
@@ -2383,10 +2457,13 @@ gtk_tree_view_column_cell_process_action (GtkTreeViewColumn  *tree_column,
       full_requested_width += info->requested_width;
     }
 
-  extra_space = cell_area->width - full_requested_width;
+  extra_space = background_area->width - full_requested_width;
   if (extra_space < 0)
     extra_space = 0;
+  else if (extra_space > 0 && expand_cell_count > 0)
+    extra_space /= expand_cell_count;
 
+  /* iterate list for GTK_PACK_START cells */
   for (list = tree_column->cell_list; list; list = list->next)
     {
       GtkTreeViewColumnCellInfo *info = (GtkTreeViewColumnCellInfo *) list->data;
@@ -2397,16 +2474,19 @@ gtk_tree_view_column_cell_process_action (GtkTreeViewColumn  *tree_column,
       if (! info->cell->visible)
 	continue;
 
-      real_cell_area.width = info->requested_width +
+      real_background_area.width = info->requested_width +
 	(info->expand?extra_space:0);
-      info->real_width = real_cell_area.width;
-      real_cell_area.x += focus_line_width;
+      info->real_width = real_background_area.width;
+
+      real_cell_area.width = real_background_area.width;
+      real_cell_area.width -= 2 * focus_line_width;
+
       if (action == CELL_ACTION_RENDER)
 	{
 	  gtk_cell_renderer_render (info->cell,
 				    window,
 				    tree_column->tree_view,
-				    background_area,
+				    &real_background_area,
 				    &real_cell_area,
 				    expose_area,
 				    flags);
@@ -2421,14 +2501,27 @@ gtk_tree_view_column_cell_process_action (GtkTreeViewColumn  *tree_column,
 				      &x_offset, &y_offset,
 				      &width, &height);
 
-	  if (min_x > (real_cell_area.x + x_offset))
-	    min_x = real_cell_area.x + x_offset;
-	  if (max_x < real_cell_area.x + x_offset + width)
-	    max_x = real_cell_area.x + x_offset + width;
-	  if (min_y > (real_cell_area.y + y_offset))
-	    min_y = real_cell_area.y + y_offset;
-	  if (max_y < real_cell_area.y + y_offset + height)
-	    max_y = real_cell_area.y + y_offset + height;
+	  if (special_cells > 1)
+	    {
+	      if (info->has_focus)
+	        {
+		  min_x = real_cell_area.x + x_offset;
+		  max_x = min_x + width;
+		  min_y = real_cell_area.y + y_offset;
+		  max_y = min_y + height;
+		}
+	    }
+	  else
+	    {
+	      if (min_x > (real_cell_area.x + x_offset))
+		min_x = real_cell_area.x + x_offset;
+	      if (max_x < real_cell_area.x + x_offset + width)
+		max_x = real_cell_area.x + x_offset + width;
+	      if (min_y > (real_cell_area.y + y_offset))
+		min_y = real_cell_area.y + y_offset;
+	      if (max_y < real_cell_area.y + y_offset + height)
+		max_y = real_cell_area.y + y_offset + height;
+	    }
 	}
       else if (action == CELL_ACTION_EVENT)
 	{
@@ -2436,7 +2529,7 @@ gtk_tree_view_column_cell_process_action (GtkTreeViewColumn  *tree_column,
 
 	  if (event)
 	    {
-	      if (_gtk_tree_view_column_count_special_cells (tree_column) == 1)
+	      if (special_cells == 1)
 	        {
 		  /* only 1 activatable cell -> whole column can activate */
 		  if (cell_area->x <= ((GdkEventButton *)event)->x &&
@@ -2450,11 +2543,11 @@ gtk_tree_view_column_cell_process_action (GtkTreeViewColumn  *tree_column,
 		   */
 		try_event = TRUE;
 	    }
-	  else /* if (info->has_focus)*/
-	    /* FIXME 73676: allow focusing individual cells (fixed on HEAD) */
-	    {
-	      try_event = TRUE;
-	    }
+	  else if (special_cells > 1 && info->has_focus)
+	    try_event = TRUE;
+	  else if (special_cells == 1)
+	    try_event = TRUE;
+
 	  if (try_event)
 	    {
 	      gboolean visible, mode;
@@ -2488,14 +2581,20 @@ gtk_tree_view_column_cell_process_action (GtkTreeViewColumn  *tree_column,
 		  if (*editable_widget != NULL)
 		    {
 		      g_return_val_if_fail (GTK_IS_CELL_EDITABLE (*editable_widget), FALSE);
+		      info->in_editing_mode = TRUE;
+		      gtk_tree_view_column_focus_cell (tree_column, info->cell);
 		      
 		      return TRUE;
 		    }
 		}
 	    }
 	}
-      real_cell_area.x += (info->requested_width + tree_column->spacing);
+
+      real_cell_area.x += (info->real_width + tree_column->spacing);
+      real_background_area.x += (info->real_width + tree_column->spacing);
     }
+
+  /* iterate list for PACK_END cells */
   for (list = g_list_last (tree_column->cell_list); list; list = list->prev)
     {
       GtkTreeViewColumnCellInfo *info = (GtkTreeViewColumnCellInfo *) list->data;
@@ -2506,15 +2605,19 @@ gtk_tree_view_column_cell_process_action (GtkTreeViewColumn  *tree_column,
       if (! info->cell->visible)
 	continue;
 
-      real_cell_area.width = info->requested_width +
+      real_background_area.width = info->requested_width +
 	(info->expand?extra_space:0);
-      info->real_width = real_cell_area.width;
+      info->real_width = real_background_area.width;
+
+      real_cell_area.width = real_background_area.width;
+      real_cell_area.width -= 2 * focus_line_width;
+
       if (action == CELL_ACTION_RENDER)
 	{
 	  gtk_cell_renderer_render (info->cell,
 				    window,
 				    tree_column->tree_view,
-				    background_area,
+				    &real_background_area,
 				    &real_cell_area,
 				    expose_area,
 				    flags);
@@ -2529,14 +2632,27 @@ gtk_tree_view_column_cell_process_action (GtkTreeViewColumn  *tree_column,
 				      &x_offset, &y_offset,
 				      &width, &height);
 
-	  if (min_x > (real_cell_area.x + x_offset))
-	    min_x = real_cell_area.x + x_offset;
-	  if (max_x < real_cell_area.x + x_offset + width)
-	    max_x = real_cell_area.x + x_offset + width;
-	  if (min_y > (real_cell_area.y + y_offset))
-	    min_y = real_cell_area.y + y_offset;
-	  if (max_y < real_cell_area.y + y_offset + height)
-	    max_y = real_cell_area.y + y_offset + height;
+	  if (special_cells > 1)
+	    {
+	      if (info->has_focus)
+	        {
+		  min_x = real_cell_area.x + x_offset;
+		  max_x = min_x + width;
+		  min_y = real_cell_area.y + y_offset;
+		  max_y = min_y + height;
+		}
+	    }
+	  else
+	    {
+	      if (min_x > (real_cell_area.x + x_offset))
+		min_x = real_cell_area.x + x_offset;
+	      if (max_x < real_cell_area.x + x_offset + width)
+		max_x = real_cell_area.x + x_offset + width;
+	      if (min_y > (real_cell_area.y + y_offset))
+		min_y = real_cell_area.y + y_offset;
+	      if (max_y < real_cell_area.y + y_offset + height)
+		max_y = real_cell_area.y + y_offset + height;
+	    }
 	}
       else if (action == CELL_ACTION_EVENT)
         {
@@ -2544,7 +2660,7 @@ gtk_tree_view_column_cell_process_action (GtkTreeViewColumn  *tree_column,
 
 	  if (event)
 	    {
-	      if (_gtk_tree_view_column_count_special_cells (tree_column) == 1)
+	      if (special_cells == 1)
 	        {
 		  /* only 1 activatable cell -> whole column can activate */
 		  if (cell_area->x <= ((GdkEventButton *)event)->x &&
@@ -2558,11 +2674,11 @@ gtk_tree_view_column_cell_process_action (GtkTreeViewColumn  *tree_column,
 		 */
 		try_event = TRUE;
 	    }
-	  else /* if (info->has_focus) */
-	    /* FIXME 73676: allow focusing individual cells (fixed on HEAD) */
-	    {
-	      try_event = TRUE;
-	    }
+	  else if (special_cells > 1 && info->has_focus)
+	    try_event = TRUE;
+	  else if (special_cells == 1)
+	    try_event = TRUE;
+
 	  if (try_event)
 	    {
 	      gboolean visible, mode;
@@ -2596,15 +2712,20 @@ gtk_tree_view_column_cell_process_action (GtkTreeViewColumn  *tree_column,
 		  if (*editable_widget != NULL)
 		    {
 		      g_return_val_if_fail (GTK_IS_CELL_EDITABLE (*editable_widget), FALSE);
+		      info->in_editing_mode = TRUE;
+		      gtk_tree_view_column_focus_cell (tree_column, info->cell);
 
 		      return TRUE;
 		    }
 		}
 	    }
 	}
-      real_cell_area.x += (info->requested_width + tree_column->spacing);
+
+      real_cell_area.x += (info->real_width + tree_column->spacing);
+      real_background_area.x += (info->real_width + tree_column->spacing);
     }
 
+  /* fill focus_rectangle when required */
   if (action == CELL_ACTION_FOCUS)
     {
       if (min_x >= max_x || min_y >= max_y)
@@ -2683,14 +2804,218 @@ _gtk_tree_view_column_cell_event (GtkTreeViewColumn  *tree_column,
 						   path_string);
 }
 
+/* cell list manipulation */
+static GList *
+gtk_tree_view_column_cell_first (GtkTreeViewColumn *tree_column)
+{
+  GList *list = tree_column->cell_list;
+
+  /* first GTK_PACK_START cell we find */
+  for ( ; list; list = list->next)
+    {
+      GtkTreeViewColumnCellInfo *info = list->data;
+      if (info->pack == GTK_PACK_START)
+	return list;
+    }
+
+  /* hmm, else the *last* GTK_PACK_END cell */
+  list = g_list_last (tree_column->cell_list);
+
+  for ( ; list; list = list->prev)
+    {
+      GtkTreeViewColumnCellInfo *info = list->data;
+      if (info->pack == GTK_PACK_END)
+	return list;
+    }
+
+  return NULL;
+}
+
+static GList *
+gtk_tree_view_column_cell_last (GtkTreeViewColumn *tree_column)
+{
+  GList *list = tree_column->cell_list;
+
+  /* *first* GTK_PACK_END cell we find */
+  for ( ; list ; list = list->next)
+    {
+      GtkTreeViewColumnCellInfo *info = list->data;
+      if (info->pack == GTK_PACK_END)
+	return list;
+    }
+
+  /* hmm, else the last GTK_PACK_START cell */
+  list = g_list_last (tree_column->cell_list);
+
+  for ( ; list; list = list->prev)
+    {
+      GtkTreeViewColumnCellInfo *info = list->data;
+      if (info->pack == GTK_PACK_START)
+	return list;
+    }
+
+  return NULL;
+}
+
+static GList *
+gtk_tree_view_column_cell_next (GtkTreeViewColumn *tree_column,
+				GList             *current)
+{
+  GList *list;
+  GtkTreeViewColumnCellInfo *info = current->data;
+
+  if (info->pack == GTK_PACK_START)
+    {
+      for (list = current->next; list; list = list->next)
+        {
+	  GtkTreeViewColumnCellInfo *inf = list->data;
+	  if (inf->pack == GTK_PACK_START)
+	    return list;
+	}
+
+      /* out of GTK_PACK_START cells, get *last* GTK_PACK_END one */
+      list = g_list_last (tree_column->cell_list);
+      for (; list; list = list->prev)
+        {
+	  GtkTreeViewColumnCellInfo *inf = list->data;
+	  if (inf->pack == GTK_PACK_END)
+	    return list;
+	}
+    }
+
+  for (list = current->prev; list; list = list->prev)
+    {
+      GtkTreeViewColumnCellInfo *inf = list->data;
+      if (inf->pack == GTK_PACK_END)
+	return list;
+    }
+
+  return NULL;
+}
+
+static GList *
+gtk_tree_view_column_cell_prev (GtkTreeViewColumn *tree_column,
+				GList             *current)
+{
+  GList *list;
+  GtkTreeViewColumnCellInfo *info = current->data;
+
+  if (info->pack == GTK_PACK_END)
+    {
+      for (list = current->next; list; list = list->next)
+        {
+	  GtkTreeViewColumnCellInfo *inf = list->data;
+	  if (inf->pack == GTK_PACK_END)
+	    return list;
+	}
+
+      /* out of GTK_PACK_END, get last GTK_PACK_START one */
+      list = g_list_last (tree_column->cell_list);
+      for ( ; list; list = list->prev)
+        {
+	  GtkTreeViewColumnCellInfo *inf = list->data;
+	  if (inf->pack == GTK_PACK_START)
+	    return list;
+	}
+    }
+
+  for (list = current->prev; list; list = list->prev)
+    {
+      GtkTreeViewColumnCellInfo *inf = list->data;
+      if (inf->pack == GTK_PACK_START)
+	return list;
+    }
+
+  return NULL;
+}
 
 gboolean
 _gtk_tree_view_column_cell_focus (GtkTreeViewColumn *tree_column,
-				  gint               direction)
+				  gint               direction,
+				  gboolean           left,
+				  gboolean           right)
 {
-  /* FIXME 73676: allow focusing individual cells */
+  gint count;
+
+  count = _gtk_tree_view_column_count_special_cells (tree_column);
+
+  /* if we are the current focus column and have multiple editable cells,
+   * try to select the next one, else move the focus to the next column
+   */
   if (GTK_TREE_VIEW (tree_column->tree_view)->priv->focus_column == tree_column)
-    return FALSE;
+    {
+      if (count > 1)
+        {
+          GList *next, *prev;
+	  GList *list = tree_column->cell_list;
+	  GtkTreeViewColumnCellInfo *info = NULL;
+
+	  /* find current focussed cell */
+	  for ( ; list; list = list->next)
+	    {
+	      info = list->data;
+	      if (info->has_focus)
+		break;
+	    }
+
+	  /* not a focussed cell in the focus column? */
+	  if (!list || !info || !info->has_focus)
+	    return FALSE;
+
+	  next = gtk_tree_view_column_cell_next (tree_column, list);
+	  prev = gtk_tree_view_column_cell_prev (tree_column, list);
+
+	  info->has_focus = FALSE;
+	  if (direction > 0 && next)
+	    {
+	      info = next->data;
+	      info->has_focus = TRUE;
+	      return TRUE;
+	    }
+	  else if (direction > 0 && !next && !right)
+	    {
+	      /* keep focus on latest cell */
+	      info = gtk_tree_view_column_cell_last (tree_column)->data;
+	      info->has_focus = TRUE;
+	      return TRUE;
+	    }
+	  else if (direction < 0 && prev)
+	    {
+	      info = prev->data;
+	      info->has_focus = TRUE;
+	      return TRUE;
+	    }
+	  else if (direction < 0 && !prev && !left)
+	    {
+	      /* keep focus on first cell */
+	      info = gtk_tree_view_column_cell_first (tree_column)->data;
+	      info->has_focus = TRUE;
+	      return TRUE;
+	    }
+	}
+      return FALSE;
+    }
+
+  /* we get focus, if we have multiple editable cells, give the correct one
+   * focus
+   */
+  if (count > 1)
+    {
+      GList *list = tree_column->cell_list;
+
+      /* clear focus first */
+      for ( ; list ; list = list->next)
+        {
+	  GtkTreeViewColumnCellInfo *info = list->data;
+	  if (info->has_focus)
+	    info->has_focus = FALSE;
+	}
+
+      if (direction > 0)
+	((GtkTreeViewColumnCellInfo *)gtk_tree_view_column_cell_first (tree_column)->data)->has_focus = TRUE;
+      else if (direction < 0)
+	((GtkTreeViewColumnCellInfo *)gtk_tree_view_column_cell_last (tree_column)->data)->has_focus = TRUE;
+    }
   return TRUE;
 }
 
@@ -2782,6 +3107,53 @@ gtk_tree_view_column_cell_is_visible (GtkTreeViewColumn *tree_column)
   return FALSE;
 }
 
+/**
+ * gtk_tree_view_column_focus_cell:
+ * @tree_view: A #GtkTreeView
+ * @cell: A #GtkCellRenderer
+ *
+ * Sets the current keyboard focus to be at @cell, if the column contains
+ * 2 or more editable and activatable cells.
+ **/
+void
+gtk_tree_view_column_focus_cell (GtkTreeViewColumn *tree_column,
+				 GtkCellRenderer   *cell)
+{
+  GList *list;
+  gboolean found_cell = FALSE;
+
+  g_return_if_fail (GTK_IS_TREE_VIEW_COLUMN (tree_column));
+  g_return_if_fail (GTK_IS_CELL_RENDERER (cell));
+
+  if (_gtk_tree_view_column_count_special_cells (tree_column) < 2)
+    return;
+
+  for (list = tree_column->cell_list; list; list = list->next)
+    {
+      GtkTreeViewColumnCellInfo *info = list->data;
+
+      if (info->cell == cell)
+        {
+	  info->has_focus = TRUE;
+	  found_cell = TRUE;
+	  break;
+	}
+    }
+
+  if (found_cell)
+    {
+      for (list = tree_column->cell_list; list; list = list->next)
+        {
+	  GtkTreeViewColumnCellInfo *info = list->data;
+
+	  if (info->cell != cell)
+	    info->has_focus = FALSE;
+        }
+
+      /* FIXME: redraw? */
+    }
+}
+
 void
 _gtk_tree_view_column_cell_set_dirty (GtkTreeViewColumn *tree_column,
 				      gboolean           install_handler)
@@ -2822,9 +3194,13 @@ _gtk_tree_view_column_start_editing (GtkTreeViewColumn *tree_column,
 void
 _gtk_tree_view_column_stop_editing (GtkTreeViewColumn *tree_column)
 {
+  GList *list;
+
   g_return_if_fail (tree_column->editable_widget != NULL);
 
   tree_column->editable_widget = NULL;
+  for (list = tree_column->cell_list; list; list = list->next)
+    ((GtkTreeViewColumnCellInfo *)list->data)->in_editing_mode = FALSE;
 }
 
 void
@@ -2838,8 +3214,9 @@ _gtk_tree_view_column_get_neighbor_sizes (GtkTreeViewColumn *column,
   if (left)
     {
       *left = 0;
+      list = gtk_tree_view_column_cell_first (column);
 
-      for (list = column->cell_list; list; list = list->next)
+      for (; list; list = gtk_tree_view_column_cell_next (column, list))
         {
 	  GtkTreeViewColumnCellInfo *info =
 	    (GtkTreeViewColumnCellInfo *)list->data;
@@ -2853,9 +3230,12 @@ _gtk_tree_view_column_get_neighbor_sizes (GtkTreeViewColumn *column,
 
   if (right)
     {
-      *right = 0;
+      GList *next;
 
-      for (list = column->cell_list; list; list = list->next)
+      *right = 0;
+      list = gtk_tree_view_column_cell_first (column);
+
+      for (; list; list = gtk_tree_view_column_cell_next (column, list))
         {
 	  GtkTreeViewColumnCellInfo *info =
 	    (GtkTreeViewColumnCellInfo *)list->data;
@@ -2865,10 +3245,11 @@ _gtk_tree_view_column_get_neighbor_sizes (GtkTreeViewColumn *column,
 	}
 
       /* skip cell */
-      if (list && list->next)
+      next = gtk_tree_view_column_cell_next (column, list);
+      if (list && next)
         {
-	  list = list->next;
-	  for ( ; list; list = list->next)
+	  list = next;
+	  for ( ; list; list = gtk_tree_view_column_cell_next (column, list))
 	    {
 	      GtkTreeViewColumnCellInfo *info =
 	        (GtkTreeViewColumnCellInfo *)list->data;
